@@ -32,7 +32,7 @@ let with_metrics ~common f =
 
 let run_build_system ~common ~request =
   let run ~(toplevel : unit Memo.Lazy.t) =
-    with_metrics ~common (fun () -> Build_system.run (fun () -> Memo.Lazy.force toplevel))
+    with_metrics ~common (fun () -> build (fun () -> Memo.Lazy.force toplevel))
   in
   let open Fiber.O in
   Fiber.finalize
@@ -127,46 +127,6 @@ let run_build_command ~(common : Common.t) ~config ~request =
     ~request
 ;;
 
-let runtest_info =
-  let doc = "Run tests." in
-  let man =
-    [ `S "DESCRIPTION"
-    ; `P {|This is a short-hand for calling:|}
-    ; `Pre {|  dune build @runtest|}
-    ; `Blocks Common.help_secs
-    ; Common.examples
-        [ ( "Run all tests in the current source tree (including those that passed on \
-             the last run)"
-          , "dune runtest --force" )
-        ; ( "Run tests sequentially without output buffering"
-          , "dune runtest --no-buffer -j 1" )
-        ]
-    ]
-  in
-  Cmd.info "runtest" ~doc ~man ~envs:Common.envs
-;;
-
-let runtest_term =
-  let name_ = Arg.info [] ~docv:"DIR" in
-  let+ builder = Common.Builder.term
-  and+ dirs = Arg.(value & pos_all string [ "." ] name_) in
-  let common, config = Common.init builder in
-  let request (setup : Import.Main.build_system) =
-    Action_builder.all_unit
-      (List.map dirs ~f:(fun dir ->
-         let dir = Path.(relative root) (Common.prefix_target common dir) in
-         Alias.in_dir
-           ~name:Dune_rules.Alias.runtest
-           ~recursive:true
-           ~contexts:setup.contexts
-           dir
-         |> Alias.request))
-  in
-  run_build_command ~common ~config ~request
-;;
-
-let runtest = Cmd.v runtest_info runtest_term
-
 let build =
   let doc = "Build the given targets, or the default ones if none are given." in
   let man =
@@ -200,42 +160,3 @@ let build =
   in
   Cmd.v (Cmd.info "build" ~doc ~man ~envs:Common.envs) term
 ;;
-
-let fmt =
-  let doc = "Format source code." in
-  let man =
-    [ `S "DESCRIPTION"
-    ; `P
-        {|$(b,dune fmt) runs the formatter on the source code. The formatter is
-        automatically selected. ocamlformat is used to format OCaml source code
-        (*.ml and *.mli files) and refmt is used to format Reason source code
-        (*.re and *.rei files).|}
-    ; `Blocks Common.help_secs
-    ]
-  in
-  let term =
-    let+ builder = Common.Builder.term
-    and+ no_promote =
-      Arg.(
-        value
-        & flag
-        & info
-            [ "preview" ]
-            ~doc:
-              "Just print the changes that would be made without actually applying them. \
-               This takes precedence over auto-promote as that flag is assumed for this \
-               command.")
-    in
-    let builder =
-      Common.Builder.set_promote builder (if no_promote then Never else Automatically)
-    in
-    let common, config = Common.init builder in
-    let request (setup : Import.Main.build_system) =
-      let dir = Path.(relative root) (Common.prefix_target common ".") in
-      Alias.in_dir ~name:Dune_rules.Alias.fmt ~recursive:true ~contexts:setup.contexts dir
-      |> Alias.request
-    in
-    run_build_command ~common ~config ~request
-  in
-  Cmd.v (Cmd.info "fmt" ~doc ~man ~envs:Common.envs) term
-;;
