@@ -154,7 +154,7 @@ let libs db (context : Context.t) =
           dir
           exes.buildable.libraries
           exes.buildable.preprocess
-          (List.map exes.names ~f:snd)
+          (List.map (Nonempty_list.to_list exes.names) ~f:snd)
           exes.package
           Item.Kind.Executables
           (exes_extensions ocaml.lib_config exes.modes)
@@ -177,7 +177,7 @@ let libs db (context : Context.t) =
           dir
           tests.exes.buildable.libraries
           tests.exes.buildable.preprocess
-          (List.map tests.exes.names ~f:snd)
+          (List.map (Nonempty_list.to_list tests.exes.names) ~f:snd)
           (if Option.is_none tests.package then tests.exes.package else tests.package)
           Item.Kind.Tests
           (exes_extensions ocaml.lib_config tests.exes.modes)
@@ -213,7 +213,7 @@ let term =
   let* setup = Import.Main.setup () in
   let* setup = Memo.run setup in
   let super_context = Import.Main.find_scontext_exn setup ~name:context_name in
-  Build_system.run_exn
+  build_exn
   @@ fun () ->
   let open Memo.O in
   let context_name =
