@@ -7,7 +7,7 @@ let term =
   let common, config = Common.init builder in
   Scheduler.go ~common ~config
   @@ fun () ->
-  Build_system.run_exn
+  build_exn
   @@ fun () ->
   let open Memo.O in
   let+ project = Source_tree.root () >>| Source_tree.Dir.project in
