@@ -274,12 +274,13 @@ module Sanitize_for_tests = struct
     let fake_workspace = lazy (Path.External.of_string "/WORKSPACE_ROOT")
 
     let sanitize_with_findlib ~findlib_paths path =
+      let path = Path.external_ path in
       List.find_map findlib_paths ~f:(fun candidate ->
         let open Option.O in
         let* candidate = Path.as_external candidate in
         (* if the path to rename is an external path, try to find the
            OCaml root inside, and replace it with a fixed string *)
-        let+ without_prefix = Path.External.drop_prefix ~prefix:candidate path in
+        let+ without_prefix = Path.drop_prefix ~prefix:(Path.external_ candidate) path in
         (* we have found the OCaml root path: let's replace it with a
            constant string *)
         Path.External.append_local (Lazy.force fake_findlib) without_prefix)
@@ -337,7 +338,9 @@ module Crawl = struct
     | { with_deps = false; _ } ->
       Action_builder.return { Ocaml.Ml_kind.Dict.intf = []; impl = [] }
     | { with_deps = true; _ } ->
-      let deps = Dune_rules.Dep_rules.immediate_deps_of unit modules obj_dir in
+      let deps ml_kind =
+        Dune_rules.Dep_rules.immediate_deps_of unit modules ~obj_dir ~ml_kind
+      in
       let open Action_builder.O in
       let+ intf, impl = Action_builder.both (deps Intf) (deps Impl) in
       { Ocaml.Ml_kind.Dict.intf; impl }
@@ -369,7 +372,9 @@ module Crawl = struct
 
   (* Builds the list of modules *)
   let modules ~obj_dir ~deps_of modules_ : Descr.Mod.t list Memo.t =
-    Modules.fold_no_vlib ~init:(Memo.return []) modules_ ~f:(fun m macc ->
+    modules_
+    |> Modules.With_vlib.drop_vlib
+    |> Modules.fold ~init:(Memo.return []) ~f:(fun m macc ->
       let* acc = macc in
       let deps = deps_of m in
       let+ { Ocaml.Ml_kind.Dict.intf = deps_for_intf; impl = deps_for_impl }, _ =
@@ -382,53 +387,63 @@ module Crawl = struct
   let executables sctx ~options ~project ~dir (exes : Executables.t)
     : (Descr.Item.t * Lib.Set.t) option Memo.t
     =
-    let first_exe = snd (List.hd exes.names) in
-    let* modules_, obj_dir =
-      Dir_contents.get sctx ~dir
-      >>= Dir_contents.ocaml
-      >>| Ml_sources.modules_and_obj_dir ~for_:(Exe { first_exe })
-    in
-    let* pp_map =
-      let+ version =
-        let+ ocaml = Super_context.context sctx |> Context.ocaml in
-        ocaml.version
+    let* expander = Super_context.expander sctx ~dir in
+    Expander.eval_blang expander exes.enabled_if
+    >>= function
+    | false -> Memo.return None
+    | true ->
+      let first_exe = snd (Nonempty_list.hd exes.names) in
+      let* scope =
+        Scope.DB.find_by_project (Super_context.context sctx |> Context.name) project
       in
-      Staged.unstage
-      @@ Preprocessing.pped_modules_map
-           (Preprocess.Per_module.without_instrumentation exes.buildable.preprocess)
-           version
-    in
-    let deps_of module_ =
-      let module_ = pp_map module_ in
-      immediate_deps_of_module ~options ~obj_dir ~modules:modules_ module_
-    in
-    let obj_dir = Obj_dir.of_local obj_dir in
-    let* scope =
-      Scope.DB.find_by_project (Super_context.context sctx |> Context.name) project
-    in
-    let* modules_ = modules ~obj_dir ~deps_of modules_ in
-    let+ requires =
-      let* compile_info = Exe_rules.compile_info ~scope exes in
-      let open Resolve.Memo.O in
-      let* requires = Lib.Compile.direct_requires compile_info in
-      if options.with_pps
-      then
-        let+ pps = Lib.Compile.pps compile_info in
-        pps @ requires
-      else Resolve.Memo.return requires
-    in
-    match Resolve.peek requires with
-    | Error () -> None
-    | Ok libs ->
-      let include_dirs = Obj_dir.all_cmis obj_dir in
-      let exe_descr =
-        { Descr.Exe.names = List.map ~f:snd exes.names
-        ; requires = List.map ~f:uid_of_library libs
-        ; modules = modules_
-        ; include_dirs
-        }
+      let* modules_, obj_dir =
+        let+ modules_, obj_dir =
+          Dir_contents.get sctx ~dir
+          >>= Dir_contents.ocaml
+          >>= Ml_sources.modules_and_obj_dir
+                ~libs:(Scope.libs scope)
+                ~for_:(Exe { first_exe })
+        in
+        Modules.With_vlib.modules modules_, obj_dir
       in
-      Some (Descr.Item.Executables exe_descr, Lib.Set.of_list libs)
+      let* pp_map =
+        let+ version =
+          let+ ocaml = Super_context.context sctx |> Context.ocaml in
+          ocaml.version
+        in
+        Staged.unstage
+        @@ Pp_spec.pped_modules_map
+             (Preprocess.Per_module.without_instrumentation exes.buildable.preprocess)
+             version
+      in
+      let deps_of module_ =
+        let module_ = pp_map module_ in
+        immediate_deps_of_module ~options ~obj_dir ~modules:modules_ module_
+      in
+      let obj_dir = Obj_dir.of_local obj_dir in
+      let* modules_ = modules ~obj_dir ~deps_of modules_ in
+      let+ requires =
+        let* compile_info = Exe_rules.compile_info ~scope exes in
+        let open Resolve.Memo.O in
+        let* requires = Lib.Compile.direct_requires compile_info in
+        if options.with_pps
+        then
+          let+ pps = Lib.Compile.pps compile_info in
+          pps @ requires
+        else Resolve.Memo.return requires
+      in
+      (match Resolve.peek requires with
+       | Error () -> None
+       | Ok libs ->
+         let include_dirs = Obj_dir.all_cmis obj_dir in
+         let exe_descr =
+           { Descr.Exe.names = List.map ~f:snd (Nonempty_list.to_list exes.names)
+           ; requires = List.map ~f:uid_of_library libs
+           ; modules = modules_
+           ; include_dirs
+           }
+         in
+         Some (Descr.Item.Executables exe_descr, Lib.Set.of_list libs))
   ;;
 
   (* Builds a workspace item for the provided library object *)
@@ -447,9 +462,17 @@ module Crawl = struct
         | true ->
           (* XXX why do we have a second object directory? *)
           let* modules_, obj_dir_ =
-            Dir_contents.get sctx ~dir:(Path.as_in_build_dir_exn src_dir)
-            >>= Dir_contents.ocaml
-            >>| Ml_sources.modules_and_obj_dir ~for_:(Library name)
+            let* libs =
+              Scope.DB.find_by_dir (Path.as_in_build_dir_exn src_dir) >>| Scope.libs
+            in
+            let+ modules_, obj_dir_ =
+              Dir_contents.get sctx ~dir:(Path.as_in_build_dir_exn src_dir)
+              >>= Dir_contents.ocaml
+              >>= Ml_sources.modules_and_obj_dir
+                    ~libs
+                    ~for_:(Library (Lib_info.lib_id info |> Lib_id.to_local_exn))
+            in
+            Modules.With_vlib.modules modules_, obj_dir_
           in
           let* pp_map =
             let+ version =
@@ -457,7 +480,7 @@ module Crawl = struct
               ocaml.version
             in
             Staged.unstage
-            @@ Preprocessing.pped_modules_map
+            @@ Pp_spec.pped_modules_map
                  (Preprocess.Per_module.without_instrumentation
                     (Lib_info.preprocess info))
                  version
@@ -636,7 +659,7 @@ let term : unit Term.t =
   @@ fun () ->
   let open Fiber.O in
   let* setup = Import.Main.setup () in
-  Build_system.run_exn
+  build_exn
   @@ fun () ->
   let open Memo.O in
   let* setup = setup in
