@@ -11,7 +11,7 @@ let term =
   let* setup = Import.Main.setup () in
   let* setup = Memo.run setup in
   let super_context = Import.Main.find_scontext_exn setup ~name:context_name in
-  Build_system.run_exn
+  build_exn
   @@ fun () ->
   let open Memo.O in
   Dune_rules.Install_rules.stanzas_to_entries super_context
@@ -20,7 +20,7 @@ let term =
 ;;
 
 let command =
-  let doc = "prints information about the entries per package" in
+  let doc = "prints information about the entries per package." in
   let info = Cmd.info ~doc "package-entries" in
   Cmd.v info term
 ;;
