@@ -41,8 +41,8 @@ module File = struct
 
     let executables_conflict (a : Dune_rules.Executables.t) (b : Dune_rules.Executables.t)
       =
-      let a_names = String.Set.of_list_map ~f:snd a.names in
-      let b_names = String.Set.of_list_map ~f:snd b.names in
+      let a_names = String.Set.of_list_map ~f:snd (Nonempty_list.to_list a.names) in
+      let b_names = String.Set.of_list_map ~f:snd (Nonempty_list.to_list b.names) in
       String.Set.inter a_names b_names |> String.Set.is_empty |> not
     ;;
 
@@ -169,14 +169,18 @@ end
 
 (** The context in which the initialization is executed *)
 module Init_context = struct
+  open Dune_config_file
+
   type t =
     { dir : Path.t
     ; project : Dune_project.t
+    ; defaults : Dune_config.Project_defaults.t
     }
 
-  let make path =
+  let make path defaults =
     let open Memo.O in
     let+ project =
+      (* CR-rgrinberg: why not get the project from the source tree? *)
       Dune_project.load
         ~dir:Path.Source.root
         ~files:Filename.Set.empty
@@ -195,7 +199,7 @@ module Init_context = struct
       | Some p -> Path.of_string p
     in
     File.create_dir dir;
-    { dir; project }
+    { dir; project; defaults }
   ;;
 end
 
@@ -372,13 +376,43 @@ module Component = struct
     let test common (() : Options.Test.t) = make "test" common []
 
     (* A list of CSTs for dune-project file content *)
-    let dune_project ~opam_file_gen dir (common : Options.Common.t) =
+    let dune_project
+      ~opam_file_gen
+      ~(defaults : Dune_config_file.Dune_config.Project_defaults.t)
+      dir
+      (common : Options.Common.t)
+      =
       let cst =
         let package =
-          Package.default (Package.Name.of_string (Atom.to_string common.name)) dir
+          Package.create
+            ~name:(Package.Name.of_string (Atom.to_string common.name))
+            ~loc:Loc.none
+            ~version:None
+            ~conflicts:[]
+            ~depopts:[]
+            ~info:Package_info.empty
+            ~sites:Site.Map.empty
+            ~allow_empty:false
+            ~deprecated_package_names:Package.Name.Map.empty
+            ~has_opam_file:(Exists false)
+            ~original_opam_file:None
+            ~dir
+            ~synopsis:(Some "A short synopsis")
+            ~description:(Some "A longer description")
+            ~tags:[ "add topics"; "to describe"; "your"; "project" ]
+            ~depends:
+              [ { Package_dependency.name = Package.Name.of_string "ocaml"
+                ; constraint_ = None
+                }
+              ]
         in
         let packages = Package.Name.Map.singleton (Package.name package) package in
-        let info = Package_info.example in
+        let info =
+          Package_info.example
+            ~authors:defaults.authors
+            ~maintainers:defaults.maintainers
+            ~license:defaults.license
+        in
         Dune_project.anonymous ~dir info packages
         |> Dune_project.set_generate_opam_files opam_file_gen
         |> Dune_project.encode
@@ -390,7 +424,7 @@ module Component = struct
         [ Cst.Comment
             ( Loc.none
             , [ " See the complete stanza docs at \
-                 https://dune.readthedocs.io/en/stable/dune-files.html#dune-project"
+                 https://dune.readthedocs.io/en/stable/reference/dune-project/index.html"
               ] )
         ]
     ;;
@@ -454,6 +488,7 @@ module Component = struct
       let content =
         Stanza_cst.dune_project
           ~opam_file_gen
+          ~defaults:context.defaults
           Path.(as_in_source_tree_exn context.dir)
           common
       in
@@ -512,7 +547,7 @@ module Component = struct
 
     let proj ({ common; options; _ } as opts : Options.Project.t Options.t) =
       let ({ template; pkg; _ } : Options.Project.t) = options in
-      let dir = Path.root in
+      let dir = Path.Source.root in
       let name =
         Package.Name.parse_string_exn (Loc.none, Dune_lang.Atom.to_string common.name)
       in
@@ -520,16 +555,19 @@ module Component = struct
         let package_files =
           match (pkg : Options.Project.Pkg.t) with
           | Opam ->
-            let opam_file = Package.file ~dir ~name in
+            let opam_file = Path.source @@ Package_name.file name ~dir in
             [ File.make_text (Path.parent_exn opam_file) (Path.basename opam_file) "" ]
-          | Esy -> [ File.make_text dir "package.json" "" ]
+          | Esy -> [ File.make_text (Path.source dir) "package.json" "" ]
         in
+        let dir = Path.source dir in
         { dir; files = dune_project_file dir opts :: package_files }
       in
       let component_targets =
-        match (template : Options.Project.Template.t) with
-        | Exec -> proj_exec dir opts
-        | Lib -> proj_lib dir opts
+        (match (template : Options.Project.Template.t) with
+         | Exec -> proj_exec
+         | Lib -> proj_lib)
+          (Path.source dir)
+          opts
       in
       proj_target :: component_targets
     ;;
