@@ -9,8 +9,4 @@ let latest_lang_version =
         |> Dune_lang.Syntax.Version.to_string))
 ;;
 
-let group =
-  Cmd.group
-    (Cmd.info "internal")
-    [ Internal_dump.command; latest_lang_version; Internal_action_runner.group ]
-;;
+let group = Cmd.group (Cmd.info "internal") [ Internal_dump.command; latest_lang_version ]
