@@ -246,7 +246,7 @@ let monitor ~quit_on_disconnect () =
       Console.Status_line.set
         (Console.Status_line.Live
            (fun () -> Pp.verbatim ("Waiting for RPC server" ^ String.make (i mod 4) '.')));
-      let+ () = Scheduler.sleep 0.3 in
+      let+ () = Scheduler.sleep ~seconds:0.3 in
       Some (i + 1))
 ;;
 
@@ -282,8 +282,7 @@ let command =
       Dune_config.for_scheduler
         config
         stats
-        ~insignificant_changes:`Ignore
-        ~signal_watcher:`Yes
+        ~print_ctrl_c_warning:true
         ~watch_exclusions:[]
     in
     Scheduler.Run.go
