@@ -71,13 +71,13 @@ end
 module External : sig
   include Path_intf.S
 
+  val root : t
   val initial_cwd : t
   val cwd : unit -> t
   val relative : t -> string -> t
   val mkdir_p : ?perms:int -> t -> unit
   val of_filename_relative_to_initial_cwd : string -> t
   val append_local : t -> Local.t -> t
-  val drop_prefix : t -> prefix:t -> Local.t option
 
   module Table : Hashtbl.S with type key = t
 end
@@ -195,6 +195,7 @@ module Build : sig
   val chmod : t -> mode:int -> unit
 
   val lstat : t -> Unix.stats
+  val unlink : t -> Fpath.unlink_status
   val unlink_no_err : t -> unit
 
   module Table : Hashtbl.S with type key = t
@@ -228,6 +229,7 @@ module Table : sig
 end
 
 val equal : t -> t -> bool
+val as_outside_build_dir : t -> Outside_build_dir.t option
 val as_outside_build_dir_exn : t -> Outside_build_dir.t
 val destruct_build_dir : t -> [ `Inside of Build.t | `Outside of Outside_build_dir.t ]
 val outside_build_dir : Outside_build_dir.t -> t
@@ -356,7 +358,7 @@ val is_dir_sep : char -> bool
 val is_directory : t -> bool
 
 val rmdir : t -> unit
-val unlink : t -> unit
+val unlink_exn : t -> unit
 val unlink_no_err : t -> unit
 val link : t -> t -> unit
 
@@ -412,7 +414,13 @@ val rename : t -> t -> unit
     you need to modify existing permissions in a non-trivial way. *)
 val chmod : t -> mode:int -> unit
 
-(** Attempts to resolve a symlink. Returns [None] if the path isn't a symlink *)
+(** Attempts to resolve a symlink. Returns:
+
+    - [Ok path] with the resolved destination
+    - [Error Not_a_symlink] if the path isn't a symlink
+    - [Error Max_depth_exceeded] if the function reached the maximum symbolic
+      link depth
+    - [Error (Unix_error _)] with the underlying syscall error. *)
 val follow_symlink : t -> (t, Fpath.follow_symlink_error) result
 
 (** [drop_prefix_exn t ~prefix] drops the [prefix] from a path, including any
