@@ -11,7 +11,7 @@ CAMLprim value stdune_is_darwin(value v_unit) {
 }
 
 CAMLprim value stdune_is_freebsd(value v_unit) {
-   CAMLparam1(v_unit);
+  CAMLparam1(v_unit);
 #if defined(__FreeBSD__)
   CAMLreturn(Val_true);
 #else
@@ -20,7 +20,7 @@ CAMLprim value stdune_is_freebsd(value v_unit) {
 }
 
 CAMLprim value stdune_is_openbsd(value v_unit) {
-   CAMLparam1(v_unit);
+  CAMLparam1(v_unit);
 #if defined(__OpenBSD__)
   CAMLreturn(Val_true);
 #else
@@ -29,10 +29,19 @@ CAMLprim value stdune_is_openbsd(value v_unit) {
 }
 
 CAMLprim value stdune_is_netbsd(value v_unit) {
-   CAMLparam1(v_unit);
+  CAMLparam1(v_unit);
 #if defined(__NetBSD__)
   CAMLreturn(Val_true);
 #else
   CAMLreturn(Val_false);
 #endif
 }
+
+CAMLprim value stdune_is_haiku(value v_unit) {
+  CAMLparam1(v_unit);
+#if defined(__HAIKU__)
+  CAMLreturn(Val_true);
+#else
+  CAMLreturn(Val_false);
+#endif
+}
\ No newline at end of file
