@@ -2,9 +2,18 @@ module Dune_config : sig
   (** Dune configuration (visible to the user) *)
 
   open Stdune
-  open Dune_config
   module Display : module type of Display
 
+  module Project_defaults : sig
+    type t =
+      { authors : string list option
+      ; maintainers : string list option
+      ; license : string list option
+      }
+
+    val decode : t Dune_lang.Decoder.t
+  end
+
   module Concurrency : sig
     type t =
       | Fixed of int
@@ -19,6 +28,21 @@ module Dune_config : sig
   end
 
   module Cache : sig
+    module Toggle : sig
+      type t =
+        | Disabled
+        | Enabled_except_user_rules
+        | Enabled
+
+      val all : (string * t) list
+
+      val decode
+        :  check:(Dune_lang.Syntax.Version.t -> unit Dune_lang.Decoder.t)
+        -> t Dune_lang.Decoder.t
+
+      val to_string : t -> string
+    end
+
     module Storage_mode : sig
       type t = Dune_cache_storage.Mode.t option
 
@@ -51,11 +75,12 @@ module Dune_config : sig
       ; concurrency : Concurrency.t field
       ; terminal_persistence : Terminal_persistence.t field
       ; sandboxing_preference : Sandboxing_preference.t field
-      ; cache_enabled : Config.Toggle.t field
+      ; cache_enabled : Cache.Toggle.t field
       ; cache_reproducibility_check : Dune_cache.Config.Reproducibility_check.t field
       ; cache_storage_mode : Cache.Storage_mode.t field
       ; action_stdout_on_success : Action_output_on_success.t field
       ; action_stderr_on_success : Action_output_on_success.t field
+      ; project_defaults : Project_defaults.t field
       ; experimental : (string * (Loc.t * string)) list field
       }
   end
@@ -102,15 +127,13 @@ module Dune_config : sig
   val hash : t -> int
   val equal : t -> t -> bool
 
-  (** [for_scheduler config ?watch_exclusions stats_opt ~insignificant_changes
-      ~signal_watcher]
+  (** [for_scheduler config ?watch_exclusions stats_opt ~signal_watcher]
       creates a configuration for a scheduler from the user-visible Dune
       [config]. *)
   val for_scheduler
     :  t
     -> watch_exclusions:string list
     -> Dune_stats.t option
-    -> insignificant_changes:[ `React | `Ignore ]
-    -> signal_watcher:[ `Yes | `No ]
+    -> print_ctrl_c_warning:bool
     -> Dune_engine.Scheduler.Config.t
 end
