@@ -245,7 +245,7 @@ end = struct
 
   (* The current version of the rule digest scheme. We should increment it when
      making any changes to the scheme, to avoid collisions. *)
-  let rule_digest_version = 21
+  let rule_digest_version = 22
 
   let compute_rule_digest
     (rule : Rule.t)
@@ -420,11 +420,8 @@ end = struct
               ; action
               }
             in
-            match (Build_config.get ()).action_runner input with
-            | None ->
-              let build_deps deps = Memo.run (build_deps deps) in
-              Action_exec.exec input ~build_deps
-            | Some runner -> Action_runner.exec_action runner input
+            let build_deps deps = Memo.run (build_deps deps) in
+            Action_exec.exec input ~build_deps
           in
           let* action_exec_result = Action_exec.Exec_result.ok_exn action_exec_result in
           let* () =
@@ -622,9 +619,12 @@ end = struct
                   ~can_go_in_shared_cache
                   ~loc
                   ~rule_digest
-                  ~execution_parameters
+                  ~should_remove_write_permissions_on_generated_files:
+                    (Execution_parameters
+                     .should_remove_write_permissions_on_generated_files
+                       execution_parameters)
                   ~produced_targets:exec_result.produced_targets
-                  ~action:action.action
+                  ~action:(fun () -> Action.for_shell action.action |> Action_to_sh.pp)
               in
               let dynamic_deps_stages =
                 List.map
@@ -722,7 +722,7 @@ end = struct
 
   (* The current version of the action digest scheme. We should increment it when
      making any changes to the scheme, to avoid collisions. *)
-  let action_digest_version = 1
+  let action_digest_version = 2
 
   let execute_action_generic
     ~observing_facts
@@ -1103,12 +1103,7 @@ let report_early_exn exn =
     let+ () = State.add_errors errors
     and+ () =
       match !Clflags.stop_on_first_error with
-      | true ->
-        let* () =
-          (Build_config.get ()).action_runners ()
-          |> Fiber.parallel_iter ~f:Action_runner.cancel_build
-        in
-        Scheduler.stop_on_first_error ()
+      | true -> Scheduler.cancel_current_build ()
       | false -> Fiber.return ()
     in
     (match !Clflags.report_errors_config with
@@ -1128,7 +1123,6 @@ let handle_final_exns exns =
 
 let run f =
   let open Fiber.O in
-  Hooks.End_of_build.once Diff_promotion.finalize;
   let* () = State.reset_progress () in
   let* () = State.reset_errors () in
   let f () =
