@@ -27,20 +27,16 @@ module Action_to_sh = Action_to_sh
 module Scheduler = Scheduler
 module Hooks = Hooks
 module Diff_promotion = Diff_promotion
-module Cached_digest = Cached_digest
 module Fs_cache = Fs_cache
 module Fs_memo = Fs_memo
 module Execution_parameters = Execution_parameters
 module Cache_debug_flags = Cache_debug_flags
-module Reversible_digest = Reversible_digest
 module Report_errors_config = Report_errors_config
 module Compound_user_error = Compound_user_error
 module Reflection = Reflection
 module No_io = No_io
 module Rpc = Rpc
 module Dune_rpc_client = Dune_rpc_client
-module Action_runner = Action_runner
 module Action_exec = Action_exec
 module Running_jobs = Running_jobs
 module Rule_cache = Rule_cache
-module Shared_cache_intf = Shared_cache_intf
