@@ -1,5 +1,6 @@
 include Stdune
-module Digest = Dune_digest
+module Digest = Dune_digest.Digest
+module Cached_digest = Dune_digest.Cached_digest
 module Console = Dune_console
 module Metrics = Dune_metrics
 module Log = Dune_util.Log
