@@ -6,10 +6,6 @@ module Timestamp = Event.Timestamp
 module Action_output_on_success = Execution_parameters.Action_output_on_success
 module Action_output_limit = Execution_parameters.Action_output_limit
 
-let with_directory_annot =
-  User_message.Annots.Key.create ~name:"with-directory" Path.to_dyn
-;;
-
 let limit_output = Dune_output_truncation.limit_output ~message:"TRUNCATED BY DUNE"
 
 module Failure_mode = struct
@@ -429,9 +425,9 @@ end = struct
   ;;
 
   let progname_and_purpose ~tag ~prog ~purpose =
-    let open Pp.O in
-    let progname = sprintf "%12s" (Fancy.short_prog_name_of_prog prog) in
-    Pp.tag tag (Pp.verbatim progname) ++ Pp.char ' ' ++ pp_purpose purpose
+    User_message.aligned_message
+      ~left:(tag, Fancy.short_prog_name_of_prog prog)
+      ~right:(pp_purpose purpose)
   ;;
 
   let pp_ok = progname_and_purpose ~tag:Ok
@@ -529,10 +525,9 @@ end = struct
       Has_output { with_color; without_color; has_embedded_location }
   ;;
 
-  let get_loc_and_annots ~dir ~metadata ~output =
+  let get_loc_annots_and_dir ~dir ~metadata ~output =
     let { loc; annots; _ } = metadata in
     let dir = Option.value dir ~default:Path.root in
-    let annots = User_message.Annots.set annots with_directory_annot dir in
     let annots =
       match output with
       | No_output -> annots
@@ -547,14 +542,15 @@ end = struct
           | errors -> User_message.Annots.set annots Compound_user_error.annot errors)
         else annots
     in
-    loc, annots
+    loc, annots, dir
   ;;
 
-  let fail ~loc ~annots paragraphs =
+  let fail ?dir ~loc ~annots paragraphs =
     (* We don't use [User_error.make] as it would add the "Error: " prefix. We
        don't need this prefix as it is already included in the output of the
        command. *)
-    raise (User_error.E (User_message.make ?loc ~annots paragraphs))
+    let dir = Option.map ~f:Path.to_string dir in
+    raise (User_error.E (User_message.make ?dir ?loc ~annots paragraphs))
   ;;
 
   let verbose t ~id ~metadata ~output ~command_line ~dir =
@@ -579,8 +575,9 @@ end = struct
         | Failed n -> sprintf "exited with code %d" n
         | Signaled signame -> sprintf "got signal %s" (Signal.name signame)
       in
-      let loc, annots = get_loc_and_annots ~dir ~metadata ~output in
+      let loc, annots, dir = get_loc_annots_and_dir ~dir ~metadata ~output in
       fail
+        ~dir
         ~loc
         ~annots
         ((Pp.tag User_message.Style.Kwd (Pp.verbatim "Command")
@@ -635,7 +632,7 @@ end = struct
       then Console.print_user_message (User_message.make paragraphs);
       n
     | Error error ->
-      let loc, annots = get_loc_and_annots ~dir ~metadata ~output in
+      let loc, annots, dir = get_loc_annots_and_dir ~dir ~metadata ~output in
       let paragraphs =
         match verbosity with
         | Short ->
@@ -658,7 +655,7 @@ end = struct
                 | Signaled signame ->
                   [ Pp.textf "Command got signal %s." (Signal.name signame) ]))
       in
-      fail ~loc ~annots paragraphs
+      fail ~dir ~loc ~annots paragraphs
   ;;
 end
 
