@@ -41,28 +41,75 @@ type t =
 let dir t = t.dir
 let map_path t p = Path.Build.append t.dir p
 
-let rec copy_recursively (src_kind : Unix.file_kind) ~src ~dst =
-  match src_kind with
-  | S_REG -> Io.copy_file ~src ~dst ()
-  | S_DIR ->
-    (match Path.Untracked.readdir_unsorted_with_kinds src with
-     | Error e -> Unix_error.Detailed.raise e
-     | Ok contents ->
-       Path.mkdir_p dst;
-       List.iter contents ~f:(fun (name, kind) ->
-         copy_recursively kind ~src:(Path.relative src name) ~dst:(Path.relative dst name)))
-  | _ ->
-    Code_error.raise
-      "Can not copy file of this kind"
-      [ "src_kind", File_kind.to_dyn src_kind; "src", Path.to_dyn src ]
+module Item = struct
+  type t =
+    | File
+    | Directory of { perms : int }
+    | Link
+    | Other of Unix.file_kind
+
+  let of_path path =
+    let { Unix.st_kind; st_perm; _ } = Path.Untracked.stat_exn path in
+    match st_kind with
+    | S_DIR -> Directory { perms = st_perm }
+    | S_REG -> File
+    | kind -> Other kind
+  ;;
+
+  let of_kind path (kind : Unix.file_kind) =
+    match kind with
+    | S_DIR -> Directory { perms = (Path.Untracked.stat_exn path).st_perm }
+    | S_REG -> File
+    | S_LNK -> Link
+    | _ -> Other kind
+  ;;
+end
+
+let copy_recursively =
+  let chmod_file = Path.Permissions.add Path.Permissions.write in
+  let chmod_dir p =
+    Path.Permissions.add Path.Permissions.execute p
+    |> Path.Permissions.add Path.Permissions.write
+  in
+  let rec loop item ~src ~dst =
+    match (item : Item.t) with
+    | Link ->
+      (match Path.Untracked.stat_exn src with
+       | { Unix.st_kind = S_REG; _ } -> Io.copy_file ~chmod:chmod_file ~src ~dst ()
+       | { Unix.st_kind = S_DIR; st_perm = perms; _ } ->
+         loop (Directory { perms }) ~src ~dst
+       | { Unix.st_kind; _ } -> loop (Other st_kind) ~src ~dst)
+    | File -> Io.copy_file ~chmod:chmod_file ~src ~dst ()
+    | Directory { perms } ->
+      (match Path.Untracked.readdir_unsorted_with_kinds src with
+       | Error e -> Unix_error.Detailed.raise e
+       | Ok contents ->
+         let perms = chmod_dir perms in
+         Path.mkdir_p ~perms dst;
+         List.iter contents ~f:(fun (name, kind) ->
+           let src = Path.relative src name in
+           let item = Item.of_kind src kind in
+           loop item ~src ~dst:(Path.relative dst name)))
+    | Other kind ->
+      User_error.raise
+        ~hints:
+          [ Pp.text
+              "Re-run Dune to delete the stale artifact, or manually delete this file"
+          ]
+        [ Pp.textf
+            "Failed to copy file %s of kind %S while creating a copy sandbox"
+            (Path.to_string_maybe_quoted src)
+            (File_kind.to_string_hum kind)
+        ]
+  in
+  loop
 ;;
 
-let create_dirs t ~deps ~rule_dir =
-  Path.Build.Set.add (Dep.Facts.necessary_dirs_for_sandboxing deps) rule_dir
-  |> Path.Build.Set.iter ~f:(fun path ->
-    (* There is no point in using the memoized version [Fs.mkdir_p] since
-       these directories are not shared between actions. *)
-    Path.mkdir_p (Path.build (map_path t path)))
+let create_dir t dir = Path.mkdir_p (Path.build (map_path t dir))
+
+let create_dirs t ~dirs ~rule_dir =
+  create_dir t rule_dir;
+  Path.Build.Set.iter dirs ~f:(fun dir -> create_dir t dir)
 ;;
 
 let link_function ~(mode : Sandbox_mode.some) =
@@ -84,8 +131,8 @@ let link_function ~(mode : Sandbox_mode.some) =
         | false -> fun src dst -> Io.portable_symlink ~src ~dst)
      | Copy ->
        fun src dst ->
-         let { Unix.st_kind; _ } = Path.Untracked.stat_exn src in
-         copy_recursively st_kind ~src ~dst
+         let what = Item.of_path src in
+         copy_recursively what ~src ~dst
      | Hardlink ->
        (match Sys.win32 with
         | true -> win32_error mode
@@ -93,13 +140,14 @@ let link_function ~(mode : Sandbox_mode.some) =
      | Patch_back_source_tree ->
        (* We need to let the action modify its dependencies, so we copy
           dependencies and make them writable. *)
+       (* CR-someday: this doesn't work with directory targets *)
        let chmod = Path.Permissions.add Path.Permissions.write in
        fun src dst -> Io.copy_file ~src ~dst ~chmod ())
 ;;
 
 let link_deps t ~mode ~deps =
   let link = Staged.unstage (link_function ~mode) in
-  Path.Map.iteri deps ~f:(fun path (_ : Digest.t) ->
+  Path.Set.iter deps ~f:(fun path ->
     match Path.as_in_build_dir path with
     | None ->
       (* This can actually raise if we try to sandbox the "copy from source
@@ -117,14 +165,14 @@ let snapshot t =
   (* CR-someday jeremiedimino: we do this kind of traversal in other places.
      Might be worth trying to factorise the code. *)
   let rec walk dir acc =
-    match Path.Untracked.readdir_unsorted dir with
+    match Path.Untracked.readdir_unsorted_with_kinds dir with
     | Error (err, func, arg) -> raise (Unix.Unix_error (err, func, arg))
     | Ok files ->
-      List.fold_left files ~init:acc ~f:(fun acc basename ->
+      List.fold_left files ~init:acc ~f:(fun acc (basename, (kind : Unix.file_kind)) ->
         let p = Path.relative dir basename in
-        let stats = Path.Untracked.lstat_exn p in
-        match stats.st_kind with
+        match kind with
         | S_REG ->
+          let stats = Path.Untracked.lstat_exn p in
           Path.Map.add_exn acc p (Cached_digest.Reduced_stats.of_unix_stats stats)
         | S_DIR -> walk p acc
         | _ -> acc)
@@ -132,7 +180,7 @@ let snapshot t =
   walk (Path.build t.dir) Path.Map.empty
 ;;
 
-let create ~mode ~dune_stats ~rule_loc ~deps ~rule_dir ~rule_digest ~expand_aliases =
+let create ~mode ~dune_stats ~rule_loc ~dirs ~deps ~rule_dir ~rule_digest =
   let event =
     Dune_stats.start dune_stats (fun () ->
       let cat = Some [ "create-sandbox" ] in
@@ -150,14 +198,9 @@ let create ~mode ~dune_stats ~rule_loc ~deps ~rule_dir ~rule_digest ~expand_alia
   let+ () =
     maybe_async (fun () ->
       Path.rm_rf (Path.build sandbox_dir);
-      create_dirs t ~deps ~rule_dir;
+      create_dirs t ~dirs ~rule_dir;
       (* CR-someday amokhov: Note that this doesn't link dynamic dependencies, so
          targets produced dynamically will be unavailable. *)
-      let deps =
-        if expand_aliases
-        then Dep.Facts.paths deps
-        else Dep.Facts.paths_without_expanding_aliases deps
-      in
       link_deps t ~mode ~deps)
   in
   Dune_stats.finish event;
@@ -198,53 +241,6 @@ let rename_optional_file ~src ~dst =
      | () -> ())
 ;;
 
-(* Recursively collect regular files from [src] to [dst] and return the set of
-   of files collected. *)
-let collect_dir_recursively ~loc ~src_dir ~dst_dir =
-  let rec loop ~src_dir ~dst_dir =
-    match
-      Dune_filesystem_stubs.read_directory_with_kinds (Path.Build.to_string src_dir)
-    with
-    | Ok files ->
-      List.map files ~f:(fun (file, kind) ->
-        match (kind : File_kind.t) with
-        | S_LNK
-          (* TODO symlinks outside of the sandbox are going to be broken,
-             but users shouldn't be doing this anyway. *)
-        | S_REG -> Appendable_list.singleton (dst_dir, file)
-        | S_DIR ->
-          loop
-            ~src_dir:(Path.Build.relative src_dir file)
-            ~dst_dir:(Path.Build.relative dst_dir file)
-        | _ ->
-          User_error.raise
-            ~loc
-            [ Pp.textf
-                "Rule produced a file with unrecognised kind %S"
-                (File_kind.to_string kind)
-            ])
-      |> Appendable_list.concat
-    | Error (ENOENT, _, _) ->
-      User_error.raise
-        ~loc
-        [ Pp.textf
-            "Rule failed to produce directory %S"
-            (Path.Build.drop_build_context_maybe_sandboxed_exn src_dir
-             |> Path.Source.to_string_maybe_quoted)
-        ]
-    | Error (unix_error, _, _) ->
-      User_error.raise
-        ~loc
-        [ Pp.textf
-            "Rule produced unreadable directory %S"
-            (Path.Build.drop_build_context_maybe_sandboxed_exn src_dir
-             |> Path.Source.to_string_maybe_quoted)
-        ; Pp.verbatim (Unix.error_message unix_error)
-        ]
-  in
-  loop ~src_dir ~dst_dir
-;;
-
 let apply_changes_to_source_tree t ~old_snapshot =
   let new_snapshot = snapshot t in
   (* Same as promotion: make the file writable when copying to the source
@@ -276,32 +272,64 @@ let apply_changes_to_source_tree t ~old_snapshot =
        | Lt | Gt -> copy_file p))
 ;;
 
-let move_targets_to_build_dir t ~loc ~should_be_skipped ~(targets : Targets.Validated.t)
-  : unit Targets.Produced.t Fiber.t
+let hint_delete_dir =
+  [ Pp.text
+      "delete this file manually or check the permissions of the parent directory of \
+       this file"
+  ]
+;;
+
+let move_targets_to_build_dir t ~should_be_skipped ~(targets : Targets.Validated.t)
+  : unit Fiber.t
   =
   maybe_async (fun () ->
     Option.iter t.snapshot ~f:(fun old_snapshot ->
       apply_changes_to_source_tree t ~old_snapshot);
-    Path.Build.Set.iter targets.files ~f:(fun target ->
-      if not (should_be_skipped target)
-      then rename_optional_file ~src:(map_path t target) ~dst:target);
-    let discovered_targets =
-      Path.Build.Set.to_list_map targets.dirs ~f:(fun target ->
+    Targets.Validated.iter
+      targets
+      ~file:(fun target ->
+        if not (should_be_skipped target)
+        then rename_optional_file ~src:(map_path t target) ~dst:target)
+      ~dir:(fun target ->
         let src_dir = map_path t target in
-        let files = collect_dir_recursively ~loc ~src_dir ~dst_dir:target in
-        if Path.Untracked.exists (Path.build target)
-        then
-          (* We clean up all targets (including directory targets) before running an
-             action, so this branch should be unreachable. *)
-          Code_error.raise
-            "Stale directory target in the build directory"
-            [ "dst_dir", Path.Build.to_dyn target ];
-        Path.rename (Path.build src_dir) (Path.build target);
-        files)
-      |> Appendable_list.concat
-      |> Appendable_list.to_list
-    in
-    Targets.Produced.expand_validated_exn targets discovered_targets)
+        (match Path.Untracked.stat (Path.build target) with
+         | Error (Unix.ENOENT, _, _) -> ()
+         | Error e ->
+           User_error.raise
+             ~hints:hint_delete_dir
+             [ Pp.textf "unable to stat %s" (Path.Build.to_string_maybe_quoted target)
+             ; Pp.text "reason:"
+             ; Pp.text (Unix_error.Detailed.to_string_hum e)
+             ]
+         | Ok { Unix.st_kind; _ } ->
+           (* We clean up all targets (including directory targets) before
+              running an action, so this branch should be unreachable unless
+              the rule somehow escaped the sandbox *)
+           User_error.raise
+             ~hints:hint_delete_dir
+             [ Pp.textf
+                 "Target %s of kind %S already exists in the build directory"
+                 (Path.Build.to_string_maybe_quoted target)
+                 (File_kind.to_string_hum st_kind)
+             ]);
+        if Path.Untracked.exists (Path.build src_dir)
+        then Path.rename (Path.build src_dir) (Path.build target)))
 ;;
 
-let destroy t = maybe_async (fun () -> Path.rm_rf (Path.build t.dir))
+let failed_to_delete_sandbox dir reason =
+  User_error.raise
+    [ Pp.textf "failed to delete sandbox in %s" (Path.Build.to_string_maybe_quoted dir)
+    ; Pp.text "Reason:"
+    ; reason
+    ]
+;;
+
+let destroy t =
+  maybe_async (fun () ->
+    try Path.rm_rf (Path.build t.dir) with
+    | Sys_error e -> failed_to_delete_sandbox t.dir (Pp.verbatim e)
+    | Unix.Unix_error (error, syscall, arg) ->
+      failed_to_delete_sandbox
+        t.dir
+        (Unix_error.Detailed.pp (Unix_error.Detailed.create error ~syscall ~arg)))
+;;
