@@ -34,3 +34,11 @@ module Dep_conf = Dep_conf
 module Relop = Relop
 module Cond = Cond
 module Toggle = Toggle
+module Site = Site
+module Warning = Warning
+module Source_kind = Source_kind
+module Package_info = Package_info
+module Section = Section
+module Package = Package
+module Dune_project_name = Dune_project_name
+module Dialect = Dialect
