@@ -1,80 +1,88 @@
 open Stdune
 
-module Op = struct
-  type t =
-    | Eq
-    | Gte
-    | Lte
-    | Gt
-    | Lt
-    | Neq
-
-  let equal a b =
-    match a, b with
-    | Eq, Eq | Gte, Gte | Lte, Lte | Gt, Gt | Lt, Lt | Neq, Neq -> true
-    | _ -> false
-  ;;
-
-  let map = [ "=", Eq; ">=", Gte; "<=", Lte; ">", Gt; "<", Lt; "<>", Neq ]
-  let to_dyn t = Dyn.variant (fst (List.find_exn ~f:(fun (_, op) -> equal t op) map)) []
-
-  let encode x =
-    let f (_, op) = equal x op in
-    (* Assumes the [map] is complete, so exception is impossible *)
-    List.find_exn ~f map |> fst |> Dune_sexp.Encoder.string
-  ;;
-end
-
-module Variable = struct
-  type t = { name : string }
-
-  let of_name name = { name }
-  let to_dyn { name } = Dyn.record [ "name", Dyn.string name ]
-  let encode { name } = Dune_sexp.Encoder.string (":" ^ name)
-
-  let decode =
-    Dune_sexp.Decoder.atom_matching ~desc:"variable" (fun s ->
-      if String.is_prefix s ~prefix:":" then Some (of_name (String.drop s 1)) else None)
-  ;;
-end
-
 module Value = struct
   type t =
     | String_literal of string
-    | Variable of Variable.t
+    | Variable of Package_variable_name.t
 
   let to_dyn = function
     | String_literal s -> Dyn.variant "String_literal" [ Dyn.string s ]
-    | Variable v -> Dyn.variant "Variable" [ Variable.to_dyn v ]
+    | Variable v -> Dyn.variant "Variable" [ Package_variable_name.to_dyn v ]
+  ;;
+
+  let compare a b =
+    match a, b with
+    | String_literal a, String_literal b -> String.compare a b
+    | String_literal _, _ -> Lt
+    | _, String_literal _ -> Gt
+    | Variable a, Variable b -> Package_variable_name.compare a b
   ;;
 
   let encode = function
     | String_literal s -> Dune_sexp.Encoder.string s
-    | Variable v -> Variable.encode v
+    | Variable v -> Package_variable_name.Project.encode v
   ;;
 
   let decode =
     let open Dune_sexp.Decoder in
-    (let+ variable = Variable.decode in
+    (let+ variable = Package_variable_name.Project.decode in
      Variable variable)
     <|> let+ s = string in
         String_literal s
   ;;
 end
 
-type t =
-  | Bvar of Variable.t
-  | Uop of Op.t * Value.t
-  | Bop of Op.t * Value.t * Value.t
-  | And of t list
-  | Or of t list
+module T = struct
+  type t =
+    | Bvar of Package_variable_name.t
+    | Uop of Relop.t * Value.t
+    | Bop of Relop.t * Value.t * Value.t
+    | And of t list
+    | Or of t list
+
+  let rec to_dyn =
+    let open Dyn in
+    function
+    | Bvar v -> variant "Bvar" [ Package_variable_name.to_dyn v ]
+    | Uop (b, x) -> variant "Uop" [ Relop.to_dyn b; Value.to_dyn x ]
+    | Bop (b, x, y) -> variant "Bop" [ Relop.to_dyn b; Value.to_dyn x; Value.to_dyn y ]
+    | And t -> variant "And" (List.map ~f:to_dyn t)
+    | Or t -> variant "Or" (List.map ~f:to_dyn t)
+  ;;
+
+  let rec compare a b =
+    let open Ordering.O in
+    match a, b with
+    | Bvar a, Bvar b -> Package_variable_name.compare a b
+    | Bvar _, _ -> Lt
+    | _, Bvar _ -> Gt
+    | Uop (a_op, a_value), Uop (b_op, b_value) ->
+      let= () = Relop.compare a_op b_op in
+      Value.compare a_value b_value
+    | Uop _, _ -> Lt
+    | _, Uop _ -> Gt
+    | Bop (a_op, a_lhs, a_rhs), Bop (b_op, b_lhs, b_rhs) ->
+      let= () = Relop.compare a_op b_op in
+      let= () = Value.compare a_lhs b_lhs in
+      Value.compare a_rhs b_rhs
+    | Bop _, _ -> Lt
+    | _, Bop _ -> Gt
+    | And a, And b -> List.compare a b ~compare
+    | And _, _ -> Lt
+    | _, And _ -> Gt
+    | Or a, Or b -> List.compare a b ~compare
+  ;;
+end
+
+include T
+include Comparable.Make (T)
 
 let rec encode c =
   let open Dune_sexp.Encoder in
   match c with
-  | Bvar x -> Variable.encode x
-  | Uop (op, x) -> pair Op.encode Value.encode (op, x)
-  | Bop (op, x, y) -> triple Op.encode Value.encode Value.encode (op, x, y)
+  | Bvar x -> Package_variable_name.Project.encode x
+  | Uop (op, x) -> pair Relop.encode Value.encode (op, x)
+  | Bop (op, x, y) -> triple Relop.encode Value.encode Value.encode (op, x, y)
   | And conjuncts -> list sexp (string "and" :: List.map ~f:encode conjuncts)
   | Or disjuncts -> list sexp (string "or" :: List.map ~f:encode disjuncts)
 ;;
@@ -98,7 +106,7 @@ let logical_op t =
 let decode =
   let open Dune_sexp.Decoder in
   let ops =
-    List.map Op.map ~f:(fun (name, op) ->
+    List.map Relop.map ~f:(fun (name, op) ->
       ( name
       , let+ x = Value.decode
         and+ y = maybe Value.decode
@@ -136,16 +144,8 @@ let decode =
     >>= function
     | Atom (_loc, A s) when String.is_prefix s ~prefix:":" ->
       let+ () = junk in
-      Bvar (Variable.of_name (String.drop s 1))
+      Bvar (Package_variable_name.of_string (String.drop s 1))
     | _ -> sum (ops @ logops))
 ;;
 
-let rec to_dyn =
-  let open Dyn in
-  function
-  | Bvar v -> variant "Bvar" [ Variable.to_dyn v ]
-  | Uop (b, x) -> variant "Uop" [ Op.to_dyn b; Value.to_dyn x ]
-  | Bop (b, x, y) -> variant "Bop" [ Op.to_dyn b; Value.to_dyn x; Value.to_dyn y ]
-  | And t -> variant "And" (List.map ~f:to_dyn t)
-  | Or t -> variant "Or" (List.map ~f:to_dyn t)
-;;
+let equal x y = Ordering.is_eq (compare x y)
