@@ -1,25 +1,6 @@
 open Import
 open Fiber.O
 
-module Monad : Opam_0install.S.Monad with type 'a t = 'a Fiber.t = struct
-  type 'a t = 'a Fiber.t
-
-  module O = Fiber.O
-
-  let return a = Fiber.return a
-
-  module Seq = struct
-    let parallel_map f t =
-      Fiber.parallel_map (List.of_seq t) ~f |> Fiber.map ~f:List.to_seq
-    ;;
-  end
-
-  module List = struct
-    let iter f x = Fiber.sequential_iter x ~f
-    let iter2 f x y = Fiber.sequential_iter (List.combine x y) ~f:(fun (x, y) -> f x y)
-  end
-end
-
 let add_self_to_filter_env package env variable =
   match OpamVariable.Full.scope variable with
   | Self | Package _ -> env variable
@@ -32,10 +13,48 @@ let add_self_to_filter_env package env variable =
     else env variable
 ;;
 
+module Priority = struct
+  (* A priority defines a package's position in the list of candidates
+     fed to the solver. Any change to package selection should be reflected in
+     this priority rather than implemented in an ad-hoc manner *)
+  type t =
+    { (* We don't really need this field, since we filter avoid-version
+         packages. If this changes, we still prefer packages
+         with [avoid-version: false] *)
+      avoid : bool
+    ; version : OpamPackage.Version.t
+    }
+
+  let compare_version =
+    let ord x y = OpamPackage.Version.compare x y |> Ordering.of_int in
+    fun (pref : Version_preference.t) x y ->
+      match pref with
+      | Oldest -> ord x y
+      | Newest -> ord y x
+  ;;
+
+  let compare pref t { avoid; version } =
+    Tuple.T2.compare
+      Bool.compare
+      (compare_version pref)
+      (t.avoid, t.version)
+      (avoid, version)
+  ;;
+
+  let make (package : OpamFile.OPAM.t) =
+    let avoid = List.mem package.flags Pkgflag_AvoidVersion ~equal:Poly.equal in
+    let version = OpamFile.OPAM.package package |> OpamPackage.version in
+    { version; avoid }
+  ;;
+end
+
 module Context_for_dune = struct
-  type 'a monad = 'a Monad.t
   type filter = OpamTypes.filter
-  type rejection = Unavailable
+
+  type rejection =
+    | (* TODO proper error messages for packages skipped via avoid-version *)
+      Unavailable
+    | Avoid_version
 
   let local_package_default_version =
     Package_version.to_opam_package_version Lock_dir.Pkg_info.default_version
@@ -72,10 +91,6 @@ module Context_for_dune = struct
     ~stats_updater
     ~constraints
     =
-    let dune_version =
-      let major, minor = Dune_lang.Stanza.latest_version in
-      OpamPackage.Version.of_string @@ sprintf "%d.%d" major minor
-    in
     let candidates_cache = Fiber_cache.create (module Package_name) in
     let constraints =
       List.map constraints ~f:(fun (constraint_ : Package_dependency.t) ->
@@ -97,7 +112,7 @@ module Context_for_dune = struct
     ; local_packages
     ; pinned_packages
     ; solver_env
-    ; dune_version
+    ; dune_version = Dune_dep.version
     ; stats_updater
     ; candidates_cache
     ; available_cache
@@ -105,22 +120,9 @@ module Context_for_dune = struct
     }
   ;;
 
-  let pp_rejection f = function
-    | Unavailable -> Format.pp_print_string f "Availability condition not satisfied"
-  ;;
-
-  let opam_version_compare =
-    let opam_file_compare_by_version =
-      let opam_package_version_compare a b =
-        OpamPackage.Version.compare a b |> Ordering.of_int
-      in
-      fun a b ->
-        opam_package_version_compare (OpamFile.OPAM.version a) (OpamFile.OPAM.version b)
-    in
-    fun t ->
-      match t.version_preference with
-      | Oldest -> opam_file_compare_by_version
-      | Newest -> Ordering.reverse opam_file_compare_by_version
+  let pp_rejection = function
+    | Unavailable -> Pp.paragraph "Availability condition not satisfied"
+    | Avoid_version -> Pp.paragraph "Package is excluded by avoid-version"
   ;;
 
   let eval_to_bool (filter : filter) : (bool, [> `Not_a_bool of string ]) result =
@@ -171,6 +173,8 @@ module Context_for_dune = struct
   let pinned_candidate t resolved_package =
     let version = Resolved_package.package resolved_package |> OpamPackage.version in
     let available =
+      (* We don't respect avoid-version for pinned packages. This is
+         intentional. *)
       [ version, Resolved_package.opam_file resolved_package |> available_or_error t ]
     in
     let resolved = OpamPackage.Version.Map.singleton version resolved_package in
@@ -181,18 +185,18 @@ module Context_for_dune = struct
     let+ resolved = Opam_repo.load_all_versions t.repos name in
     let available =
       OpamPackage.Version.Map.values resolved
-      (* This sort is not strictly necessary. The values returned from the map
-         are already sorted in ascending order. So it would be enough to just
-         reverse this list if we want the highest versions first. We leave the
-         sorting for clarity. *)
-      |> List.sort ~compare:(fun p1 p2 ->
-        opam_version_compare
-          t
-          (Resolved_package.opam_file p1)
-          (Resolved_package.opam_file p2))
-      |> List.map ~f:(fun resolved_package ->
+      |> List.map ~f:(fun p -> p, Priority.make (Resolved_package.opam_file p))
+      (* Note that although the packages are taken from a map,
+         explicitly sorting them is still necessary. This sort applies
+         the configured version preference and also allows the solver to
+         prefer versions without the avoid-version flag set. *)
+      |> List.sort ~compare:(fun (_, x) (_, y) ->
+        Priority.compare t.version_preference x y)
+      |> List.map ~f:(fun (resolved_package, (priority : Priority.t)) ->
         let opam_file = Resolved_package.opam_file resolved_package in
-        let opam_file_result = available_or_error t opam_file in
+        let opam_file_result =
+          if priority.avoid then Error Avoid_version else available_or_error t opam_file
+        in
         OpamFile.OPAM.version opam_file, opam_file_result)
     in
     { available; resolved }
@@ -219,9 +223,10 @@ module Context_for_dune = struct
 
   let user_restrictions : t -> OpamPackage.Name.t -> OpamFormula.version_constraint option
     =
-    let dune = OpamPackage.Name.of_string "dune" in
     fun t pkg ->
-      if OpamPackage.Name.equal dune pkg then Some (`Eq, t.dune_version) else None
+    if Package_name.equal Dune_dep.name (Package_name.of_opam_package_name pkg)
+    then Some (`Eq, t.dune_version)
+    else None
   ;;
 
   let filter_deps t package filtered_formula =
@@ -252,7 +257,7 @@ module Context_for_dune = struct
   ;;
 end
 
-module Solver = Opam_0install.Solver.Make (Monad) (Context_for_dune)
+module Solver = Opam_0install.Solver.Make (Context_for_dune)
 
 let is_valid_global_variable_name = function
   | "root" -> false
@@ -510,6 +515,41 @@ let make_action = function
   | actions -> Some (Action.Progn actions)
 ;;
 
+(* Heuristic to determine whether a package is an ocaml compiler *)
+let opam_file_is_compiler (opam_package : OpamFile.OPAM.t) =
+  (* Identify compiler packages by using the fact that all compiler
+     Packages declare conflicts with other compiler packages. note
+     that relying on the "compiler" flag to identify compiler packages
+     will not work, as compiler options packages (such as
+     ocaml-option-flambda) also have this flag. *)
+  let ocaml_core_compiler = OpamPackage.Name.of_string "ocaml-core-compiler" in
+  List.mem opam_package.conflict_class ocaml_core_compiler ~equal:OpamPackage.Name.equal
+;;
+
+let resolve_depopts ~resolve depopts =
+  let rec collect acc depopts =
+    match (depopts : OpamTypes.filtered_formula) with
+    | Or ((Atom (_, _) as dep), depopts) -> collect (dep :: acc) depopts
+    | Atom (_, _) as dep -> dep :: acc
+    | Empty -> acc
+    | _ ->
+      (* We rely on depopts always being a list of or'ed package names. Opam
+         verifies this for us at parsing time. Packages defined in dune-project
+         files have this restriction for depopts and regular deps *)
+      Code_error.raise "invalid depopts" [ "depopts", Opam_dyn.filtered_formula depopts ]
+  in
+  OpamFormula.ors_to_list depopts
+  |> List.concat_map ~f:(fun x ->
+    collect [] x
+    |> List.rev
+    |> List.concat_map ~f:(fun depopt ->
+      match resolve depopt with
+      | Error _ -> []
+      | Ok { Resolve_opam_formula.post = _; regular } ->
+        (* CR-someday rgrinberg: think about post deps *)
+        regular))
+;;
+
 let opam_package_to_lock_file_pkg
   solver_env
   stats_updater
@@ -526,11 +566,8 @@ let opam_package_to_lock_file_pkg
     (Table.find_exn candidates_cache name).resolved
     |> OpamPackage.Version.Map.find (Package_version.to_opam_package_version version)
   in
-  let opam_file, loc =
-    let opam_file = Resolved_package.opam_file resolved_package in
-    let loc = Resolved_package.loc resolved_package in
-    opam_file, loc
-  in
+  let opam_file = Resolved_package.opam_file resolved_package in
+  let loc = Resolved_package.loc resolved_package in
   let extra_sources =
     OpamFile.OPAM.extra_sources opam_file
     |> List.map ~f:(fun (opam_basename, opam_url) ->
@@ -539,10 +576,9 @@ let opam_package_to_lock_file_pkg
         let checksum =
           match OpamFile.URL.checksum opam_url with
           | [] -> None
-          (* opam discards the later checksums, so we only take the first one *)
           | checksum :: _ -> Some (Loc.none, Checksum.of_opam_hash checksum)
         in
-        Source.Fetch { Source.url; checksum } ))
+        { Source.url; checksum } ))
   in
   let info =
     let url = OpamFile.OPAM.url opam_file in
@@ -554,7 +590,7 @@ let opam_package_to_lock_file_pkg
           |> Option.map ~f:(fun hash -> Loc.none, Checksum.of_opam_hash hash)
         in
         let url = Loc.none, OpamFile.URL.url url in
-        Source.Fetch { url; checksum })
+        { Source.url; checksum })
     in
     let dev =
       Package_name.Set.mem pinned_package_names name
@@ -566,21 +602,29 @@ let opam_package_to_lock_file_pkg
     { Lock_dir.Pkg_info.name; version; dev; source; extra_sources }
   in
   let depends =
-    match
+    let resolve what =
       Resolve_opam_formula.filtered_formula_to_package_names
         ~with_test:false
         (add_self_to_filter_env opam_package (Solver_env.to_env solver_env))
         version_by_package_name
-        opam_file.depends
-    with
-    | Ok dep_package_names ->
-      List.map dep_package_names ~f:(fun package_name -> Loc.none, package_name)
-    | Error (`Formula_could_not_be_satisfied hints) ->
-      Code_error.raise
-        "Dependencies of package can't be satisfied from packages in solution"
-        [ "package", Dyn.string (OpamFile.OPAM.package opam_file |> OpamPackage.to_string)
-        ; "hints", Dyn.list Resolve_opam_formula.Unsatisfied_formula_hint.to_dyn hints
-        ]
+        what
+    in
+    let depends =
+      match resolve opam_file.depends with
+      | Ok { regular; _ } -> regular
+      | Error (`Formula_could_not_be_satisfied hints) ->
+        Code_error.raise
+          "Dependencies of package can't be satisfied from packages in solution"
+          [ "package", Dyn.string (opam_package |> OpamPackage.to_string)
+          ; "hints", Dyn.list Resolve_opam_formula.Unsatisfied_formula_hint.to_dyn hints
+          ]
+    in
+    let depopts =
+      resolve_depopts ~resolve opam_file.depopts
+      |> List.filter ~f:(fun package_name ->
+        not (List.mem depends package_name ~equal:Package_name.equal))
+    in
+    depends @ depopts |> List.map ~f:(fun package_name -> Loc.none, package_name)
   in
   let build_env action =
     let env_update =
@@ -633,6 +677,14 @@ let opam_package_to_lock_file_pkg
       |> Option.map ~f:build_env
       |> Option.map ~f:(fun action -> Lock_dir.Build_command.Action action))
   in
+  let depexts =
+    OpamFile.OPAM.depexts opam_file
+    |> List.concat_map ~f:(fun (sys_pkgs, filter) ->
+      let env = Solver_env.to_env solver_env in
+      if OpamFilter.eval_to_bool ~default:false env filter
+      then OpamSysPkg.Set.to_list_map OpamSysPkg.to_string sys_pkgs
+      else [])
+  in
   let install_command =
     OpamFile.OPAM.install opam_file
     |> opam_commands_to_actions get_solver_var loc opam_package
@@ -642,15 +694,13 @@ let opam_package_to_lock_file_pkg
   let exported_env =
     OpamFile.OPAM.env opam_file |> List.map ~f:opam_env_update_to_env_update
   in
-  let kind =
-    if List.mem (OpamFile.OPAM.flags opam_file) ~equal:Poly.equal Pkgflag_Compiler
-    then `Compiler
-    else `Non_compiler
-  in
-  kind, { Lock_dir.Pkg.build_command; install_command; depends; info; exported_env }
+  let kind = if opam_file_is_compiler opam_file then `Compiler else `Non_compiler in
+  ( kind
+  , { Lock_dir.Pkg.build_command; install_command; depends; depexts; info; exported_env }
+  )
 ;;
 
-let solve_package_list packages context =
+let solve_package_list packages ~context =
   Fiber.collect_errors (fun () ->
     (* [Solver.solve] returns [Error] when it's unable to find a solution to
        the dependencies, but can also raise exceptions, for example if opam
@@ -670,7 +720,7 @@ let solve_package_list packages context =
   | Ok packages -> Fiber.return @@ Ok (Solver.packages_of_result packages)
   | Error (`Diagnostics e) ->
     let+ diagnostics = Solver.diagnostics e in
-    Error (`Diagnostic_message (Pp.text diagnostics))
+    Error (`Diagnostic_message diagnostics)
   | Error (`Exn exn) ->
     (match exn with
      | OpamPp.(Bad_format _ | Bad_format_list _ | Bad_version _) as bad_format ->
@@ -691,6 +741,87 @@ module Solver_result = struct
     }
 end
 
+let reject_unreachable_packages =
+  let reachable deps_of_package ~roots =
+    let seen = ref Package_name.Set.empty in
+    let rec loop = function
+      | [] -> ()
+      | pkg :: rest ->
+        if Package_name.Set.mem !seen pkg
+        then loop rest
+        else (
+          seen := Package_name.Set.add !seen pkg;
+          let deps =
+            match Package_name.Map.find deps_of_package pkg with
+            | None -> []
+            | Some deps -> deps
+          in
+          loop (List.rev_append deps rest))
+    in
+    loop roots;
+    !seen
+  in
+  fun solver_env ~dune_version ~local_packages ~pkgs_by_name ->
+    let roots = Package_name.Map.keys local_packages in
+    let pkgs_by_version =
+      Package_name.Map.merge pkgs_by_name local_packages ~f:(fun name lhs rhs ->
+        match lhs, rhs with
+        | None, None -> assert false
+        | Some _, Some _ ->
+          Code_error.raise
+            "package is both local and returned by solver"
+            [ "name", Package_name.to_dyn name ]
+        | Some (lock_dir_pkg : Lock_dir.Pkg.t), None -> Some lock_dir_pkg.info.version
+        | None, Some _pkg ->
+          let version = Package_version.of_string "dev" in
+          Some version)
+    in
+    let pkgs_by_name =
+      Package_name.Map.merge pkgs_by_name local_packages ~f:(fun name lhs rhs ->
+        match lhs, rhs with
+        | None, None -> assert false
+        | Some _, Some _ ->
+          Code_error.raise
+            "package is both local and returned by solver"
+            [ "name", Package_name.to_dyn name ]
+        | Some (pkg : Lock_dir.Pkg.t), None -> Some (List.map pkg.depends ~f:snd)
+        | None, Some (pkg : Local_package.For_solver.t) ->
+          let formula = Dependency_formula.to_filtered_formula pkg.dependencies in
+          (* Use `dev` because at this point we don't have any version *)
+          let opam_package =
+            OpamPackage.of_string (sprintf "%s.dev" (Package_name.to_string pkg.name))
+          in
+          let env = add_self_to_filter_env opam_package (Solver_env.to_env solver_env) in
+          let resolved =
+            Resolve_opam_formula.filtered_formula_to_package_names
+              env
+              ~with_test:true
+              (Package_name.Map.set pkgs_by_version Dune_dep.name dune_version)
+              formula
+          in
+          let deps =
+            match resolved with
+            | Ok { regular; post = _ (* discard post deps *) } ->
+              (* remove Dune from the formula as we remove it from solutions *)
+              List.filter regular ~f:(fun pkg ->
+                not (Package_name.equal Dune_dep.name pkg))
+            | Error _ ->
+              Code_error.raise
+                "can't find a valid solution for the dependencies"
+                [ "name", Package_name.to_dyn pkg.name ]
+          in
+          let depopts =
+            List.filter_map pkg.depopts ~f:(fun (d : Package_dependency.t) ->
+              Option.some_if
+                (Package_name.Map.mem local_packages d.name
+                 || Package_name.Map.mem pkgs_by_name d.name)
+                d.name)
+          in
+          Some (deps @ depopts))
+    in
+    reachable pkgs_by_name ~roots
+;;
+
 let solve_lock_dir
   solver_env
   version_preference
@@ -712,32 +843,28 @@ let solve_lock_dir
       ~stats_updater
       ~constraints
   in
-  let packages =
-    Package_name.Map.to_list_map local_packages ~f:(fun name _ ->
-      Package_name.to_opam_package_name name)
-  in
-  solve_package_list packages context
+  Package_name.Map.to_list_map local_packages ~f:(fun name _ ->
+    Package_name.to_opam_package_name name)
+  |> solve_package_list ~context
   >>= function
   | Error _ as e -> Fiber.return e
   | Ok solution ->
-    let version_by_package_name =
-      List.map solution ~f:(fun (package : OpamPackage.t) ->
-        ( Package_name.of_opam_package_name (OpamPackage.name package)
-        , Package_version.of_opam_package_version (OpamPackage.version package) ))
-      |> Package_name.Map.of_list_exn
-    in
-    (* don't include local packages in the lock dir *)
+    (* don't include local packages or dune in the lock dir *)
     let opam_packages_to_lock =
-      let is_local_package package =
-        OpamPackage.name package
-        |> Package_name.of_opam_package_name
-        |> Package_name.Map.mem local_packages
-      in
-      List.filter solution ~f:(Fun.negate is_local_package)
+      let is_local_package = Package_name.Map.mem local_packages in
+      List.filter solution ~f:(fun package ->
+        let name = OpamPackage.name package |> Package_name.of_opam_package_name in
+        (not (is_local_package name)) && not (Package_name.equal Dune_dep.name name))
     in
     let* candidates_cache = Fiber_cache.to_table context.candidates_cache in
     let ocaml, pkgs =
       let pkgs =
+        let version_by_package_name =
+          List.map solution ~f:(fun (package : OpamPackage.t) ->
+            ( Package_name.of_opam_package_name (OpamPackage.name package)
+            , Package_version.of_opam_package_version (OpamPackage.version package) ))
+          |> Package_name.Map.of_list_exn
+        in
         List.map opam_packages_to_lock ~f:(fun opam_package ->
           opam_package_to_lock_file_pkg
             solver_env
@@ -776,8 +903,8 @@ let solve_lock_dir
           "Solver selected multiple versions for the same package"
           [ "name", Package_name.to_dyn name ]
       | Ok pkgs_by_name ->
-        let stats = Solver_stats.Updater.snapshot stats_updater in
         let expanded_solver_variable_bindings =
+          let stats = Solver_stats.Updater.snapshot stats_updater in
           Solver_stats.Expanded_variable_bindings.of_variable_set
             stats.expanded_variables
             solver_env
@@ -797,6 +924,17 @@ let solve_lock_dir
                       (Package_name.to_string name)
                       (Package_name.to_string dep_name)
                   ]));
+        let reachable =
+          reject_unreachable_packages
+            solver_env
+            ~dune_version:(Package_version.of_opam_package_version context.dune_version)
+            ~local_packages
+            ~pkgs_by_name
+        in
+        let pkgs_by_name =
+          Package_name.Map.filteri pkgs_by_name ~f:(fun name _ ->
+            Package_name.Set.mem reachable name)
+        in
         Lock_dir.create_latest_version
           pkgs_by_name
           ~local_packages:(Package_name.Map.values local_packages)
@@ -807,12 +945,11 @@ let solve_lock_dir
     let+ files =
       let resolved_packages =
         List.map opam_packages_to_lock ~f:(fun opam_package ->
-          let package_name =
+          let candidates =
             OpamPackage.name opam_package
-            |> OpamPackage.Name.to_string
-            |> Package_name.of_string
+            |> Package_name.of_opam_package_name
+            |> Table.find_exn candidates_cache
           in
-          let candidates = Table.find_exn candidates_cache package_name in
           OpamPackage.Version.Map.find
             (OpamPackage.version opam_package)
             candidates.resolved)
@@ -822,8 +959,7 @@ let solve_lock_dir
         let package_name =
           Resolved_package.package resolved_package
           |> OpamPackage.name
-          |> OpamPackage.Name.to_string
-          |> Package_name.of_string
+          |> Package_name.of_opam_package_name
         in
         package_name, entries)
       >>| List.filter ~f:(fun (_, entries) -> List.is_non_empty entries)
