@@ -15,4 +15,5 @@ val solve_lock_dir
   -> local_packages:Local_package.For_solver.t Package_name.Map.t
   -> pins:Resolved_package.t Package_name.Map.t
   -> constraints:Dune_lang.Package_dependency.t list
-  -> (Solver_result.t, [ `Diagnostic_message of _ Pp.t ]) result Fiber.t
+  -> (Solver_result.t, [ `Diagnostic_message of User_message.Style.t Pp.t ]) result
+       Fiber.t
