@@ -9,25 +9,33 @@ module Package : sig end
 module DB : sig
   type t
 
-  type context =
-    | Workspace
-    | Project of { dir : Path.Source.t }
-
-  val empty : context -> t
+  val empty : t
   val to_dyn : t -> Dyn.t
   val equal : t -> t -> bool
   val hash : t -> int
-  val decode : context -> t Dune_lang.Decoder.fields_parser
+  val decode : dir:Path.Source.t -> t Dune_lang.Decoder.fields_parser
   val encode : t -> Dune_lang.t list
   val combine_exn : t -> t -> t
-  val add_opam_pins : t -> Local_package.t Package_name.Map.t -> t
+  val add_opam_pins : t -> Dune_lang.Package.t Package_name.Map.t -> t
+
+  module Workspace : sig
+    type db := t
+    type t
+
+    val decode : t Dune_lang.Decoder.fields_parser
+    val empty : t
+    val extract : t -> names:(Loc.t * string) list -> db
+    val equal : t -> t -> bool
+    val to_dyn : t -> Dyn.t
+    val hash : t -> int
+  end
 end
 
 module Scan_project : sig
   type t =
     read:(Path.Source.t -> string Fiber.t)
     -> files:Filename.Set.t
-    -> (DB.t * Local_package.t Package_name.Map.t) option Fiber.t
+    -> (DB.t * Dune_lang.Package.t Package_name.Map.t) option Fiber.t
 end
 
 val resolve
