@@ -9,10 +9,12 @@ include
 
       let write t = function
         | None -> close t
-        | Some packets -> (
-          write t packets >>| function
-          | Ok () -> ()
-          | Error `Closed -> raise Dune_util.Report_error.Already_reported)
+        | Some packets ->
+          write t packets
+          >>| (function
+           | Ok () -> ()
+           | Error `Closed -> raise Dune_util.Report_error.Already_reported)
+      ;;
     end)
 
 type chan = Csexp_rpc.Session.t
@@ -29,16 +31,15 @@ module Connection = struct
     | Error exn ->
       Error
         (User_error.make
-           [ Pp.textf "failed to connect to RPC server %s"
-               (Where.to_string where)
+           [ Pp.textf "failed to connect to RPC server %s" (Where.to_string where)
            ; Exn_with_backtrace.pp exn
            ])
+  ;;
 
   let connect_exn where =
     let+ conn = connect where in
-    match conn with
-    | Ok s -> s
-    | Error msg -> raise (User_error.E msg)
+    User_error.ok_exn conn
+  ;;
 end
 
 let client ?handler ~private_menu connection init ~f =
@@ -48,3 +49,4 @@ let client ?handler ~private_menu connection init ~f =
       ~finally:(fun () -> Csexp_rpc.Session.close connection)
   in
   connect_with_menu ?handler ~private_menu connection init ~f
+;;
