@@ -404,8 +404,14 @@ end = struct
                  if Dune_project.dune_version project >= (3, 14)
                  then Artifacts.Original_path
                  else Install_dir
+               and* artifacts = Expander.artifacts env.expander in
+               let hint =
+                 match s with
+                 | "refmt" -> Some "opam install reason"
+                 | "rescript_syntax" -> Some "opam install rescript-syntax"
+                 | _ -> None
                in
-               Artifacts.binary ~loc:(Some loc) ~where (Expander.artifacts env.expander) s)
+               Artifacts.binary ?hint ~loc:(Some loc) ~where artifacts s)
         in
         let prog = Result.map prog ~f:(Expander.map_exe env.expander) in
         let args = Value.L.to_strings ~dir args in
@@ -508,15 +514,15 @@ let rec expand (t : Dune_lang.Action.t) : Action.t Action_expander.t =
     O.Symlink (x, y)
   | Copy_and_add_line_directive (x, y) ->
     A.with_expander (fun expander ->
-      Memo.return
-      @@
-      let context = Expander.context expander in
-      let+ x = E.dep x
-      and+ y = E.target y in
-      Copy_line_directive.action context ~src:x ~dst:y)
+      Expander.context expander
+      |> Context.DB.get
+      |> Memo.map ~f:(fun context ->
+        let+ x = E.dep x
+        and+ y = E.target y in
+        Copy_line_directive.action context ~src:x ~dst:y))
   | System x ->
     let+ x = E.string x in
-    O.System x
+    System.action x
   | Bash x ->
     let+ x = E.string x in
     O.Bash x
@@ -552,7 +558,7 @@ let rec expand (t : Dune_lang.Action.t) : Action.t Action_expander.t =
         let+ p = E.dep file2 in
         Expander0.as_in_build_dir p ~loc:(String_with_vars.loc file2) ~what:"File"
     in
-    O.Diff { optional; file1; file2; mode }
+    Promote.Diff_action.diff ~optional ~mode file1 file2
   | No_infer t -> A.no_infer (expand t)
   | Pipe (outputs, l) ->
     let+ l = A.all (List.map l ~f:expand) in
