@@ -11,6 +11,10 @@ val lint : Name.t
 val private_doc : Name.t
 val doc_new : Name.t
 val check : Name.t
+val ocaml_index : Name.t
 val install : Name.t
+val pkg_install : Name.t
 val runtest : Name.t
 val all : Name.t
+val is_standard : Name.t -> bool
+val register_as_standard : Name.t -> unit
