@@ -5,4 +5,4 @@ open Import
 val dep_on_alias_rec
   :  Alias.Name.t
   -> Path.Build.t
-  -> Action_builder.Alias_status.t Action_builder.t
+  -> Alias_builder.Alias_status.t Action_builder.t
