@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 (* Artifact substitutions works as follow: the substitution is encoded as a
    string of the form:
@@ -94,7 +95,6 @@ module Conf = struct
   ;;
 
   let sign_hook_of_context (context : Context.t) =
-    let open Memo.O in
     let+ config =
       let+ ocaml = Context.ocaml context in
       ocaml.ocaml_config
@@ -116,8 +116,7 @@ module Conf = struct
   ;;
 
   let of_context (context : Context.t) =
-    let open Memo.O in
-    let get_vcs = Vcs_db.nearest_vcs in
+    let get_vcs = Source_tree.nearest_vcs in
     let name = Context.name context in
     let get_location = Install.Paths.get_local_location name in
     let get_config_path = function
@@ -139,8 +138,7 @@ module Conf = struct
   ;;
 
   let of_install ~relocatable ~roots ~(context : Context.t) =
-    let open Memo.O in
-    let get_vcs = Vcs_db.nearest_vcs in
+    let get_vcs = Source_tree.nearest_vcs in
     let hardcoded_ocaml_path =
       match relocatable with
       | Some prefix -> Memo.return @@ Relocatable prefix
@@ -175,10 +173,18 @@ module Conf = struct
     match has_subst with
     | No_substitution -> Fiber.return ()
     | Some_substitution ->
-      Memo.run t.sign_hook
-      |> Fiber.bind ~f:(function
-        | Some hook -> hook file
-        | None -> Fiber.return ())
+      let executable =
+        match Path.Untracked.stat file with
+        | Error _ -> false
+        | Ok { st_perm; _ } -> Path.Permissions.test Path.Permissions.execute st_perm
+      in
+      if executable
+      then
+        Memo.run t.sign_hook
+        |> Fiber.bind ~f:(function
+          | Some hook -> hook file
+          | None -> Fiber.return ())
+      else Fiber.return ()
   ;;
 end
 
@@ -186,7 +192,6 @@ let eval t ~(conf : Conf.t) =
   let relocatable path =
     (* return a relative path to the install directory in case of relocatable
        instead of absolute path *)
-    let open Memo.O in
     conf.hardcoded_ocaml_path
     >>| function
     | Hardcoded _ -> Path.to_absolute_filename path
@@ -197,8 +202,7 @@ let eval t ~(conf : Conf.t) =
     Fiber.return (Array.make n s |> Array.to_list |> String.concat ~sep:"")
   | Vcs_describe p ->
     Memo.run
-      (let open Memo.O in
-       conf.get_vcs p
+      (conf.get_vcs p
        >>= function
        | None -> Memo.return ""
        | Some vcs ->
@@ -207,7 +211,6 @@ let eval t ~(conf : Conf.t) =
   | Location (name, lib_name) ->
     conf.get_location name lib_name |> relocatable |> Memo.run
   | Configpath d ->
-    let open Memo.O in
     (let* dir = conf.get_config_path d in
      match dir with
      | None -> Memo.return None
@@ -215,13 +218,12 @@ let eval t ~(conf : Conf.t) =
     >>| Option.value ~default:""
     |> Memo.run
   | Hardcoded_ocaml_path ->
-    (let open Memo.O in
-     conf.hardcoded_ocaml_path
-     >>| function
-     | Relocatable _ -> "relocatable"
-     | Hardcoded l ->
-       let l = List.map l ~f:Path.to_absolute_filename in
-       "hardcoded\000" ^ String.concat ~sep:"\000" l)
+    conf.hardcoded_ocaml_path
+    >>| (function
+           | Relocatable _ -> "relocatable"
+           | Hardcoded l ->
+             let l = List.map l ~f:Path.to_absolute_filename in
+             "hardcoded\000" ^ String.concat ~sep:"\000" l)
     |> Memo.run
 ;;
 
@@ -655,7 +657,7 @@ let copy_file_non_atomic ~conf ?chmod ~src ~dst () =
 let replace_if_different ~delete_dst_if_it_is_a_directory ~src ~dst =
   let up_to_date =
     match Path.Untracked.stat dst with
-    | Ok { st_kind; _ } when st_kind = S_DIR ->
+    | Ok { st_kind = S_DIR; _ } ->
       (match delete_dst_if_it_is_a_directory with
        | true ->
          Path.rm_rf dst;
@@ -675,15 +677,7 @@ let replace_if_different ~delete_dst_if_it_is_a_directory ~src ~dst =
   if not up_to_date then Path.rename src dst
 ;;
 
-let copy_file
-  ~conf
-  ?(executable = false)
-  ?chmod
-  ?(delete_dst_if_it_is_a_directory = false)
-  ~src
-  ~dst
-  ()
-  =
+let copy_file ~conf ?chmod ?(delete_dst_if_it_is_a_directory = false) ~src ~dst () =
   (* We create a temporary file in the same directory to ensure it's on the same
      partition as [dst] (otherwise, [Path.rename temp_file dst] won't work). The
      prefix ".#" is used because Dune ignores such files and so creating this
@@ -698,11 +692,7 @@ let copy_file
       let open Fiber.O in
       Path.parent dst |> Option.iter ~f:Path.mkdir_p;
       let* has_subst = copy_file_non_atomic ~conf ?chmod ~src ~dst:temp_file () in
-      let+ () =
-        if executable
-        then Conf.run_sign_hook conf ~has_subst temp_file
-        else Fiber.return ()
-      in
+      let+ () = Conf.run_sign_hook conf ~has_subst temp_file in
       replace_if_different ~delete_dst_if_it_is_a_directory ~src:temp_file ~dst)
     ~finally:(fun () ->
       Path.unlink_no_err temp_file;
