@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 type t =
   { loc : Loc.t
@@ -75,7 +76,6 @@ let () =
 ;;
 
 let gen_rules sctx t ~dir ~scope =
-  let open Memo.O in
   let loc = t.loc in
   (* Files checked by cinaps *)
   let* cinapsed_files =
@@ -154,8 +154,7 @@ let gen_rules sctx t ~dir ~scope =
     Pp_spec.pp_module preprocess module_ >>| Modules.With_vlib.singleton_exe
   in
   let dune_version = Scope.project scope |> Dune_project.dune_version in
-  let names = [ t.loc, name ] in
-  let merlin_ident = Merlin_ident.for_exes ~names:(List.map ~f:snd names) in
+  let names = Nonempty_list.[ t.loc, name ] in
   let compile_info =
     Lib.DB.resolve_user_written_deps
       (Scope.libs scope)
@@ -163,7 +162,6 @@ let gen_rules sctx t ~dir ~scope =
       (Lib_dep.Direct (loc, Lib_name.of_string "cinaps.runtime") :: t.libraries)
       ~pps:(Preprocess.Per_module.pps t.preprocess)
       ~dune_version
-      ~merlin_ident
       ~allow_overlaps:false
       ~forbidden_libraries:[]
   in
@@ -181,7 +179,7 @@ let gen_rules sctx t ~dir ~scope =
       ~requires_compile
       ~requires_link
       ~flags:(Ocaml_flags.of_list [ "-w"; "-24" ])
-      ~js_of_ocaml:None
+      ~js_of_ocaml:(Js_of_ocaml.Mode.Pair.make None)
       ~melange_package_name:None
       ~package:None
   in
@@ -225,7 +223,7 @@ let gen_rules sctx t ~dir ~scope =
             [ A.run (Ok cinaps_exe) [ "-diff-cmd"; "-" ]
             ; A.concurrent
               @@ List.map cinapsed_files ~f:(fun fn ->
-                A.diff
+                Promote.Diff_action.diff
                   ~optional:true
                   (Path.build fn)
                   (Path.Build.extend_basename fn ~suffix:".cinaps-corrected"))
