@@ -16,6 +16,9 @@ val capture_outputs : bool ref
 (** Print debug info about artifact substitution *)
 val debug_artifact_substitution : bool ref
 
+(** Print package output when building with package management *)
+val debug_package_logs : bool ref
+
 (** Whether we are ignoring "dune.lock/". *)
 val ignore_lock_dir : bool ref
 
@@ -26,3 +29,5 @@ type on_missing_dune_project_file =
 
 (** Desired behavior when dune project file is absent *)
 val on_missing_dune_project_file : on_missing_dune_project_file ref
+
+val concurrency : int ref
