@@ -9,7 +9,7 @@ type t
 val make
   :  ?excluded_modules:Module_name.t list
   -> obj_dir:Path.Build.t Obj_dir.t
-  -> modules:Modules.t
+  -> modules:Modules.With_vlib.t
   -> top_sorted_modules:Module.t list Action_builder.t
   -> ext_obj:Filename.Extension.t
   -> unit
