@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 (* This module expands either a (library ... (ctypes ...)) rule or an
    (executables ... (ctypes ...)) rule into the generated set of .ml and .c
@@ -224,7 +225,8 @@ let build_c_program
       Lib.DB.resolve (Scope.libs scope) (Loc.none, ctypes) |> Resolve.Memo.read
     in
     let ctypes_include_dirs =
-      Lib_flags.L.include_paths [ lib ] (Ocaml Native) |> Path.Set.to_list
+      Lib_flags.L.include_paths [ lib ] (Ocaml Native) ocaml.lib_config
+      |> Path.Set.to_list
     in
     let include_dirs = ocaml_where :: ctypes_include_dirs in
     Command.Args.S
@@ -295,7 +297,6 @@ let gen_rules ~cctx ~(buildable : Buildable.t) ~loc ~scope ~dir ~sctx =
   let type_description_functor = ctypes.type_description.functor_ in
   let c_types_includer_module = ctypes.generated_types in
   let c_generated_types_module = Ctypes_field.c_generated_types_module ctypes in
-  let open Memo.O in
   let foreign_archives_deps =
     let { Lib_config.ext_lib; ext_dll; _ } =
       (Compilation_context.ocaml cctx).lib_config
@@ -331,7 +332,6 @@ let gen_rules ~cctx ~(buildable : Buildable.t) ~loc ~scope ~dir ~sctx =
       |> Memo.return
     | Pkg_config ->
       let+ () =
-        let open Memo.O in
         let setup query =
           let* res = Pkg_config.gen_rule sctx ~dir ~loc query in
           match res with
