@@ -9,7 +9,7 @@ val for_module
 
 val immediate_deps_of
   :  Module.t
-  -> Modules.t
+  -> Modules.With_vlib.t
   -> obj_dir:Path.Build.t Obj_dir.t
   -> ml_kind:Ml_kind.t
   -> Module.t list Action_builder.t
