@@ -1,13 +1,11 @@
 open Import
 
 type t =
-  { scope : Scope.t
-  ; local_binaries : File_binding.Expanded.t list Memo.Lazy.t
+  { local_binaries : File_binding.Expanded.t list Memo.Lazy.t
   ; external_env : Env.t Memo.Lazy.t
   ; artifacts : Artifacts.t Memo.Lazy.t
   }
 
-let scope t = t.scope
 let local_binaries t = Memo.Lazy.force t.local_binaries
 let external_env t = Memo.Lazy.force t.external_env
 let artifacts t = Memo.Lazy.force t.artifacts
@@ -17,17 +15,16 @@ let expand_str_lazy expander sw =
   | Some s -> Memo.return s
   | None ->
     let open Memo.O in
-    let* expander = Memo.Lazy.force expander in
+    let* expander = expander in
     Expander.No_deps.expand_str expander sw
 ;;
 
 let make
   ~dir
   ~inherit_from
-  ~scope
   ~config_stanza
   ~profile
-  ~expander_for_artifacts
+  ~expander
   ~default_env
   ~default_artifacts
   =
@@ -36,7 +33,7 @@ let make
   let inherited ~field ~root extend =
     Memo.lazy_ (fun () ->
       (match inherit_from with
-       | None -> Memo.return root
+       | None -> root
        | Some t -> Memo.Lazy.force t >>= field)
       >>= extend)
   in
@@ -45,10 +42,7 @@ let make
     Memo.lazy_ (fun () ->
       Memo.parallel_map
         config_binaries
-        ~f:
-          (File_binding.Unexpanded.expand
-             ~dir
-             ~f:(expand_str_lazy expander_for_artifacts)))
+        ~f:(File_binding.Unexpanded.expand ~dir ~f:(expand_str_lazy expander)))
   in
   let external_env =
     inherited ~field:external_env ~root:default_env (fun env ->
@@ -64,8 +58,7 @@ let make
   in
   let artifacts =
     inherited ~field:artifacts ~root:default_artifacts (fun binaries ->
-      let+ local_binaries = Memo.Lazy.force local_binaries in
-      Artifacts.add_binaries binaries ~dir local_binaries)
+      Memo.Lazy.force local_binaries >>| Artifacts.add_binaries binaries ~dir)
   in
-  { scope; external_env; artifacts; local_binaries }
+  { external_env; artifacts; local_binaries }
 ;;
