@@ -3,30 +3,33 @@ open Import
 let encode_lang = function
   | Foreign_language.C -> "c"
   | Cxx -> "cxx"
+;;
 
 let decode_lang =
   let open Foreign_language in
-  Dune_lang.Decoder.enum [ (encode_lang C, C); (encode_lang Cxx, Cxx) ]
+  Dune_lang.Decoder.enum [ encode_lang C, C; encode_lang Cxx, Cxx ]
+;;
 
 let drop_source_extension fn ~dune_version =
   let open Option.O in
   let* obj, ext = String.rsplit2 fn ~on:'.' in
-  let* language, version =
-    String.Map.find Foreign_language.source_extensions ext
-  in
+  let* language, version = String.Map.find Foreign_language.source_extensions ext in
   Option.some_if (dune_version >= version) (obj, language)
+;;
 
 let possible_sources ~language obj ~dune_version =
   String.Map.to_list Foreign_language.source_extensions
   |> List.filter_map ~f:(fun (ext, (lang, version)) ->
-         Option.some_if
-           (Foreign_language.equal lang language && dune_version >= version)
-           (obj ^ "." ^ ext))
+    Option.some_if
+      (Foreign_language.equal lang language && dune_version >= version)
+      (obj ^ "." ^ ext))
+;;
 
 let add_mode_suffix mode s =
   match mode with
   | Mode.Select.All -> s
   | Only mode -> String.concat ~sep:"_" [ s; Mode.to_string mode ]
+;;
 
 module Archive = struct
   module Name = struct
@@ -37,30 +40,32 @@ module Archive = struct
     let path ~dir ~mode archive_name =
       let archive_name = add_mode_suffix mode archive_name in
       Path.Build.relative dir archive_name
+    ;;
 
     let decode =
       Dune_lang.Decoder.plain_string (fun ~loc s ->
-          match s with
-          | "." | ".." ->
-            User_error.raise ~loc
-              [ Pp.textf "%S is not a valid archive name." s ]
-          | fn when String.exists fn ~f:Path.is_dir_sep ->
-            User_error.raise ~loc
-              [ Pp.textf "Path separators are not allowed in archive names." ]
-          | fn -> fn)
+        match s with
+        | "." | ".." ->
+          User_error.raise ~loc [ Pp.textf "%S is not a valid archive name." s ]
+        | fn when String.exists fn ~f:Path.is_dir_sep ->
+          User_error.raise
+            ~loc
+            [ Pp.textf "Path separators are not allowed in archive names." ]
+        | fn -> fn)
+    ;;
 
     let stubs archive_name = archive_name ^ "_stubs"
-
     let lib_file_prefix = "lib"
 
     let lib_file archive_name ~dir ~ext_lib ~mode =
       let archive_name = add_mode_suffix mode archive_name in
-      Path.Build.relative dir
-        (sprintf "%s%s%s" lib_file_prefix archive_name ext_lib)
+      Path.Build.relative dir (sprintf "%s%s%s" lib_file_prefix archive_name ext_lib)
+    ;;
 
     let dll_file archive_name ~dir ~ext_dll ~mode =
       let archive_name = add_mode_suffix mode archive_name in
       Path.Build.relative dir (sprintf "dll%s%s" archive_name ext_dll)
+    ;;
   end
 
   (** Archive directories can appear as part of the [(foreign_archives ...)]
@@ -77,23 +82,24 @@ module Archive = struct
     }
 
   let dir_path ~dir t = Path.Build.relative dir t.dir
-
   let name ~mode t = add_mode_suffix mode t.name
-
   let stubs archive_name = { dir = "."; name = Name.stubs archive_name }
 
   let decode =
     let open Dune_lang.Decoder in
     let+ s = string in
     { dir = Filename.dirname s; name = Filename.basename s }
+  ;;
 
   let lib_file ~archive ~dir ~ext_lib ~mode =
     let dir = dir_path ~dir archive in
     Name.lib_file archive.name ~dir ~ext_lib ~mode
+  ;;
 
   let dll_file ~archive ~dir ~ext_dll ~mode =
     let dir = dir_path ~dir archive in
     Name.dll_file archive.name ~dir ~ext_dll ~mode
+  ;;
 end
 
 module Stubs = struct
@@ -116,15 +122,19 @@ module Stubs = struct
           ]
       in
       parse_dir <|> parse_lib
+    ;;
   end
 
   module Include_dir = struct
     type t = Include_dir_without_include.t Recursive_include.t
 
     let decode =
-      Recursive_include.decode ~base_term:Include_dir_without_include.decode
-        ~include_keyword:"include" ~non_sexp_behaviour:`Parse_as_base_term
+      Recursive_include.decode
+        ~base_term:Include_dir_without_include.decode
+        ~include_keyword:"include"
+        ~non_sexp_behaviour:`Parse_as_base_term
         ~include_allowed_in_versions:(`Since (3, 5))
+    ;;
 
     let expand_include = Recursive_include.expand_include
 
@@ -143,72 +153,69 @@ module Stubs = struct
 
   let make ~loc ~language ~names ~mode ~flags =
     { loc; language; names; mode; flags; include_dirs = []; extra_deps = [] }
+  ;;
 
   let syntax =
     let name = "mode_specific_stubs" in
     let desc = "syntax extension for mode-specific foreign stubs" in
-    Dune_lang.Syntax.create ~name ~desc [ ((0, 1), `Since (3, 5)) ]
+    Dune_lang.Syntax.create ~name ~desc [ (0, 1), `Since (3, 5) ]
+  ;;
 
-  let () =
-    Dune_project.Extension.register_simple syntax (Dune_lang.Decoder.return [])
+  let () = Dune_project.Extension.register_simple syntax (Dune_lang.Decoder.return [])
 
   let decode_stubs ~for_library =
     let open Dune_lang.Decoder in
     let* loc = loc in
-    let+ loc_archive_name, archive_name =
-      located (field_o "archive_name" string)
+    let+ loc_archive_name, archive_name = located (field_o "archive_name" string)
     and+ language = field "language" decode_lang
     and+ names = Ordered_set_lang.field "names"
     and+ loc_mode, mode =
-      located
-        (field_o "mode" (Dune_lang.Syntax.since syntax (0, 1) >>> Mode.decode))
+      located (field_o "mode" (Dune_lang.Syntax.since syntax (0, 1) >>> Mode.decode))
     and+ flags = Ordered_set_lang.Unexpanded.field "flags"
-    and+ include_dirs =
-      field ~default:[] "include_dirs" (repeat Include_dir.decode)
+    and+ include_dirs = field ~default:[] "include_dirs" (repeat Include_dir.decode)
     and+ extra_deps = field_o "extra_deps" (repeat Dep_conf.decode) in
     let extra_deps = Option.value ~default:[] extra_deps in
     let () =
       match archive_name with
       | None -> ()
       | Some _ ->
-        User_error.raise ~loc:loc_archive_name
+        User_error.raise
+          ~loc:loc_archive_name
           [ Pp.textf
-              "The field \"archive_name\" is not allowed in the (foreign_stubs \
-               ...) stanza. For named foreign archives use the \
-               (foreign_library ...) stanza."
+              "The field \"archive_name\" is not allowed in the (foreign_stubs ...) \
+               stanza. For named foreign archives use the (foreign_library ...) stanza."
           ]
     in
     let () =
       match mode with
       | Some _ when for_library ->
-        User_error.raise ~loc:loc_mode
-          [ Pp.textf "The field \"mode\" is not available for foreign libraries"
-          ]
+        User_error.raise
+          ~loc:loc_mode
+          [ Pp.textf "The field \"mode\" is not available for foreign libraries" ]
       | _ -> ()
     in
     let mode = Mode.Select.of_option mode in
     { loc; language; names; mode; flags; include_dirs; extra_deps }
+  ;;
 
   let decode = Dune_lang.Decoder.fields @@ decode_stubs ~for_library:false
-
   let is_mode_dependent t = Mode.Select.is_not_all t.mode
 end
 
 module Objects = struct
   module Object_name = struct
     let decode = Dune_lang.Decoder.string
-
     let filename t ~ext_obj = t ^ ext_obj
 
     let build_path t ~error_loc ~ext_obj ~dir =
       Path.Build.relative ~error_loc dir (filename t ~ext_obj)
+    ;;
   end
 
   (* Associate each object name with its location in the config *)
   type t = (Loc.t * string) list
 
   let empty = []
-
   let is_empty = List.is_empty
 
   let decode =
@@ -222,18 +229,23 @@ module Objects = struct
       let annots =
         let main = User_message.make ~loc [ Pp.text main_message ] in
         let related = [ User_message.make ~loc:loc' [ Pp.text "" ] ] in
-        User_message.Annots.singleton Compound_user_error.annot
+        User_message.Annots.singleton
+          Compound_user_error.annot
           [ Compound_user_error.make ~main ~related ]
       in
-      User_error.raise ~loc ~annots
+      User_error.raise
+        ~loc
+        ~annots
         [ Pp.textf "%s Already appears at:" main_message
         ; Pp.textf "- %s" (Loc.to_file_colon_line loc')
         ]
+  ;;
 
   let build_paths t ~ext_obj ~dir =
     (* Foreign objects are not mode-dependent *)
     List.map t ~f:(fun (loc, name) ->
-        Object_name.build_path name ~error_loc:loc ~ext_obj ~dir |> Path.build)
+      Object_name.build_path name ~error_loc:loc ~ext_obj ~dir |> Path.build)
+  ;;
 end
 
 module Library = struct
@@ -241,6 +253,7 @@ module Library = struct
     { archive_name : Archive.Name.t
     ; archive_name_loc : Loc.t
     ; stubs : Stubs.t
+    ; enabled_if : Blang.t
     }
 
   let decode =
@@ -248,8 +261,16 @@ module Library = struct
     fields
       (let+ archive_name_loc, archive_name =
          located (field "archive_name" Archive.Name.decode)
-       and+ stubs = Stubs.decode_stubs ~for_library:true in
-       { archive_name; archive_name_loc; stubs })
+       and+ stubs = Stubs.decode_stubs ~for_library:true
+       and+ enabled_if = Enabled_if.decode ~allowed_vars:Any ~since:(Some (3, 14)) () in
+       { archive_name; archive_name_loc; stubs; enabled_if })
+  ;;
+
+  include Stanza.Make (struct
+      type nonrec t = t
+
+      include Poly
+    end)
 end
 
 module Source = struct
@@ -268,6 +289,7 @@ module Source = struct
     match t.kind with
     | Stubs stubs -> stubs.language
     | Ctypes _ -> C
+  ;;
 
   let path t = t.path
 
@@ -275,12 +297,13 @@ module Source = struct
     match t.kind with
     | Stubs s -> s.mode
     | Ctypes _ -> All
+  ;;
 
   let user_object_name t =
     t.path |> Path.Build.split_extension |> fst |> Path.Build.basename
+  ;;
 
   let object_name t = user_object_name t |> add_mode_suffix (mode t)
-
   let make kind ~path = { kind; path }
 end
 
@@ -288,33 +311,33 @@ module Sources = struct
   type t = (Loc.t * Source.t) String.Map.t
 
   let object_files t ~dir ~ext_obj =
-    String.Map.to_list_map t ~f:(fun c _ ->
-        Path.Build.relative dir (c ^ ext_obj))
+    String.Map.to_list_map t ~f:(fun c _ -> Path.Build.relative dir (c ^ ext_obj))
+  ;;
 
   let has_cxx_sources (t : t) =
     String.Map.exists t ~f:(fun (_loc, source) ->
-        let language = Source.language source in
-        Foreign_language.(equal Cxx language))
+      let language = Source.language source in
+      Foreign_language.(equal Cxx language))
+  ;;
 
   module Unresolved = struct
     type t = (Foreign_language.t * Path.Build.t) String.Map.Multi.t
 
     let to_dyn t =
-      String.Map.to_dyn
-        (fun xs ->
-          Dyn.List
-            (List.map xs ~f:(fun (language, path) ->
-                 Dyn.Tuple
-                   [ Foreign_language.to_dyn language; Path.Build.to_dyn path ])))
-        t
+      let entry_to_dyn (language, path) =
+        Dyn.Tuple [ Foreign_language.to_dyn language; Path.Build.to_dyn path ]
+      in
+      String.Map.to_dyn (Dyn.list entry_to_dyn) t
+    ;;
 
     let load ~dune_version ~dir ~files =
       let init = String.Map.empty in
       String.Set.fold files ~init ~f:(fun fn acc ->
-          match drop_source_extension fn ~dune_version with
-          | None -> acc
-          | Some (obj, language) ->
-            let path = Path.Build.relative dir fn in
-            String.Map.add_multi acc obj (language, path))
+        match drop_source_extension fn ~dune_version with
+        | None -> acc
+        | Some (obj, language) ->
+          let path = Path.Build.relative dir fn in
+          String.Map.add_multi acc obj (language, path))
+    ;;
   end
 end
