@@ -28,7 +28,11 @@ let default_context_flags (ctx : Build_context.t) ocaml_config ~project =
       in
       let cxx =
         let+ fdiagnostics_color = fdiagnostics_color
-        and+ db_flags = Cxx_flags.get_flags ~for_:Compile ctx in
+        and+ db_flags =
+          Cxx_flags.get_flags
+            ~for_:(Compile (Ocaml.Version.make (Ocaml_config.version ocaml_config)))
+            ctx
+        in
         List.concat [ db_flags; cxxflags; fdiagnostics_color ]
       in
       c, cxx
@@ -45,7 +49,6 @@ let foreign_flags_env =
         let+ ocaml = Context.ocaml context in
         default_context_flags (Context.build_context context) ocaml.ocaml_config ~project)
       ~f:(fun ~parent expander (env : Dune_env.config) ->
-        let open Memo.O in
         let+ parent = parent in
         let foreign_flags lang =
           Foreign_language.Dict.get env.foreign_flags lang
@@ -164,6 +167,7 @@ let include_dir_flags ~expander ~dir ~include_dirs =
                      Source_tree_map_reduce.map_reduce
                        dir
                        ~traverse:Source_dir_status.Set.all
+                       ~trace_event_name:"Foreign rules"
                        ~f:(fun t ->
                          let deps =
                            let dir =
