@@ -106,19 +106,19 @@ end = struct
      being handled directly in the stanza. *)
   let of_stanza stanza ~sctx ~src_dir ~ctx_dir ~scope ~dir_contents ~expander =
     let dir = ctx_dir in
-    let toplevel_setup = Toplevel.Stanza.setup in
     match Stanza.repr stanza with
     | Toplevel_stanza.T toplevel ->
-      let+ () = toplevel_setup ~sctx ~dir ~toplevel in
+      let+ () = Toplevel.Stanza.setup ~sctx ~dir ~toplevel in
       empty_none
     | Library.T lib ->
-      let* enabled_if = Lib.DB.available (Scope.libs scope) (Library.best_name lib) in
+      let* enabled_if =
+        Lib.DB.available_by_lib_id
+          (Scope.libs scope)
+          (Local (Library.to_lib_id ~src_dir lib))
+      in
       if_available_buildable
         ~loc:lib.buildable.loc
-        (fun () ->
-          let+ () = Odoc.setup_private_library_doc_alias sctx ~scope ~dir:ctx_dir lib
-          and+ rules = Lib_rules.rules lib ~sctx ~dir ~scope ~dir_contents ~expander in
-          rules)
+        (fun () -> Lib_rules.rules lib ~sctx ~dir ~scope ~dir_contents ~expander)
         enabled_if
     | Foreign.Library.T lib ->
       Expander.eval_blang expander lib.enabled_if
@@ -136,14 +136,15 @@ end = struct
         { (with_cctx_merlin ~loc:exes.buildable.loc cctx_merlin) with
           js =
             Some
-              (List.map exes.names ~f:(fun (_, exe) ->
-                 Path.Build.relative dir (exe ^ Js_of_ocaml.Ext.exe)))
+              (List.concat_map (Nonempty_list.to_list exes.names) ~f:(fun (_, exe) ->
+                 List.map Js_of_ocaml.Mode.all ~f:(fun mode ->
+                   Path.Build.relative dir (exe ^ Js_of_ocaml.Ext.exe ~mode))))
         })
     | Alias_conf.T alias ->
       let+ () = Simple_rules.alias sctx alias ~dir ~expander in
       empty_none
     | Tests.T tests ->
-      Expander.eval_blang expander tests.build_if
+      Expander.eval_blang expander tests.exes.enabled_if
       >>= if_available_buildable ~loc:tests.exes.buildable.loc (fun () ->
         Test_rules.rules tests ~sctx ~dir ~scope ~expander ~dir_contents)
     | Copy_files.T { files = glob; _ } ->
@@ -211,9 +212,9 @@ let define_all_alias ~dir ~project ~js_targets =
 
 let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_dir =
   let src_dir = Dune_file.dir dune_file in
-  let* stanzas = Dune_file.stanzas dune_file in
+  let* stanzas = Dune_file.stanzas dune_file
+  and* scope = Scope.DB.find_by_dir ctx_dir in
   let* { For_stanza.merlin = merlins; cctx = cctxs; js = js_targets; source_dirs } =
-    let* scope = Scope.DB.find_by_dir ctx_dir in
     For_stanza.of_stanzas
       stanzas
       ~cctxs
@@ -240,26 +241,27 @@ let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_di
          | false -> Memo.return ()
          | true ->
            let* ml_sources = Dir_contents.ocaml dir_contents in
-           (match
-              let base_path =
-                match Ml_sources.include_subdirs ml_sources with
-                | Include Unqualified | No -> []
-                | Include Qualified ->
-                  Path.Local.descendant
-                    (Path.Build.local ctx_dir)
-                    ~of_:(Path.Build.local (Dir_contents.dir dir_contents))
-                  |> Option.value_exn
-                  |> Path.Local.explode
-                  |> List.map ~f:Module_name.of_string
-              in
-              Menhir_rules.module_names m
-              |> List.find_map ~f:(fun name ->
-                let open Option.O in
-                let path = base_path @ [ name ] in
-                let* origin = Ml_sources.find_origin ml_sources path in
-                List.find_map cctxs ~f:(fun (loc, cctx) ->
-                  Option.some_if (Loc.equal loc (Ml_sources.Origin.loc origin)) cctx))
-            with
+           let base_path =
+             match Ml_sources.include_subdirs ml_sources with
+             | Include Unqualified | No -> []
+             | Include Qualified ->
+               Path.Local.descendant
+                 (Path.Build.local ctx_dir)
+                 ~of_:(Path.Build.local (Dir_contents.dir dir_contents))
+               |> Option.value_exn
+               |> Path.Local.explode
+               |> List.map ~f:Module_name.of_string
+           in
+           Menhir_rules.module_names m
+           |> Memo.List.find_map ~f:(fun name ->
+             let path = base_path @ [ name ] in
+             Ml_sources.find_origin ml_sources ~libs:(Scope.libs scope) path
+             >>| function
+             | None -> None
+             | Some origin ->
+               List.find_map cctxs ~f:(fun (loc, cctx) ->
+                 Option.some_if (Loc.equal loc (Ml_sources.Origin.loc origin)) cctx))
+           >>= (function
             | Some cctx -> Menhir_rules.gen_rules cctx m ~dir:ctx_dir
             | None ->
               (* This happens often when passing a [-p ...] option that hides a
@@ -297,20 +299,16 @@ let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_di
   cctxs
 ;;
 
-let gen_format_and_cram_rules sctx ~expander ~dir source_dir =
+let gen_format_and_cram_rules sctx ~dir source_dir =
   let+ () = Format_rules.setup_alias ~dir
-  and+ () = Cram_rules.rules source_dir ~sctx ~expander ~dir in
+  and+ () = Cram_rules.rules source_dir ~sctx ~dir in
   ()
 ;;
 
 let gen_rules_source_only sctx ~dir source_dir =
   Rules.collect_unit (fun () ->
     let* sctx = sctx in
-    let* expander =
-      let+ expander = Super_context.expander sctx ~dir in
-      Dir_contents.add_sources_to_expander sctx expander
-    in
-    let+ () = gen_format_and_cram_rules sctx ~expander ~dir source_dir
+    let+ () = gen_format_and_cram_rules sctx ~dir source_dir
     and+ () =
       define_all_alias ~dir ~js_targets:[] ~project:(Source_tree.Dir.project source_dir)
     in
@@ -320,11 +318,8 @@ let gen_rules_source_only sctx ~dir source_dir =
 let gen_rules_group_part_or_root sctx dir_contents cctxs ~source_dir ~dir
   : (Loc.t * Compilation_context.t) list Memo.t
   =
-  let* expander =
-    let+ expander = Super_context.expander sctx ~dir in
-    Dir_contents.add_sources_to_expander sctx expander
-  in
-  let* () = gen_format_and_cram_rules sctx ~expander ~dir source_dir
+  let* expander = Super_context.expander sctx ~dir in
+  let* () = gen_format_and_cram_rules sctx ~dir source_dir
   and+ stanzas =
     (* CR-soon rgrinberg: we shouldn't have to fetch the stanzas yet again *)
     Dune_load.stanzas_in_dir dir
@@ -359,6 +354,7 @@ let gen_project_rules =
     let+ () = Install_rules.gen_project_rules sctx project
     and+ () = Odoc.gen_project_rules sctx project
     and+ () = Odoc_new.gen_project_rules sctx project
+    and+ () = Ocaml_index.project_rule sctx project
     and+ () =
       let version = 2, 8 in
       match Dune_project.allow_approximate_merlin project with
@@ -376,9 +372,9 @@ let gen_project_rules =
             version
             ~what:"This field"
     and+ () =
-      match Dune_project.name project with
-      | Named _ -> Memo.return ()
-      | Anonymous _ ->
+      match Dune_project_name.name (Dune_project.name project) with
+      | Some _ -> Memo.return ()
+      | None ->
         (match
            if Dune_project.dune_version project >= (2, 8)
               && Dune_project.generate_opam_files project
@@ -525,7 +521,12 @@ let gen_rules_regular_directory sctx ~src_dir ~components ~dir =
       in
       let+ rules =
         let+ make_rules =
-          let+ directory_targets = Dir_status.directory_targets dir_status ~dir in
+          let+ directory_targets =
+            Dir_status.directory_targets
+              dir_status
+              ~jsoo_enabled:Jsoo_rules.jsoo_enabled
+              ~dir
+          in
           let allowed_subdirs =
             let automatic = Automatic_subdir.subdirs components in
             let toplevel =
@@ -617,7 +618,7 @@ let gen_rules ctx sctx ~dir components : Gen_rules.result Memo.t =
        | _ -> Subdir_set.empty)
       (fun () ->
         let* sctx = sctx in
-        Preprocessing.gen_rules sctx rest)
+        Pp_spec_rules.gen_rules sctx rest)
   | [ ".dune" ] ->
     has_rules
       ~dir
@@ -661,6 +662,35 @@ let private_context ~dir components _ctx =
     Gen_rules.make ~build_dir_only_sub_dirs (Memo.return Rules.empty)
 ;;
 
+let raise_on_lock_dir_out_of_sync =
+  Per_context.create_by_name ~name:"check-lock-dir" (fun ctx ->
+    Memo.lazy_ (fun () ->
+      let* lock_dir_available = Lock_dir.lock_dir_active ctx in
+      if lock_dir_available
+      then
+        let* path, lock_dir = Lock_dir.get_with_path ctx >>| User_error.ok_exn in
+        let+ local_packages =
+          Dune_load.packages ()
+          >>| Dune_lang.Package.Name.Map.map ~f:Dune_pkg.Local_package.of_package
+        in
+        match
+          Dune_pkg.Package_universe.up_to_date
+            local_packages
+            ~dependency_hash:(Option.map ~f:snd lock_dir.dependency_hash)
+        with
+        | `Valid -> ()
+        | `Invalid ->
+          let loc = Loc.in_file (Path.source (Path.Source.relative path "lock.dune")) in
+          let hints = Pp.[ text "run dune pkg lock" ] in
+          User_error.raise
+            ~loc
+            ~hints
+            [ Pp.text "The lock dir is not sync with your dune-project" ]
+      else Memo.return ())
+    |> Memo.Lazy.force)
+  |> Staged.unstage
+;;
+
 let gen_rules ctx ~dir components =
   if Context_name.equal ctx Install.Context.install_context.name
   then (
@@ -687,5 +717,11 @@ let gen_rules ctx ~dir components =
           (Memo.return rules)))
   else if Context_name.equal ctx Private_context.t.name
   then private_context ~dir components ctx
-  else gen_rules ctx (Super_context.find_exn ctx) ~dir components
+  else if Context_name.equal ctx Fetch_rules.context.name
+  then Fetch_rules.gen_rules ~dir ~components
+  else
+    let* () = raise_on_lock_dir_out_of_sync ctx in
+    let gen_pkg_alias_rule = Pkg_rules.setup_pkg_install_alias ~dir ctx in
+    let+ sctx_rules = gen_rules ctx (Super_context.find_exn ctx) ~dir components in
+    Gen_rules.combine sctx_rules gen_pkg_alias_rule
 ;;
