@@ -120,14 +120,14 @@ let build_lib
 
 let gen_wrapped_compat_modules (lib : Library.t) cctx =
   let modules = Compilation_context.modules cctx in
-  let wrapped_compat = Modules.wrapped_compat modules in
+  let wrapped_compat = Modules.With_vlib.wrapped_compat modules in
   let transition_message =
     lazy
-      (match Modules.wrapped modules with
+      (match Modules.With_vlib.wrapped modules with
        | Simple _ -> assert false
        | Yes_with_transition r -> r)
   in
-  Module_name.Map_traversals.parallel_iter wrapped_compat ~f:(fun name m ->
+  Memo.parallel_iter_seq (Module_name.Map.to_seq wrapped_compat) ~f:(fun (name, m) ->
     let main_module_name =
       match Library.main_module_name lib with
       | This (Some mmn) -> Module_name.to_string mmn
@@ -213,7 +213,6 @@ let ocamlmklib
            | false -> [ static_target ])
     >>= build ~sandbox:Sandbox_config.no_special_requirements ~custom:false
   else
-    let open Memo.O in
     (* Build the static target only by passing the [-custom] flag. *)
     let* () =
       build ~sandbox:Sandbox_config.no_special_requirements ~custom:true [ static_target ]
@@ -289,20 +288,13 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
     Foreign_sources.for_lib foreign_sources ~name
   in
   let* o_files =
-    let lib_foreign_o_files =
-      let { Lib_config.ext_obj; _ } = (Compilation_context.ocaml cctx).lib_config in
-      Foreign.Objects.build_paths lib.buildable.extra_objects ~ext_obj ~dir
-    in
-    let+ tbl =
-      Foreign_rules.build_o_files
-        ~sctx
-        ~dir
-        ~expander
-        ~requires
-        ~dir_contents
-        ~foreign_sources
-    in
-    Mode.Map.Multi.add_all tbl Mode.Select.All lib_foreign_o_files
+    Foreign_rules.build_o_files
+      ~sctx
+      ~dir
+      ~expander
+      ~requires
+      ~dir_contents
+      ~foreign_sources
   in
   let all_o_files = Mode.Map.Multi.to_flat_list o_files in
   let* () = Check_rules.add_files sctx ~dir all_o_files in
@@ -328,7 +320,14 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents ~vlib_stubs_o_f
       | _ -> Action_builder.return []
     in
     let c_library_flags =
-      Expander.expand_and_eval_set expander lib.c_library_flags ~standard
+      let open Action_builder.O in
+      let+ c_lib = Expander.expand_and_eval_set expander lib.c_library_flags ~standard
+      and+ ctypes_lib =
+        (* CR rgrinberg: Should we add these flags to :standard? to make
+           it possible for users to remove these *)
+        Ctypes_rules.ctypes_cclib_flags sctx ~expander ~buildable:lib.buildable
+      in
+      c_lib @ ctypes_lib
     in
     let lib_o_files_for_all_modes = Mode.Map.Multi.for_all_modes o_files in
     let for_all_modes = List.rev_append vlib_stubs_o_files lib_o_files_for_all_modes in
@@ -415,7 +414,6 @@ let build_shared lib ~native_archives ~sctx ~dir ~flags =
 ;;
 
 let iter_modes_concurrently (t : _ Ocaml.Mode.Dict.t) ~(f : Ocaml.Mode.t -> unit Memo.t) =
-  let open Memo.O in
   let+ () = Memo.when_ t.byte (fun () -> f Byte)
   and+ () = Memo.when_ t.native (fun () -> f Native) in
   ()
@@ -430,9 +428,8 @@ let setup_build_archives (lib : Library.t) ~top_sorted_modules ~cctx ~expander ~
   let sctx = Compilation_context.super_context cctx in
   let ocaml = Compilation_context.ocaml cctx in
   let { Lib_config.ext_obj; natdynlink_supported; _ } = ocaml.lib_config in
-  let open Memo.O in
   let* () =
-    Modules.exit_module modules
+    Modules.With_vlib.exit_module modules
     |> Memo.Option.iter ~f:(fun m ->
       (* These files needs to be alongside stdlib.cma as the compiler
          implicitly adds this module. *)
@@ -474,24 +471,28 @@ let setup_build_archives (lib : Library.t) ~top_sorted_modules ~cctx ~expander ~
     iter_modes_concurrently modes.ocaml ~f:(fun mode ->
       build_lib lib ~native_archives ~dir ~sctx ~expander ~flags ~mode ~cm_files)
   and* () =
-    (* Build *.cma.js *)
+    (* Build *.cma.js / *.wasma *)
     Memo.when_ modes.ocaml.byte (fun () ->
       let src = Library.archive lib ~dir ~ext:(Mode.compiled_lib_ext Mode.Byte) in
-      let action_with_targets =
-        List.map Jsoo_rules.Config.all ~f:(fun config ->
-          Jsoo_rules.build_cm
-            sctx
-            ~dir
-            ~in_context:js_of_ocaml
-            ~config:(Some config)
-            ~src:(Path.build src)
-            ~obj_dir)
-      in
-      Memo.parallel_iter action_with_targets ~f:(fun rule ->
-        Super_context.add_rule sctx ~dir ~loc:lib.buildable.loc rule))
+      Memo.parallel_iter Js_of_ocaml.Mode.all ~f:(fun mode ->
+        let action_with_targets =
+          List.map Jsoo_rules.Config.all ~f:(fun config ->
+            Jsoo_rules.build_cm
+              sctx
+              ~dir
+              ~in_context:(Js_of_ocaml.Mode.Pair.select ~mode js_of_ocaml)
+              ~mode
+              ~config:(Some config)
+              ~src:(Path.build src)
+              ~obj_dir)
+        in
+        Memo.parallel_iter action_with_targets ~f:(fun rule ->
+          Super_context.add_rule sctx ~dir ~loc:lib.buildable.loc rule)))
   in
   Memo.when_
-    (Dynlink_supported.By_the_os.get natdynlink_supported && modes.ocaml.native)
+    (Lib_info.dynlink_supported lib_info
+     && Dynlink_supported.By_the_os.get natdynlink_supported
+     && modes.ocaml.native)
     (fun () -> build_shared ~native_archives ~sctx lib ~dir ~flags)
 ;;
 
@@ -521,14 +522,16 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
   in
   let package = Library.package lib in
   let js_of_ocaml = Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml in
-  (* XXX(anmonteiro): `public_lib_name` is used to derive Melange's
+  (* XXX(anmonteiro): `melange_package_name` is used to derive Melange's
      `--bs-package-name` argument. We only use the library name for public
-     libraries because we need melange to preserve relative paths for private
-     libs (i.e. not pass the `--bs-package-name` arg). *)
-  let public_lib_name =
+     libraries / private libraries with `(package ..)` because we need Melange
+     to preserve relative paths for private libs (i.e. not pass the
+     `--bs-package-name` arg). *)
+  let melange_package_name =
     match lib.visibility with
     | Public p -> Some (Public_lib.name p)
-    | Private _ -> None
+    | Private (Some pkg) -> Some (Lib_name.mangled (Package.name pkg) (snd lib.name))
+    | Private None -> None
   in
   Compilation_context.create
     ()
@@ -541,11 +544,11 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
     ~requires_link
     ~preprocessing:pp
     ~opaque:Inherit_from_settings
-    ~js_of_ocaml:(Some js_of_ocaml)
+    ~js_of_ocaml:(Js_of_ocaml.Mode.Pair.map ~f:Option.some js_of_ocaml)
     ?stdlib:lib.stdlib
     ~package
     ?vimpl
-    ?public_lib_name
+    ~melange_package_name
     ~modes
 ;;
 
@@ -556,6 +559,7 @@ let library_rules
   ~source_modules
   ~dir_contents
   ~compile_info
+  ~ctx_dir
   =
   let source_modules =
     Modules.fold_user_written source_modules ~init:[] ~f:(fun m acc -> m :: acc)
@@ -569,9 +573,10 @@ let library_rules
   let scope = Compilation_context.scope cctx in
   let* requires_compile = Compilation_context.requires_compile cctx in
   let ocaml = Compilation_context.ocaml cctx in
+  let* requires_hidden = Compilation_context.requires_hidden cctx in
   let stdlib_dir = ocaml.lib_config.stdlib_dir in
   let top_sorted_modules =
-    let impl_only = Modules.impl_only modules in
+    let impl_only = Modules.With_vlib.impl_only modules in
     Dep_graph.top_closed_implementations
       (Compilation_context.dep_graphs cctx).impl
       impl_only
@@ -579,18 +584,27 @@ let library_rules
   let* () =
     Memo.Option.iter vimpl ~f:(Virtual_rules.setup_copy_rules_for_impl ~sctx ~dir)
   in
+  let* expander = Super_context.expander sctx ~dir in
   let* () = Check_rules.add_cycle_check sctx ~dir top_sorted_modules in
   let* () = gen_wrapped_compat_modules lib cctx
   and* () = Module_compilation.build_all cctx
-  and* expander = Super_context.expander sctx ~dir
   and* lib_info =
     let lib_config = ocaml.lib_config in
-    let* info = Library.to_lib_info lib ~dir ~lib_config in
+    let info =
+      Library.to_lib_info
+        lib
+        ~expander:(Memo.return (Expander.to_expander0 expander))
+        ~dir
+        ~lib_config
+    in
     let mode = Lib_mode.Map.Set.for_merlin (Lib_info.modes info) in
     let+ () = Check_rules.add_obj_dir sctx ~obj_dir mode in
     info
   in
   let+ () =
+    Memo.when_ (Compilation_context.bin_annot cctx) (fun () ->
+      Ocaml_index.cctx_rules cctx)
+  and+ () =
     Memo.when_
       (not (Library.is_virtual lib))
       (fun () -> setup_build_archives lib ~lib_info ~top_sorted_modules ~cctx ~expander)
@@ -607,6 +621,7 @@ let library_rules
           ~requires:requires_compile
           ~dir_contents
           ~vlib_stubs_o_files)
+  and+ () = Odoc.setup_private_library_doc_alias sctx ~scope ~dir:ctx_dir lib
   and+ () = Odoc.setup_library_odoc_rules cctx local_lib
   and+ () =
     Sub_system.gen_rules
@@ -614,32 +629,35 @@ let library_rules
   in
   ( cctx
   , Merlin.make
-      ~requires:requires_compile
+      ~requires_compile
+      ~requires_hidden
       ~stdlib_dir
       ~flags
       ~modules
-      ~source_dirs:Path.Source.Set.empty
       ~preprocess:(Preprocess.Per_module.without_instrumentation lib.buildable.preprocess)
       ~libname:(Some (snd lib.name))
       ~obj_dir
       ~dialects:(Dune_project.dialects (Scope.project scope))
-      ~ident:(Lib.Compile.merlin_ident compile_info)
+      ~ident:(Merlin_ident.for_lib (Library.best_name lib))
       ~modes:(`Lib (Lib_info.modes lib_info)) )
 ;;
 
 let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope =
   let buildable = lib.buildable in
+  let libs = Scope.libs scope in
+  let lib_id =
+    let src_dir = Path.Build.drop_build_context_exn dir in
+    Library.to_lib_id ~src_dir lib
+  in
   let* local_lib, compile_info =
     Lib.DB.get_compile_info
-      (Scope.libs scope)
-      (Library.best_name lib)
+      libs
+      (Local lib_id)
       ~allow_overlaps:buildable.allow_overlapping_dependencies
   in
-  let local_lib = Lib.Local.of_lib_exn local_lib in
   let f () =
     let* source_modules =
-      Dir_contents.ocaml dir_contents
-      >>| Ml_sources.modules ~for_:(Library (Library.best_name lib))
+      Dir_contents.ocaml dir_contents >>= Ml_sources.modules ~libs ~for_:(Library lib_id)
     in
     let* cctx = cctx lib ~sctx ~source_modules ~dir ~scope ~expander ~compile_info in
     let* () =
@@ -648,8 +666,16 @@ let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope =
       | Some _ ->
         Ctypes_rules.gen_rules ~loc:(fst lib.name) ~cctx ~buildable ~sctx ~scope ~dir
     in
-    library_rules lib ~local_lib ~cctx ~source_modules ~dir_contents ~compile_info
+    library_rules
+      lib
+      ~local_lib:(Lib.Local.of_lib_exn local_lib)
+      ~cctx
+      ~source_modules
+      ~dir_contents
+      ~compile_info
+      ~ctx_dir:dir
   in
   let* () = Buildable_rules.gen_select_rules sctx compile_info ~dir in
-  Buildable_rules.with_lib_deps (Super_context.context sctx) compile_info ~dir ~f
+  let merlin_ident = Merlin_ident.for_lib (Library.best_name lib) in
+  Buildable_rules.with_lib_deps (Super_context.context sctx) merlin_ident ~dir ~f
 ;;
