@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 type t = Link_time_code_gen_type.t =
   { to_link : Lib_flags.Lib_and_module.L.t
@@ -8,7 +9,6 @@ type t = Link_time_code_gen_type.t =
 let generate_and_compile_module cctx ~precompiled_cmi ~obj_name ~name ~lib ~code ~requires
   =
   let sctx = Compilation_context.super_context cctx in
-  let open Memo.O in
   let* module_ =
     let+ modules = Dir_contents.modules_of_lib sctx lib in
     let obj_name =
@@ -16,7 +16,7 @@ let generate_and_compile_module cctx ~precompiled_cmi ~obj_name ~name ~lib ~code
       | Some _ -> obj_name
       | None ->
         Option.map modules ~f:(fun modules ->
-          Modules.find modules name |> Option.value_exn |> Module.obj_name)
+          Modules.With_vlib.find modules name |> Option.value_exn |> Module.obj_name)
     in
     let src_dir =
       let obj_dir = Compilation_context.obj_dir cctx in
@@ -91,7 +91,7 @@ let build_info_code cctx ~libs ~api_version =
   (match api_version with
    | Lib_info.Special_builtin_support.Build_info.V1 -> ());
   let placeholder placeholders p =
-    Vcs_db.nearest_vcs p
+    Source_tree.nearest_vcs p
     >>| function
     | None -> "None", placeholders
     | Some vcs ->
