@@ -3,7 +3,10 @@ module Pkg = Dune_pkg.Lock_dir.Pkg
 
 type t := Dune_pkg.Lock_dir.t
 
-val get : Context_name.t -> t Memo.t
+val get_with_path : Context_name.t -> (Path.Source.t * t, User_message.t) result Memo.t
+val get : Context_name.t -> (t, User_message.t) result Memo.t
+val get_exn : Context_name.t -> t Memo.t
+val of_dev_tool : Dune_pkg.Dev_tool.t -> t Memo.t
 val lock_dir_active : Context_name.t -> bool Memo.t
 val get_path : Context_name.t -> Path.Source.t option Memo.t
 
@@ -19,3 +22,7 @@ module Sys_vars : sig
 
   val poll : t
 end
+
+val source_kind
+  :  Dune_pkg.Source.t
+  -> [ `Local of [ `Directory | `File ] * Path.External.t | `Fetch ] Memo.t
