@@ -46,8 +46,6 @@ let execution_parameters =
 ;;
 
 let init
-  ?(action_runner = fun _ -> None)
-  ?(action_runners = fun _ -> [])
   ~stats
   ~sandboxing_preference
   ~cache_config
@@ -70,7 +68,7 @@ let init
       ()
   in
   let module Shared_cache =
-    Dune_shared_cache.Make (struct
+    Dune_cache.Shared.Make (struct
       let debug_shared_cache = cache_debug_flags.shared_cache
       let config = cache_config
       let upload ~rule_digest:_ = Fiber.return ()
@@ -97,8 +95,6 @@ let init
     ~execution_parameters
     ~source_tree:(module Source_tree)
     ~shared_cache:(module Shared_cache)
-    ~action_runner
-    ~action_runners
     ~write_error_summary:(fun _ -> Fiber.return ())
 ;;
 
