@@ -1,17 +1,24 @@
 open Import
 open Memo.O
 
-let output_of_lib ~target_dir lib =
-  let info = Lib.info lib in
-  match Lib_info.status info with
-  | Private _ -> `Private_library_or_emit target_dir
-  | Installed | Installed_private | Public _ ->
-    let lib_name = Lib_info.name info in
-    let src_dir = Lib_info.src_dir info in
+let output_of_lib =
+  let public_lib ~info ~target_dir lib_name =
     `Public_library
-      ( src_dir
+      ( Lib_info.src_dir info
       , Path.Build.L.relative target_dir [ "node_modules"; Lib_name.to_string lib_name ]
       )
+  in
+  fun ~target_dir lib ->
+    let info = Lib.info lib in
+    match Lib_info.status info with
+    | Private (_, None) -> `Private_library_or_emit target_dir
+    | Private (_, Some pkg) ->
+      public_lib
+        ~info
+        ~target_dir
+        (Lib_name.mangled (Package.name pkg) (Lib_name.to_local_exn (Lib.name lib)))
+    | Installed | Installed_private | Public _ ->
+      public_lib ~info ~target_dir (Lib_info.name info)
 ;;
 
 let lib_output_path ~output_dir ~lib_dir src =
@@ -40,10 +47,23 @@ let make_js_name ~js_ext ~output m =
     Path.Build.relative dst_dir basename
 ;;
 
-let impl_only_modules_defined_in_this_lib sctx lib =
-  let+ modules = Dir_contents.modules_of_lib sctx lib in
-  match modules with
-  | None ->
+let modules_in_obj_dir ~sctx ~scope ~preprocess modules =
+  let* version =
+    let+ ocaml = Context.ocaml (Super_context.context sctx) in
+    ocaml.version
+  and* preprocess =
+    Resolve.Memo.read_memo
+      (Preprocess.Per_module.with_instrumentation
+         preprocess
+         ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
+  in
+  let pped_map = Staged.unstage (Pp_spec.pped_modules_map preprocess version) in
+  Modules.map_user_written modules ~f:(fun m -> Memo.return @@ pped_map m)
+;;
+
+let impl_only_modules_defined_in_this_lib ~sctx ~scope lib =
+  match Lib_info.modules (Lib.info lib) with
+  | External None ->
     User_error.raise
       [ Pp.textf
           "The library %s was not compiled with Dune or it was compiled with Dune but \
@@ -51,14 +71,43 @@ let impl_only_modules_defined_in_this_lib sctx lib =
            melange support"
           (Lib.name lib |> Lib_name.to_string)
       ]
-  | Some modules ->
-    (* for a virtual library,this will return all modules *)
-    (Modules.split_by_lib modules).impl |> List.filter ~f:(Module.has ~ml_kind:Impl)
+  | External (Some modules) ->
+    Memo.return
+      ( modules
+      , (Modules.With_vlib.split_by_lib modules).impl
+        |> List.filter ~f:(Module.has ~ml_kind:Impl) )
+  | Local ->
+    let lib = Lib.Local.of_lib_exn lib in
+    let info = Lib.Local.info lib in
+    let+ modules =
+      let* modules = Dir_contents.modules_of_local_lib sctx lib in
+      let preprocess = Lib_info.preprocess info in
+      modules_in_obj_dir ~sctx ~scope ~preprocess modules >>| Modules.With_vlib.modules
+    in
+    let () =
+      let modes = Lib_info.modes info in
+      match modes.melange with
+      | false ->
+        let lib_name = Lib_name.to_string (Lib_info.name info) in
+        User_error.raise
+          ~loc:(Lib_info.loc info)
+          [ Pp.textf
+              "The library `%s` was added as a dependency of a `melange.emit` stanza, \
+               but this library is not compatible with Melange. To fix this, add \
+               `melange` to the `modes` field of the library `%s`."
+              lib_name
+              lib_name
+          ]
+      | true -> ()
+    in
+    ( modules
+    , (Modules.With_vlib.split_by_lib modules).impl
+      |> List.filter ~f:(Module.has ~ml_kind:Impl) )
 ;;
 
 let cmj_glob = Glob.of_string_exn Loc.none "*.cmj"
 
-let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope =
+let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope lib_config =
   let project = Scope.project scope in
   let deps_of_lib lib =
     let info = Lib.info lib in
@@ -66,17 +115,19 @@ let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope =
     let dir = Obj_dir.melange_dir obj_dir in
     Dep.file_selector @@ File_selector.of_glob ~dir cmj_glob
   in
-  let open Resolve.O in
   Command.Args.memo
   @@ Resolve.args
-  @@ let+ requires_link = requires_link in
-     let deps = List.map requires_link ~f:deps_of_lib |> Dep.Set.of_list in
-     Command.Args.S
-       [ Lib_flags.L.include_flags ~project requires_link Melange; Hidden_deps deps ]
+  @@
+  let open Resolve.O in
+  let+ requires_link = requires_link in
+  let deps = List.map requires_link ~f:deps_of_lib |> Dep.Set.of_list in
+  Command.Args.S
+    [ Lib_flags.L.melange_emission_include_flags ~project requires_link lib_config
+    ; Hidden_deps deps
+    ]
 ;;
 
 let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
-  let open Memo.O in
   let dune_version = Scope.project scope |> Dune_project.dune_version in
   let+ pps =
     Resolve.Memo.read_memo
@@ -85,13 +136,12 @@ let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
          ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
     >>| Preprocess.Per_module.pps
   in
-  let merlin_ident = Merlin_ident.for_melange ~target:mel.target in
   let libraries =
     match mel.emit_stdlib with
+    | false -> mel.libraries
     | true ->
       let builtin_melange_dep = Lib_dep.Direct (mel.loc, Lib_name.of_string "melange") in
       builtin_melange_dep :: mel.libraries
-    | false -> mel.libraries
   in
   Lib.DB.resolve_user_written_deps
     (Scope.libs scope)
@@ -101,12 +151,13 @@ let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
     libraries
     ~pps
     ~dune_version
-    ~merlin_ident
 ;;
 
 let js_targets_of_modules modules ~module_systems ~output =
   List.map module_systems ~f:(fun (_, js_ext) ->
-    Modules.fold_no_vlib modules ~init:Path.Set.empty ~f:(fun m acc ->
+    modules
+    |> Modules.With_vlib.drop_vlib
+    |> Modules.fold ~init:Path.Set.empty ~f:(fun m acc ->
       if Module.has m ~ml_kind:Impl
       then (
         let target = Path.build @@ make_js_name ~js_ext ~output m in
@@ -115,16 +166,14 @@ let js_targets_of_modules modules ~module_systems ~output =
   |> Path.Set.union_all
 ;;
 
-let js_targets_of_libs sctx libs ~module_systems ~target_dir =
+let js_targets_of_libs ~sctx ~scope ~module_systems ~target_dir libs =
   Resolve.Memo.List.concat_map module_systems ~f:(fun (_, js_ext) ->
     let of_lib lib =
-      let open Memo.O in
-      let+ modules = impl_only_modules_defined_in_this_lib sctx lib in
+      let+ _, modules = impl_only_modules_defined_in_this_lib ~sctx ~scope lib in
       let output = output_of_lib ~target_dir lib in
       List.rev_map modules ~f:(fun m -> Path.build @@ make_js_name ~output ~js_ext m)
     in
     Resolve.Memo.List.concat_map libs ~f:(fun lib ->
-      let open Memo.O in
       let* base = of_lib lib in
       match Lib.implements lib with
       | None -> Resolve.Memo.return base
@@ -135,37 +184,70 @@ let js_targets_of_libs sctx libs ~module_systems ~target_dir =
         List.rev_append for_vlib base))
 ;;
 
-let build_js ~loc ~dir ~pkg_name ~mode ~module_systems ~output ~obj_dir ~sctx ~includes m =
-  let open Memo.O in
+let build_js
+  ~loc
+  ~dir
+  ~pkg_name
+  ~mode
+  ~module_systems
+  ~output
+  ~obj_dir
+  ~sctx
+  ~includes
+  ~(compile_flags : Ocaml_flags.t)
+  ~local_modules_and_obj_dir
+  m
+  =
   let* compiler = Melange_binary.melc sctx ~loc:(Some loc) ~dir in
   Memo.parallel_iter module_systems ~f:(fun (module_system, js_ext) ->
-    let src = Obj_dir.Module.cm_file_exn obj_dir m ~kind:(Melange Cmj) in
-    let output = make_js_name ~output ~js_ext m in
-    let obj_dir = [ Command.Args.A "-I"; Path (Obj_dir.melange_dir obj_dir) ] in
-    let melange_package_args =
-      let pkg_name_args =
-        match pkg_name with
-        | None -> []
-        | Some pkg_name -> [ "--bs-package-name"; Package.Name.to_string pkg_name ]
+    let build =
+      let command =
+        let src = Obj_dir.Module.cm_file_exn obj_dir m ~kind:(Melange Cmj) in
+        let output = make_js_name ~output ~js_ext m in
+        let obj_dir = [ Command.Args.A "-I"; Path (Obj_dir.melange_dir obj_dir) ] in
+        let melange_package_args =
+          let pkg_name_args =
+            match pkg_name with
+            | None -> []
+            | Some pkg_name -> [ "--bs-package-name"; Package.Name.to_string pkg_name ]
+          in
+          let js_modules_str = Melange.Module_system.to_string module_system in
+          "--bs-module-type" :: js_modules_str :: pkg_name_args
+        in
+        Command.run
+          ~dir:(Super_context.context sctx |> Context.build_dir |> Path.build)
+          compiler
+          [ Command.Args.S obj_dir
+          ; Command.Args.as_any includes
+          ; Command.Args.dyn (Ocaml_flags.get compile_flags Melange)
+          ; As melange_package_args
+          ; A "-o"
+          ; Target output
+          ; Dep src
+          ]
       in
-      let js_modules_str = Melange.Module_system.to_string module_system in
-      "--bs-module-type" :: js_modules_str :: pkg_name_args
+      With_targets.map_build command ~f:(fun command ->
+        let open Action_builder.O in
+        match local_modules_and_obj_dir with
+        | Some (modules, obj_dir) ->
+          let paths =
+            let+ module_deps =
+              Dep_rules.immediate_deps_of m modules ~obj_dir ~ml_kind:Impl
+            in
+            List.fold_left module_deps ~init:[] ~f:(fun acc dep_m ->
+              if Module.has dep_m ~ml_kind:Impl
+              then (
+                let cmj_file =
+                  let kind : Lib_mode.Cm_kind.t = Melange Cmj in
+                  Obj_dir.Module.cm_file_exn obj_dir dep_m ~kind |> Path.build
+                in
+                cmj_file :: acc)
+              else acc)
+          in
+          Action_builder.dyn_paths_unit paths >>> command
+        | None -> command)
     in
-    Super_context.add_rule
-      sctx
-      ~dir
-      ~loc
-      ~mode
-      (Command.run
-         ~dir:(Super_context.context sctx |> Context.build_dir |> Path.build)
-         compiler
-         [ Command.Args.S obj_dir
-         ; Command.Args.as_any includes
-         ; As melange_package_args
-         ; A "-o"
-         ; Target output
-         ; Dep src
-         ]))
+    Super_context.add_rule sctx ~dir ~loc ~mode build)
 ;;
 
 (* attach [deps] to the specified [alias] AND the (dune default) [all] alias.
@@ -179,6 +261,13 @@ let add_deps_to_aliases ?(alias = Melange_stanzas.Emit.implicit_alias) ~dir deps
   Memo.parallel_iter ~f:attach [ alias; dune_default_alias ]
 ;;
 
+let melange_compile_flags ~sctx ~dir (mel : Melange_stanzas.Emit.t) =
+  let specific = Lib_mode.Map.make_all mel.compile_flags in
+  Ocaml_flags.Spec.make ~common:Ordered_set_lang.Unexpanded.standard ~specific
+  |> Ocaml_flags_db.ocaml_flags sctx ~dir
+  >>| Ocaml_flags.allow_only_melange
+;;
+
 let setup_emit_cmj_rules
   ~sctx
   ~dir
@@ -187,40 +276,37 @@ let setup_emit_cmj_rules
   ~dir_contents
   (mel : Melange_stanzas.Emit.t)
   =
-  let open Memo.O in
   let* compile_info = compile_info ~scope mel in
   let ctx = Super_context.context sctx in
+  let merlin_ident = Merlin_ident.for_melange ~target:mel.target in
   let f () =
-    (* Use "mobjs" rather than "objs" to avoid a potential conflict with a library
-       of the same name *)
     let* modules, obj_dir =
       Dir_contents.ocaml dir_contents
-      >>| Ml_sources.modules_and_obj_dir ~for_:(Melange { target = mel.target })
+      >>= Ml_sources.modules_and_obj_dir
+            ~libs:(Scope.libs scope)
+            ~for_:(Melange { target = mel.target })
     in
     let* () = Check_rules.add_obj_dir sctx ~obj_dir Melange in
     let* modules, pp =
-      Buildable_rules.modules_rules
-        sctx
-        (Melange
-           { preprocess = mel.preprocess
-           ; preprocessor_deps = mel.preprocessor_deps
-           ; (* TODO still needed *)
-             lint = Preprocess.Per_module.default ()
-           ; (* why is this always false? *)
-             empty_module_interface_if_absent = false
-           })
-        expander
-        ~dir
-        scope
-        modules
+      let+ modules, pp =
+        Buildable_rules.modules_rules
+          sctx
+          (Melange
+             { preprocess = mel.preprocess
+             ; preprocessor_deps = mel.preprocessor_deps
+             ; lint = mel.lint
+             ; (* why is this always false? *)
+               empty_module_interface_if_absent = false
+             })
+          expander
+          ~dir
+          scope
+          modules
+      in
+      Modules.With_vlib.modules modules, pp
     in
     let requires_link = Lib.Compile.requires_link compile_info in
-    let* flags =
-      let specific = Lib_mode.Map.make_all mel.compile_flags in
-      Ocaml_flags.Spec.make ~common:Ordered_set_lang.Unexpanded.standard ~specific
-      |> Ocaml_flags_db.ocaml_flags sctx ~dir
-      >>| Ocaml_flags.allow_only_melange
-    in
+    let* flags = melange_compile_flags ~sctx ~dir mel in
     let* cctx =
       let direct_requires = Lib.Compile.direct_requires compile_info in
       Compilation_context.create
@@ -234,14 +320,16 @@ let setup_emit_cmj_rules
         ~requires_link
         ~requires_compile:direct_requires
         ~preprocessing:pp
-        ~js_of_ocaml:None
+        ~js_of_ocaml:(Js_of_ocaml.Mode.Pair.make None)
         ~opaque:Inherit_from_settings
+        ~melange_package_name:None
         ~package:mel.package
         ~modes:
           { ocaml = { byte = None; native = None }; melange = Some (Requested mel.loc) }
     in
     let* () = Module_compilation.build_all cctx in
     let* requires_compile = Compilation_context.requires_compile cctx in
+    let* requires_hidden = Compilation_context.requires_hidden cctx in
     let stdlib_dir = (Compilation_context.ocaml cctx).lib_config.stdlib_dir in
     let+ () =
       let emit_and_libs_deps =
@@ -255,13 +343,12 @@ let setup_emit_cmj_rules
             modules
           |> Action_builder.path_set
         and+ () =
-          let open Action_builder.O in
           let* deps =
             Resolve.Memo.read
             @@
             let open Resolve.Memo.O in
             Compilation_context.requires_link cctx
-            >>= js_targets_of_libs sctx ~module_systems ~target_dir
+            >>= js_targets_of_libs ~sctx ~scope ~module_systems ~target_dir
           in
           Action_builder.paths deps
         in
@@ -271,25 +358,24 @@ let setup_emit_cmj_rules
     in
     ( cctx
     , Merlin.make
-        ~requires:requires_compile
+        ~requires_compile
+        ~requires_hidden
         ~stdlib_dir
         ~flags
         ~modules
-        ~source_dirs:Path.Source.Set.empty
         ~libname:None
         ~preprocess:(Preprocess.Per_module.without_instrumentation mel.preprocess)
         ~obj_dir
-        ~ident:(Lib.Compile.merlin_ident compile_info)
+        ~ident:merlin_ident
         ~dialects:(Dune_project.dialects (Scope.project scope))
         ~modes:`Melange_emit )
   in
   let* () = Buildable_rules.gen_select_rules sctx compile_info ~dir in
-  Buildable_rules.with_lib_deps ctx compile_info ~dir ~f
+  Buildable_rules.with_lib_deps ctx merlin_ident ~dir ~f
 ;;
 
 module Runtime_deps = struct
   let targets sctx ~dir ~output ~for_ (mel : Melange_stanzas.Emit.t) =
-    let open Memo.O in
     let raise_external_dep_error src =
       let lib_info =
         match for_ with
@@ -340,7 +426,6 @@ module Runtime_deps = struct
 end
 
 let setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_ mel =
-  let open Memo.O in
   let* copy, non_copy = Runtime_deps.targets sctx ~dir ~output ~for_ mel in
   let deps =
     Action_builder.paths
@@ -355,30 +440,18 @@ let setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_ mel =
 ;;
 
 let modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope (mel : Melange_stanzas.Emit.t) =
-  let open Memo.O in
-  (* Use "mobjs" rather than "objs" to avoid a potential conflict with a library
-     of the same name *)
   let* modules, obj_dir =
     Dir_contents.ocaml dir_contents
-    >>| Ml_sources.modules_and_obj_dir ~for_:(Melange { target = mel.target })
-  in
-  let+ modules =
-    let* ocaml = Context.ocaml (Super_context.context sctx) in
-    let version = ocaml.version in
-    let* preprocess =
-      Resolve.Memo.read_memo
-        (Preprocess.Per_module.with_instrumentation
-           mel.preprocess
-           ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
-    in
-    let pped_map = Staged.unstage (Preprocessing.pped_modules_map preprocess version) in
-    Modules.map_user_written modules ~f:(fun m -> Memo.return @@ pped_map m)
+    >>= Ml_sources.modules_and_obj_dir
+          ~libs:(Scope.libs scope)
+          ~for_:(Melange { target = mel.target })
   in
+  let+ modules = modules_in_obj_dir ~sctx ~scope ~preprocess:mel.preprocess modules in
   let modules_for_js =
-    Modules.fold_no_vlib modules ~init:[] ~f:(fun x acc ->
+    Modules.fold modules ~init:[] ~f:(fun x acc ->
       if Module.has x ~ml_kind:Impl then x :: acc else acc)
   in
-  modules_for_js, obj_dir
+  modules, modules_for_js, obj_dir
 ;;
 
 let setup_entries_js
@@ -391,8 +464,7 @@ let setup_entries_js
   ~mode
   (mel : Melange_stanzas.Emit.t)
   =
-  let open Memo.O in
-  let* modules_for_js, obj_dir =
+  let* local_modules, modules_for_js, local_obj_dir =
     modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope mel
   in
   let requires_link = Lib.Compile.requires_link compile_info in
@@ -400,78 +472,166 @@ let setup_entries_js
   let loc = mel.loc in
   let module_systems = mel.module_systems in
   let* requires_link = Memo.Lazy.force requires_link in
-  let includes = cmj_includes ~requires_link ~scope in
+  let* includes =
+    let+ lib_config =
+      let+ ocaml = Super_context.context sctx |> Context.ocaml in
+      ocaml.lib_config
+    in
+    cmj_includes ~requires_link ~scope lib_config
+  and* compile_flags = melange_compile_flags ~sctx ~dir mel in
   let output = `Private_library_or_emit target_dir in
-  let obj_dir = Obj_dir.of_local obj_dir in
+  let obj_dir = Obj_dir.of_local local_obj_dir in
   let* () =
     setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_:`Emit mel
   in
+  let local_modules_and_obj_dir =
+    Some (Modules.With_vlib.modules local_modules, local_obj_dir)
+  in
   Memo.parallel_iter modules_for_js ~f:(fun m ->
-    build_js ~dir ~loc ~pkg_name ~mode ~module_systems ~output ~obj_dir ~sctx ~includes m)
+    build_js
+      ~dir
+      ~loc
+      ~pkg_name
+      ~mode
+      ~module_systems
+      ~output
+      ~obj_dir
+      ~sctx
+      ~includes
+      ~compile_flags
+      ~local_modules_and_obj_dir
+      m)
 ;;
 
-let setup_js_rules_libraries
-  ~dir
-  ~scope
-  ~target_dir
-  ~sctx
-  ~requires_link
-  ~mode
-  (mel : Melange_stanzas.Emit.t)
-  =
-  let build_js = build_js ~sctx ~mode ~module_systems:mel.module_systems in
-  Memo.parallel_iter requires_link ~f:(fun lib ->
-    let open Memo.O in
-    let lib_name = Lib.name lib in
-    let* lib, lib_compile_info =
-      Lib.DB.get_compile_info
-        (Scope.libs scope)
-        lib_name
-        ~allow_overlaps:mel.allow_overlapping_dependencies
+let setup_js_rules_libraries =
+  let local_modules_and_obj_dir ~lib modules =
+    Lib.Local.of_lib lib
+    |> Option.map ~f:(fun lib ->
+      let obj_dir = Lib.Local.obj_dir lib in
+      modules, obj_dir)
+  in
+  let parallel_build_source_modules ~sctx ~scope ~f:build_js lib =
+    let* local_modules_and_obj_dir, source_modules =
+      let+ lib_modules, source_modules =
+        impl_only_modules_defined_in_this_lib ~sctx ~scope lib
+      in
+      local_modules_and_obj_dir ~lib lib_modules, source_modules
     in
-    let info = Lib.info lib in
-    let loc = Lib_info.loc info in
-    let build_js =
-      let obj_dir = Lib_info.obj_dir info in
-      let pkg_name = Lib_info.package info in
-      build_js ~loc ~pkg_name ~obj_dir
+    Memo.parallel_iter source_modules ~f:(build_js ~local_modules_and_obj_dir)
+  in
+  fun ~dir ~scope ~target_dir ~sctx ~requires_link ~mode (mel : Melange_stanzas.Emit.t) ->
+    let build_js = build_js ~sctx ~mode ~module_systems:mel.module_systems in
+    let with_vlib_implementations =
+      let vlib_implementations =
+        (* vlib_name => concrete_impl *)
+        List.fold_left requires_link ~init:Lib_name.Map.empty ~f:(fun acc dep ->
+          match Lib_info.implements (Lib.info dep) with
+          | None -> acc
+          | Some (_, vlib_name) -> Lib_name.Map.add_exn acc vlib_name dep)
+      in
+      fun lib deps ->
+        (* Depend on the concrete implementations of virtual libraries so
+           that Melange can find their `.cmj` files. *)
+        List.fold_left deps ~init:deps ~f:(fun acc dep ->
+          match Lib_name.Map.find vlib_implementations (Lib.name dep) with
+          | None -> acc
+          | Some sub -> if Lib.equal sub lib then acc else sub :: acc)
     in
-    let* includes =
-      let+ requires_link = Memo.Lazy.force (Lib.Compile.requires_link lib_compile_info) in
-      cmj_includes ~requires_link ~scope
+    let* lib_config =
+      let+ ocaml = Super_context.context sctx |> Context.ocaml in
+      ocaml.lib_config
     in
-    let output = output_of_lib ~target_dir lib in
-    let* () =
-      setup_runtime_assets_rules
-        sctx
-        ~dir
-        ~target_dir
-        ~mode
-        ~output
-        ~for_:(`Library info)
-        mel
-    in
-    let* () =
-      match Lib.implements lib with
-      | None -> Memo.return ()
-      | Some vlib ->
-        let* vlib = Resolve.Memo.read_memo vlib in
-        let* includes =
-          let+ requires_link =
-            Lib.Compile.for_lib
-              ~allow_overlaps:mel.allow_overlapping_dependencies
-              (Scope.libs scope)
-              vlib
-            |> Lib.Compile.requires_link
-            |> Memo.Lazy.force
-          in
-          cmj_includes ~requires_link ~scope
+    Memo.parallel_iter requires_link ~f:(fun lib ->
+      let lib_compile_info =
+        Lib.Compile.for_lib
+          ~allow_overlaps:mel.allow_overlapping_dependencies
+          (Scope.libs scope)
+          lib
+      in
+      let info = Lib.info lib in
+      let loc = Lib_info.loc info in
+      let build_js =
+        let obj_dir = Lib_info.obj_dir info in
+        let pkg_name = Lib_info.package info in
+        build_js ~loc ~pkg_name ~obj_dir
+      in
+      let output = output_of_lib ~target_dir lib in
+      let* includes =
+        let+ requires_link =
+          Memo.Lazy.force (Lib.Compile.requires_link lib_compile_info)
+          |> Resolve.Memo.map ~f:(with_vlib_implementations lib)
         in
-        impl_only_modules_defined_in_this_lib sctx vlib
-        >>= Memo.parallel_iter ~f:(build_js ~dir ~output ~includes)
-    in
-    let* source_modules = impl_only_modules_defined_in_this_lib sctx lib in
-    Memo.parallel_iter source_modules ~f:(build_js ~dir ~output ~includes))
+        cmj_includes ~requires_link ~scope lib_config
+      and* compile_flags = melange_compile_flags ~sctx ~dir mel in
+      let+ () =
+        setup_runtime_assets_rules
+          sctx
+          ~dir
+          ~target_dir
+          ~mode
+          ~output
+          ~for_:(`Library info)
+          mel
+      and+ () =
+        match Lib.implements lib with
+        | None -> Memo.return ()
+        | Some vlib ->
+          let* vlib = Resolve.Memo.read_memo vlib in
+          let vlib_output = output_of_lib ~target_dir vlib in
+          (match vlib_output, output with
+           | `Public_library _, `Private_library_or_emit _ ->
+             let info = Lib.info lib in
+             User_error.raise
+               ~loc:(Lib_info.loc info)
+               [ Pp.text
+                   "Dune doesn't currently support building private implementations of \
+                    virtual public libaries for `(modes melange)`"
+               ]
+               ~hints:
+                 [ Pp.textf
+                     "Add a `public_name` to the library `%s'."
+                     (Lib_name.to_string (Lib_info.name info))
+                 ]
+           | `Public_library _, `Public_library _ | `Private_library_or_emit _, _ ->
+             let* includes =
+               let+ requires_link =
+                 let+ requires_link =
+                   Lib.Compile.for_lib
+                     ~allow_overlaps:mel.allow_overlapping_dependencies
+                     (Scope.libs scope)
+                     vlib
+                   |> Lib.Compile.requires_link
+                   |> Memo.Lazy.force
+                 in
+                 let open Resolve.O in
+                 let+ requires_link = requires_link in
+                 (* Whenever a `concrete_lib` implementation contains a field
+                    `(implements virt_lib)`, we also set up the JS targets for the
+                    modules defined in `virt_lib`.
+
+                    In the cases where `virt_lib` (concrete) modules depend on any
+                    virtual modules (i.e. programming against the interface), we
+                    need to make sure that the JS rules that dune emits for
+                    `virt_lib` depend on `concrete_lib`, such that Melange can find
+                    the correct `.cmj` file, which is needed to emit the correct
+                    path in `import` / `require`. *)
+                 lib :: requires_link
+               in
+               cmj_includes ~requires_link ~scope lib_config
+             in
+             parallel_build_source_modules
+               ~sctx
+               ~scope
+               vlib
+               ~f:(build_js ~dir ~output:vlib_output ~includes ~compile_flags))
+      and+ () =
+        parallel_build_source_modules
+          ~sctx
+          ~scope
+          lib
+          ~f:(build_js ~dir ~output ~includes ~compile_flags)
+      in
+      ())
 ;;
 
 let setup_js_rules_libraries_and_entries
@@ -485,7 +645,6 @@ let setup_js_rules_libraries_and_entries
   ~target_dir
   mel
   =
-  let open Memo.O in
   let+ () =
     setup_js_rules_libraries ~dir ~scope ~target_dir ~sctx ~requires_link ~mode mel
   and+ () =
@@ -495,7 +654,6 @@ let setup_js_rules_libraries_and_entries
 ;;
 
 let setup_emit_js_rules ~dir_contents ~dir ~scope ~sctx mel =
-  let open Memo.O in
   let target_dir =
     Melange_stanzas.Emit.target_dir ~dir:(Dir_contents.dir dir_contents) mel
   in
@@ -527,7 +685,7 @@ let setup_emit_js_rules ~dir_contents ~dir ~scope ~sctx mel =
        package). When resolution fails, we replace the JS entries with the
        resolution error inside [Action_builder.fail] to give Dune a chance to
        fail if any of the targets end up attached to a package installation. *)
-    let* modules_for_js, _obj_dir =
+    let* _, modules_for_js, _obj_dir =
       modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope mel
     in
     let module_systems = mel.module_systems in
@@ -568,20 +726,33 @@ let emit_rules sctx { stanza_dir; stanza } =
 ;;
 
 (* Detect if [dir] is under the target directory of a melange.emit stanza. *)
-let rec under_melange_emit_target ~dir =
+let rec under_melange_emit_target ~sctx ~dir =
   match Path.Build.parent dir with
   | None -> Memo.return None
   | Some parent ->
     Dune_load.stanzas_in_dir parent
     >>= (function
-     | None -> under_melange_emit_target ~dir:parent
+     | None -> under_melange_emit_target ~sctx ~dir:parent
      | Some stanzas ->
        Dune_file.find_stanzas stanzas Melange_stanzas.Emit.key
-       >>| List.find_map ~f:(fun mel ->
+       >>= Memo.List.find_map ~f:(fun (mel : Melange_stanzas.Emit.t) ->
          let target_dir = Melange_stanzas.Emit.target_dir ~dir:parent mel in
-         Option.some_if (Path.Build.equal target_dir dir) mel)
+         match Path.Build.equal target_dir dir with
+         | false -> Memo.return None
+         | true ->
+           (* In the case where we have two melange.emit stanzas in the same folder,
+              with one enabled in the current context and one disabled, we want to
+              make sure that we pick the enabled one *)
+           let+ enabled =
+             let* expander =
+               let* sctx = sctx in
+               Super_context.expander sctx ~dir
+             in
+             Expander.eval_blang expander mel.enabled_if
+           in
+           Option.some_if enabled mel)
        >>= (function
-        | None -> under_melange_emit_target ~dir:parent
+        | None -> under_melange_emit_target ~sctx ~dir:parent
         | Some stanza -> Memo.return @@ Some { stanza_dir = parent; stanza }))
 ;;
 
@@ -589,10 +760,10 @@ let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
   match Path.Build.equal dir (Melange_stanzas.Emit.target_dir ~dir:stanza_dir stanza) with
   | false -> Memo.return None
   | true ->
-    under_melange_emit_target ~dir:stanza_dir
+    under_melange_emit_target ~sctx ~dir:stanza_dir
     >>| (function
      | None -> Some (emit_rules sctx for_melange)
-     | Some { stanza_dir = parent_melange_emit_dir; stanza = parent_stanza } ->
+     | Some { stanza_dir = _; stanza = parent_stanza } ->
        let main_message = Pp.text "melange.emit stanzas cannot be nested" in
        let annots =
          let main = User_message.make ~loc:stanza.loc [ main_message ] in
@@ -612,22 +783,13 @@ let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
          [ main_message
          ; Pp.enumerate ~f:Loc.pp_file_colon_line [ parent_stanza.loc; stanza.loc ]
          ]
-         ~hints:
-           (let emit_dir = Path.Build.drop_build_context_exn stanza_dir in
-            let parent_melange_emit_dir =
-              Path.Build.drop_build_context_exn parent_melange_emit_dir
-            in
-            [ Pp.textf
-                "Move the melange.emit stanza from %s to at least the level of %s"
-                (Path.Source.to_string emit_dir)
-                (Path.Source.to_string parent_melange_emit_dir)
-            ]))
+         ~hints:[ Pp.text "Move both `melange.emit' stanzas to the same level." ])
 ;;
 
-module Gen_rules = Import.Build_config.Gen_rules
+module Gen_rules = Build_config.Gen_rules
 
 let setup_emit_js_rules sctx ~dir =
-  under_melange_emit_target ~dir
+  under_melange_emit_target ~sctx ~dir
   >>= function
   | Some melange ->
     gen_emit_rules sctx ~dir melange
