@@ -55,8 +55,7 @@ module Unique : sig
   include Dune_lang.Conv.S with type t := t
   include Comparable_intf.S with type key := t
 
-  module Map_traversals : sig
-    val parallel_iter : 'a Map.t -> f:(t -> 'a -> unit Memo.t) -> unit Memo.t
+  module Parallel_map : sig
     val parallel_map : 'a Map.t -> f:(t -> 'a -> 'b Memo.t) -> 'b Map.t Memo.t
   end
 end
@@ -83,8 +82,7 @@ val wrap : t -> with_:Path.t -> Unique.t
 
 include Comparable_intf.S with type key := t
 
-module Map_traversals : sig
-  val parallel_iter : 'a Map.t -> f:(t -> 'a -> unit Memo.t) -> unit Memo.t
+module Parallel_map : sig
   val parallel_map : 'a Map.t -> f:(t -> 'a -> 'b Memo.t) -> 'b Map.t Memo.t
 end
 
