@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 module Virtual = struct
   type t = { virtual_modules : Ordered_set_lang.Unexpanded.t }
@@ -34,11 +35,12 @@ let eval0 =
      they are only removed from a set (for jbuild file compatibility) *)
   let expand_and_eval t set ~parse ~key ~standard =
     let open Action_builder.O in
-    let dir = Path.build (Expander.dir t) in
-    let+ set = Expander.expand_ordered_set_lang set ~dir ~f:(Expander.expand_pform t) in
+    let+ set = Expander.expand_ordered_set_lang t set in
     let fake_modules = ref Module_name.Map.empty in
-    let parse ~loc x = parse ~loc ~fake_modules x in
-    let r = Unordered.eval_loc set ~parse ~key ~standard in
+    let r =
+      let parse ~loc x = parse ~loc ~fake_modules x in
+      Unordered.eval_loc set ~parse ~key ~standard
+    in
     r, !fake_modules
   in
   let parse ~all_modules ~loc ~fake_modules s =
@@ -50,7 +52,6 @@ let eval0 =
       Error name
   in
   fun ~expander ~loc ~all_modules ~standard osl ->
-    let open Memo.O in
     let parse = parse ~all_modules in
     let standard = Module_trie.map standard ~f:(fun m -> loc, Ok m) in
     let+ (modules, fake_modules), _ =
@@ -339,7 +340,6 @@ let eval
   }
   modules
   =
-  let open Memo.O in
   (* Fake modules are modules that do not exist but it doesn't matter because
      they are only removed from a set (for jbuild file compatibility) *)
   let eval = eval0 ~expander ~loc:stanza_loc ~all_modules in
@@ -407,7 +407,6 @@ let eval
   ~version
   (settings : Stanza_common.Modules_settings.t)
   =
-  let open Memo.O in
   Memo.push_stack_frame ~human_readable_description:(fun () ->
     Pp.textf "(modules) field at %s" (Loc.to_file_colon_line stanza_loc))
   @@ fun () ->
