@@ -245,14 +245,33 @@ let odoc_base_flags quiet build_dir =
   | Nonfatal -> S []
 ;;
 
+let odoc_dev_tool_lock_dir_exists () =
+  let path = Dune_pkg.Lock_dir.dev_tool_lock_dir_path Odoc in
+  Fs_memo.dir_exists (Path.Outside_build_dir.In_source_dir path)
+;;
+
+let odoc_dev_tool_exe_path_building_if_necessary () =
+  let open Action_builder.O in
+  let path = Path.build (Pkg_dev_tool.exe_path Odoc) in
+  let+ () = Action_builder.path path in
+  Ok path
+;;
+
 let odoc_program sctx dir =
-  Super_context.resolve_program
-    sctx
-    ~dir
-    ~where:Original_path
-    "odoc"
-    ~loc:None
-    ~hint:"opam install odoc"
+  let open Action_builder.O in
+  let* odoc_dev_tool_lock_dir_exists =
+    Action_builder.of_memo (odoc_dev_tool_lock_dir_exists ())
+  in
+  match odoc_dev_tool_lock_dir_exists with
+  | true -> odoc_dev_tool_exe_path_building_if_necessary ()
+  | false ->
+    Super_context.resolve_program
+      sctx
+      ~dir
+      ~where:Original_path
+      "odoc"
+      ~loc:None
+      ~hint:"opam install odoc"
 ;;
 
 let run_odoc sctx ~dir command ~quiet ~flags_for args =
@@ -294,7 +313,6 @@ let compile_module
   ~mode
   =
   let odoc_file = Obj_dir.Module.odoc obj_dir m in
-  let open Memo.O in
   let+ () =
     let action_with_targets =
       let doc_dir = Path.build (Obj_dir.odoc_dir obj_dir) in
@@ -333,7 +351,6 @@ let compile_module
 ;;
 
 let compile_mld sctx (m : Mld.t) ~includes ~doc_dir ~pkg =
-  let open Memo.O in
   let odoc_file = Mld.odoc_file m ~doc_dir in
   let odoc_input = Mld.odoc_input m in
   let run_odoc =
@@ -580,15 +597,14 @@ let setup_toplevel_index_rules sctx =
 ;;
 
 let libs_of_pkg ctx ~pkg =
-  let+ entries = Scope.DB.lib_entries_of_package ctx pkg in
+  let+ { Scope.DB.Lib_entry.Set.libraries; _ } =
+    Scope.DB.lib_entries_of_package ctx pkg
+  in
   (* Filter out all implementations of virtual libraries *)
-  List.filter_map entries ~f:(fun (entry : Scope.DB.Lib_entry.t) ->
-    match entry with
-    | Deprecated_library_name _ -> None
-    | Library lib ->
-      (match Lib.Local.to_lib lib |> Lib.info |> Lib_info.implements with
-       | None -> Some lib
-       | Some _ -> None))
+  List.filter_map libraries ~f:(fun lib ->
+    match Lib.Local.to_lib lib |> Lib.info |> Lib_info.implements with
+    | None -> Some lib
+    | Some _ -> None)
 ;;
 
 let entry_modules_by_lib sctx lib =
