@@ -14,9 +14,8 @@ end
 
 type t
 
-val path : t -> Path.Source.t
+val empty : t
 val dirs : t -> (Filename.t * File.t) list
 val files : t -> Filename.Set.t
 val to_dyn : t -> Dyn.t
-val empty : Path.Source.t -> t
 val of_source_path : Path.Source.t -> (t, Unix_error.Detailed.t) result Memo.t
