@@ -10,3 +10,7 @@ let roots : string option Install.Roots.t =
   ; sbin = None
   ; libexec_root = None
   }
+
+let toolchains = `Enabled
+let pkg_build_progress = `Disabled
+let lock_dev_tool = `Disabled
