@@ -1,11 +1,5 @@
 open Import
 
-module Lint : sig
-  type t = Preprocess.Without_instrumentation.t Preprocess.Per_module.t
-
-  val no_lint : t
-end
-
 type for_ =
   | Executable
   | Library of Wrapped.t option
@@ -22,7 +16,7 @@ type t =
   ; preprocessor_deps : Dep_conf.t list
   ; lint : Lint.t
   ; flags : Ocaml_flags.Spec.t
-  ; js_of_ocaml : Js_of_ocaml.In_buildable.t
+  ; js_of_ocaml : Js_of_ocaml.In_buildable.t Js_of_ocaml.Mode.Pair.t
   ; allow_overlapping_dependencies : bool
   ; ctypes : Ctypes_field.t option
   }
