@@ -403,6 +403,11 @@ let main_module_name t : Lib_info.Main_module_name.t =
     This (Some (Module_name.of_local_lib_name t.name))
 ;;
 
+let to_lib_id ~src_dir t =
+  let loc, _ = t.name in
+  Lib_id.Local.make ~loc ~src_dir (Lib_name.of_local t.name)
+;;
+
 let to_lib_info
   conf
   ~expander
@@ -421,7 +426,13 @@ let to_lib_info
     Mode.Dict.of_func (fun ~mode -> archive_for_mode ~f_ext ~mode |> Option.to_list)
   in
   let jsoo_runtime =
-    List.map conf.buildable.js_of_ocaml.javascript_files ~f:(Path.Build.relative dir)
+    let in_buildable = conf.buildable.js_of_ocaml.js in
+    List.map in_buildable.javascript_files ~f:(Path.Build.relative dir)
+  in
+  let wasmoo_runtime =
+    let in_buildable = conf.buildable.js_of_ocaml.wasm in
+    List.map in_buildable.javascript_files ~f:(Path.Build.relative dir)
+    @ List.map in_buildable.wasm_files ~f:(Path.Build.relative dir)
   in
   let status =
     match conf.visibility with
@@ -476,6 +487,10 @@ let to_lib_info
   in
   let main_module_name = main_module_name conf in
   let name = best_name conf in
+  let lib_id =
+    let src_dir = Path.drop_optional_build_context_src_exn (Path.build dir) in
+    Lib_id.Local (to_lib_id ~src_dir conf)
+  in
   let enabled =
     let+ enabled_if_result =
       let* expander = expander in
@@ -537,6 +552,7 @@ let to_lib_info
     ~loc
     ~path_kind:Local
     ~name
+    ~lib_id
     ~kind
     ~status
     ~src_dir
@@ -556,6 +572,7 @@ let to_lib_info
     ~native_archives
     ~foreign_dll_files
     ~jsoo_runtime
+    ~wasmoo_runtime
     ~preprocess
     ~enabled
     ~virtual_deps
