@@ -7,7 +7,6 @@ type t =
   ; package : Package.t option
   ; deps : Dep_conf.t Bindings.t
   ; enabled_if : Blang.t
-  ; build_if : Blang.t
   ; action : Dune_lang.Action.t option
   }
 
@@ -65,14 +64,13 @@ let gen_parse names =
             ; embed_in_plugin_libraries = []
             ; forbidden_libraries
             ; bootstrap_info = None
-            ; enabled_if
+            ; enabled_if = build_if
             ; dune_version
             }
         ; locks
         ; package
         ; deps
         ; enabled_if
-        ; build_if
         ; action
         }))
 ;;
