@@ -15,8 +15,8 @@ let impl_cm_kind t = t.impl_cm_kind
 
 let impl_modules t m =
   match t with
-  | None -> m
-  | Some t -> Modules.impl ~vlib:t.vlib_modules m
+  | None -> Modules.With_vlib.modules m
+  | Some t -> Modules.With_vlib.impl ~vlib:t.vlib_modules m
 ;;
 
 let make ~vlib ~impl ~vlib_modules ~vlib_foreign_objects =
