@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 let setup_copy_rules_for_impl ~sctx ~dir vimpl =
   let ctx = Super_context.context sctx in
@@ -10,7 +11,6 @@ let setup_copy_rules_for_impl ~sctx ~dir vimpl =
   let copy_to_obj_dir ~src ~dst =
     add_rule ~loc:(Loc.of_pos __POS__) (Action_builder.symlink ~src ~dst)
   in
-  let open Memo.O in
   let* { Lib_config.has_native; ext_obj; _ } =
     let+ ocaml = Context.ocaml ctx in
     ocaml.lib_config
@@ -56,12 +56,10 @@ let setup_copy_rules_for_impl ~sctx ~dir vimpl =
         copy_to_obj_dir ~src:(object_file vlib_obj_dir) ~dst:(object_file impl_obj_dir)))
   in
   let vlib_modules = Vimpl.vlib_modules vimpl in
-  Modules.fold_no_vlib vlib_modules ~init:(Memo.return ()) ~f:(fun m acc ->
-    acc >>> copy_objs m)
+  Modules.fold vlib_modules ~init:(Memo.return ()) ~f:(fun m acc -> acc >>> copy_objs m)
 ;;
 
 let impl sctx ~(lib : Library.t) ~scope =
-  let open Memo.O in
   match lib.implements with
   | None -> Memo.return None
   | Some (loc, implements) ->
@@ -77,8 +75,7 @@ let impl sctx ~(lib : Library.t) ~scope =
      | Some vlib ->
        let info = Lib.info vlib in
        let virtual_ =
-         let virtual_ = Lib_info.virtual_ info in
-         match virtual_ with
+         match Lib_info.virtual_ info with
          | Some v -> v
          | None ->
            User_error.raise
@@ -103,29 +100,31 @@ let impl sctx ~(lib : Library.t) ~scope =
            in
            let* ocaml = Context.ocaml (Super_context.context sctx) in
            let* modules =
+             let db = Scope.libs scope in
              let* preprocess =
                (* TODO wrong, this should be delayed *)
-               Resolve.Memo.read_memo
-                 (Preprocess.Per_module.with_instrumentation
-                    lib.buildable.preprocess
-                    ~instrumentation_backend:
-                      (Lib.DB.instrumentation_backend (Scope.libs scope)))
-             in
-             let pp_spec =
-               Staged.unstage (Preprocessing.pped_modules_map preprocess ocaml.version)
+               Preprocess.Per_module.with_instrumentation
+                 lib.buildable.preprocess
+                 ~instrumentation_backend:(Lib.DB.instrumentation_backend db)
+               |> Resolve.Memo.read_memo
              in
              Dir_contents.ocaml dir_contents
-             >>| Ml_sources.modules ~for_:(Library name)
-             >>= Modules.map_user_written ~f:(fun m -> Memo.return (pp_spec m))
+             >>= Ml_sources.modules
+                   ~libs:db
+                   ~for_:(Library (Lib_info.lib_id info |> Lib_id.to_local_exn))
+             >>=
+             let pp_spec =
+               Staged.unstage (Pp_spec.pped_modules_map preprocess ocaml.version)
+             in
+             Modules.map_user_written ~f:(fun m -> Memo.return (pp_spec m))
            in
            let+ foreign_objects =
-             let ext_obj = ocaml.lib_config.ext_obj in
-             let dir = Obj_dir.obj_dir (Lib.Local.obj_dir vlib) in
-             let+ foreign_sources = Dir_contents.foreign_sources dir_contents in
-             foreign_sources
-             |> Foreign_sources.for_lib ~name
-             |> Foreign.Sources.object_files ~ext_obj ~dir
-             |> List.map ~f:Path.build
+             Dir_contents.foreign_sources dir_contents
+             >>| Foreign_sources.for_lib ~name
+             >>| (let ext_obj = ocaml.lib_config.ext_obj in
+                  let dir = Obj_dir.obj_dir (Lib.Local.obj_dir vlib) in
+                  Foreign.Sources.object_files ~ext_obj ~dir)
+             >>| List.map ~f:Path.build
            in
            modules, foreign_objects
        in
