@@ -6,8 +6,4 @@ val setup_copy_rules_for_impl
   -> Vimpl.t
   -> unit Memo.t
 
-val impl
-  :  Super_context.t
-  -> lib:Dune_file.Library.t
-  -> scope:Scope.t
-  -> Vimpl.t option Memo.t
+val impl : Super_context.t -> lib:Library.t -> scope:Scope.t -> Vimpl.t option Memo.t
