@@ -12,7 +12,7 @@ module Lock_dir = struct
     ; unset_solver_vars : Package_variable_name.Set.t option
     ; repositories : (Loc.t * Dune_pkg.Pkg_workspace.Repository.Name.t) list
     ; constraints : Dune_lang.Package_dependency.t list
-    ; sources : string list
+    ; pins : (Loc.t * string) list
     }
 
   let to_dyn
@@ -22,7 +22,7 @@ module Lock_dir = struct
     ; unset_solver_vars
     ; repositories
     ; constraints
-    ; sources
+    ; pins
     }
     =
     Dyn.record
@@ -36,7 +36,7 @@ module Lock_dir = struct
             Dune_pkg.Pkg_workspace.Repository.Name.to_dyn
             (List.map repositories ~f:snd) )
       ; "constraints", Dyn.list Dune_lang.Package_dependency.to_dyn constraints
-      ; "sources", (Dyn.list Dyn.string) sources
+      ; "pins", (Dyn.list Dyn.string) (List.map pins ~f:snd)
       ]
   ;;
 
@@ -47,7 +47,7 @@ module Lock_dir = struct
     ; unset_solver_vars
     ; repositories
     ; constraints
-    ; sources
+    ; pins
     }
     =
     Poly.hash
@@ -57,7 +57,7 @@ module Lock_dir = struct
       , unset_solver_vars
       , repositories
       , constraints
-      , sources )
+      , pins )
   ;;
 
   let equal
@@ -67,7 +67,7 @@ module Lock_dir = struct
     ; unset_solver_vars
     ; repositories
     ; constraints
-    ; sources
+    ; pins
     }
     t
     =
@@ -83,7 +83,7 @@ module Lock_dir = struct
          repositories
          t.repositories
     && List.equal Dune_lang.Package_dependency.equal constraints t.constraints
-    && List.equal String.equal sources t.sources
+    && List.equal (Tuple.T2.equal Loc.equal String.equal) pins t.pins
   ;;
 
   let decode ~dir =
@@ -108,7 +108,7 @@ module Lock_dir = struct
       and+ repositories = Dune_lang.Ordered_set_lang.field "repositories"
       and+ constraints =
         field ~default:[] "constraints" (repeat Dune_lang.Package_dependency.decode)
-      and+ sources = field ~default:[] "sources" (repeat string) in
+      and+ pins = field ~default:[] "pins" (repeat (located string)) in
       Option.iter solver_env ~f:(fun solver_env ->
         Option.iter
           unset_solver_vars
@@ -133,7 +133,7 @@ module Lock_dir = struct
       ; version_preference
       ; repositories = repositories_of_ordered_set repositories
       ; constraints
-      ; sources
+      ; pins
       }
     in
     fields decode
@@ -266,6 +266,28 @@ module Context = struct
     ;;
   end
 
+  module Merlin = struct
+    type t =
+      | Selected
+      | Rules_only
+      | Not_selected
+
+    let equal x y =
+      match x, y with
+      | Selected, Selected | Rules_only, Rules_only | Not_selected, Not_selected -> true
+      | Selected, (Rules_only | Not_selected)
+      | (Rules_only | Not_selected), Selected
+      | Rules_only, Not_selected
+      | Not_selected, Rules_only -> false
+    ;;
+
+    let to_dyn : t -> Dyn.t = function
+      | Selected -> String "selected"
+      | Rules_only -> String "rules_only"
+      | Not_selected -> String "not_selected"
+    ;;
+  end
+
   module Common = struct
     type t =
       { loc : Loc.t
@@ -279,7 +301,7 @@ module Context = struct
       ; fdo_target_exe : Path.t option
       ; dynamically_linked_foreign_archives : bool
       ; instrument_with : Lib_name.t list
-      ; merlin : bool
+      ; merlin : Merlin.t
       }
 
     let to_dyn { name; targets; host_context; _ } =
@@ -318,7 +340,7 @@ module Context = struct
            dynamically_linked_foreign_archives
            t.dynamically_linked_foreign_archives
       && List.equal Lib_name.equal instrument_with t.instrument_with
-      && Bool.equal merlin t.merlin
+      && Merlin.equal merlin t.merlin
     ;;
 
     let fdo_suffix t =
@@ -384,7 +406,12 @@ module Context = struct
           "instrument_with"
           (Dune_lang.Syntax.since syntax (2, 7) >>> repeat Lib_name.decode)
       and+ loc = loc
-      and+ merlin = field_b "merlin" in
+      and+ merlin = field_b "merlin"
+      and+ generate_merlin_rules =
+        field_b
+          ~check:(Dune_lang.Syntax.since Stanza.syntax (3, 16))
+          "generate_merlin_rules"
+      in
       fun ~profile_default ~instrument_with_default ->
         let profile = Option.value profile ~default:profile_default in
         let instrument_with =
@@ -409,7 +436,13 @@ module Context = struct
         ; fdo_target_exe
         ; dynamically_linked_foreign_archives
         ; instrument_with
-        ; merlin
+        ; merlin =
+            (match merlin with
+             | true -> Selected
+             | false ->
+               (match generate_merlin_rules with
+                | true -> Rules_only
+                | false -> Not_selected))
         }
     ;;
   end
@@ -429,6 +462,14 @@ module Context = struct
       Common.equal base t.base && Opam_switch.equal switch t.switch
     ;;
 
+    let name_hint_opt name =
+      if String.is_prefix ~prefix:"/" name
+      then (
+        let context_name = Filename.basename name in
+        Some [ Pp.textf "(name %s) would be a valid context name" context_name ])
+      else None
+    ;;
+
     let decode =
       let+ loc_switch, switch = field "switch" (located string)
       and+ name = field_o "name" Context_name.decode
@@ -446,9 +487,10 @@ module Context = struct
              | None ->
                User_error.raise
                  ~loc:loc_switch
-                 [ Pp.textf "Generated context name %S is invalid" name
-                 ; Pp.text
-                     "Please specify a context name manually with the (name ..) field"
+                 ?hints:(name_hint_opt name)
+                 [ Pp.textf
+                     "The name generated from this switch can not be used as a context \
+                      name. Please use (name) to disambiguate."
                  ])
         in
         let base = { base with targets = Target.add base.targets x; name } in
@@ -571,7 +613,7 @@ module Context = struct
           ; fdo_target_exe = None
           ; dynamically_linked_foreign_archives = true
           ; instrument_with = Option.value instrument_with ~default:[]
-          ; merlin = false
+          ; merlin = Not_selected
           }
       }
   ;;
@@ -596,7 +638,7 @@ type t =
   ; repos : Dune_pkg.Pkg_workspace.Repository.t list
   ; lock_dirs : Lock_dir.t list
   ; dir : Path.Source.t
-  ; sources : Dune_pkg.Pin_stanza.DB.t
+  ; sources : Dune_pkg.Pin_stanza.DB.Workspace.t
   }
 
 let to_dyn { merlin_context; contexts; env; config; repos; lock_dirs; sources; dir } =
@@ -609,7 +651,7 @@ let to_dyn { merlin_context; contexts; env; config; repos; lock_dirs; sources; d
     ; "repos", list Repository.to_dyn repos
     ; "solver", (list Lock_dir.to_dyn) lock_dirs
     ; "dir", Path.Source.to_dyn dir
-    ; "sources", Dune_pkg.Pin_stanza.DB.to_dyn sources
+    ; "sources", Dune_pkg.Pin_stanza.DB.Workspace.to_dyn sources
     ]
 ;;
 
@@ -621,7 +663,7 @@ let equal { merlin_context; contexts; env; config; repos; lock_dirs; dir; source
   && List.equal Repository.equal repos w.repos
   && List.equal Lock_dir.equal lock_dirs w.lock_dirs
   && Path.Source.equal dir w.dir
-  && Dune_pkg.Pin_stanza.DB.equal sources w.sources
+  && Dune_pkg.Pin_stanza.DB.Workspace.equal sources w.sources
 ;;
 
 let hash { merlin_context; contexts; env; config; repos; lock_dirs; dir; sources } =
@@ -633,13 +675,15 @@ let hash { merlin_context; contexts; env; config; repos; lock_dirs; dir; sources
     , List.hash Repository.hash repos
     , List.hash Lock_dir.hash lock_dirs
     , Path.Source.hash dir
-    , Dune_pkg.Pin_stanza.DB.hash sources )
+    , Dune_pkg.Pin_stanza.DB.Workspace.hash sources )
 ;;
 
 let find_lock_dir t path =
   List.find t.lock_dirs ~f:(fun lock_dir -> Path.Source.equal lock_dir.path path)
 ;;
 
+let add_repo t repo = { t with repos = repo :: t.repos }
+
 include Dune_lang.Versioned_file.Make (struct
     type t = unit
   end)
@@ -801,7 +845,7 @@ let step1 clflags =
          ~default:(lazy []))
   and+ config_from_workspace_file = Dune_config.decode_fields_of_workspace_file
   and+ lock_dirs = multi_field "lock_dir" (Lock_dir.decode ~dir)
-  and+ sources = Dune_pkg.Pin_stanza.DB.decode Workspace in
+  and+ sources = Dune_pkg.Pin_stanza.DB.Workspace.decode in
   let+ contexts = multi_field "context" (lazy_ Context.decode) in
   let config =
     create_final_config
@@ -840,11 +884,11 @@ let step1 clflags =
                 !defined_names
                 (Context_name.Set.of_list (Context.all_names ctx));
            match Context.base ctx, acc with
-           | { merlin = true; _ }, Some _ ->
+           | { merlin = Selected; _ }, Some _ ->
              User_error.raise
                ~loc:(Context.loc ctx)
                [ Pp.text "you can only have one context for merlin" ]
-           | { merlin = true; _ }, None -> Some name
+           | { merlin = Selected; _ }, None -> Some name
            | _ -> acc)
        in
        let contexts =
@@ -907,7 +951,7 @@ let default clflags =
   ; repos = default_repositories
   ; lock_dirs = []
   ; dir = Path.Source.root
-  ; sources = Dune_pkg.Pin_stanza.DB.empty Workspace
+  ; sources = Dune_pkg.Pin_stanza.DB.Workspace.empty
   }
 ;;
 
