@@ -250,18 +250,22 @@ module Key = struct
 end
 
 module Error_msg = struct
-  let since t ver ~what =
-    let lang_or_using = if t.name = "dune" then "lang" else "using" in
+  let fmt_error_msg t ver ~what ~file =
+    let lang_or_using name = if name = "dune" then "lang" else "using" in
     Printf.sprintf
-      "%s is only available since version %s of %s. Please update your dune-project file \
-       to have (%s %s %s)."
+      "%s is only available since version %s of %s. Please update your %s file to have \
+       (%s %s %s)."
       what
       (Version.to_string ver)
       t.desc
-      lang_or_using
+      file
+      (lang_or_using t.name)
       t.name
       (Version.to_string ver)
   ;;
+
+  let since t ver ~what = fmt_error_msg t ver ~what ~file:"dune-project"
+  let since_config t ver ~what = fmt_error_msg t ver ~what ~file:"dune config"
 end
 
 module Error = struct
@@ -520,16 +524,20 @@ let renamed_in t ver ~to_ =
     Error.renamed_in loc t ver ~what ~to_
 ;;
 
-let since ?what ?(fatal = true) t ver =
+let since_fmt ?(fatal = true) ~fmt t ver loc =
   let open Version.Infix in
   let* current_ver = get_exn t in
   if current_ver >= ver
   then return ()
-  else
-    let* loc, what_ctx = desc () in
-    let what = Option.value what ~default:what_ctx in
+  else (
     if fatal
-    then Error.since loc t ver ~what
-    else User_warning.emit ~loc [ Pp.text (Error_msg.since t ver ~what) ];
-    return ()
+    then User_error.raise ~loc [ Pp.text (fmt t ver) ]
+    else User_warning.emit ~loc [ Pp.text (fmt t ver) ];
+    return ())
+;;
+
+let since ?what ?(fatal = true) t ver =
+  let* loc, what_ctx = desc () in
+  let what = Option.value what ~default:what_ctx in
+  since_fmt ~fatal ~fmt:(Error_msg.since ~what) t ver loc
 ;;
