@@ -12,8 +12,7 @@ module Diff = struct
     ; file2 : 'target
     }
 
-  let map t ~path ~target =
-    { t with file1 = path t.file1; file2 = target t.file2 }
+  let map t ~path ~target = { t with file1 = path t.file1; file2 = target t.file2 }
 end
 
 module Outputs = struct
@@ -26,6 +25,7 @@ module Outputs = struct
     | Stdout -> "stdout"
     | Stderr -> "stderr"
     | Outputs -> "outputs"
+  ;;
 end
 
 module Inputs = struct
@@ -33,6 +33,7 @@ module Inputs = struct
 
   let to_string = function
     | Stdin -> "stdin"
+  ;;
 end
 
 module File_perm = struct
@@ -43,8 +44,10 @@ module File_perm = struct
   let suffix = function
     | Normal -> ""
     | Executable -> "-executable"
+  ;;
 
   let to_unix_perm = function
     | Normal -> 0o666
     | Executable -> 0o777
+  ;;
 end
