@@ -13,6 +13,7 @@ module Action = Action
 module Alias_name = Alias_name
 module Terminal_signals = Terminal_signals
 module Gc = Gc
+module Prog = Prog
 open Stdune
 
 let xdg =
@@ -28,3 +29,10 @@ let xdg =
      in
      Xdg.create ~env:(Env.get env_map) ())
 ;;
+
+let frames_per_second () =
+  match Dune_config.Config.(get threaded_console_frames_per_second) with
+  | `Custom fps -> fps
+  | `Default when Execution_env.inside_emacs -> 15
+  | `Default -> 60
+;;
