@@ -52,11 +52,16 @@ module Lock = struct
      | `Failure -> ()
      | `Success ->
        let fd = Flock.fd t in
+       Unix.ftruncate fd 0;
        write_pid fd);
     res
   ;;
 
-  let unlock () = Lazy.force t |> Flock.unlock |> or_raise_unix ~name:"unlock"
+  let unlock () =
+    let lock = Lazy.force t in
+    Unix.ftruncate (Flock.fd lock) 0;
+    Flock.unlock lock |> or_raise_unix ~name:"unlock"
+  ;;
 end
 
 let locked = ref false
@@ -95,11 +100,10 @@ let lock_exn ~timeout =
 ;;
 
 let unlock () =
-  match Config.(get global_lock) with
-  | `Disabled -> ()
-  | `Enabled ->
-    if !locked
-    then (
-      Lock.unlock ();
-      locked := false)
+  if !locked
+  then (
+    Lock.unlock ();
+    locked := false)
 ;;
+
+let () = at_exit unlock
