@@ -1,4 +1,3 @@
-open Stdune
 open Import
 
 exception Already_reported
@@ -63,6 +62,15 @@ let get_error_from_exn = function
   | User_error.E msg ->
     let has_embedded_location = User_message.has_embedded_location msg in
     let needs_stack_trace = User_message.needs_stack_trace msg in
+    let msg =
+      match msg.dir with
+      | None -> msg
+      | Some path ->
+        let build_context = Path.extract_build_context (Path.of_string path) in
+        (match build_context with
+         | None -> msg
+         | Some (ctxt, _) -> { msg with context = Some ctxt })
+    in
     { responsible = User; msg; has_embedded_location; needs_stack_trace }
   | Code_error.E e ->
     code_error ~loc:e.loc ~dyn_without_loc:(Code_error.to_dyn_without_loc e)
