@@ -8,7 +8,6 @@ module Kind : sig
     | Hg
 
   val of_dir_name : Filename.t -> t option
-
   val of_dir_contents : Filename.Set.t -> t option
 end
 
@@ -18,7 +17,6 @@ type t =
   }
 
 val equal : t -> t -> bool
-
 val to_dyn : t -> Dyn.t
 
 (** Nice description of the current tip *)
@@ -28,9 +26,12 @@ val describe : t -> string option Memo.t
 val commit_id : t -> string option Memo.t
 
 (** List of files committed in the repo *)
-val files : t -> Path.t list Memo.t
+val files : t -> Path.Source.t list Memo.t
 
 (** VCS commands *)
 val git : Path.t Lazy.t
 
 val hg : Path.t Lazy.t
+
+(** Valid git exit codes *)
+val git_accept : unit -> ('a, ('a, int) result) Dune_engine.Process.Failure_mode.t
