@@ -91,8 +91,7 @@ let%expect_test "csexp server life cycle" =
   let config =
     { Scheduler.Config.concurrency = 1
     ; stats = None
-    ; insignificant_changes = `React
-    ; signal_watcher = `No
+    ; print_ctrl_c_warning = false
     ; watch_exclusions = []
     }
   in
