@@ -33,7 +33,8 @@ let%expect_test _ =
   try
     add_assuming_missing node41 root;
     print_endline "no cycle"
-  with Cycle cycle ->
+  with
+  | Cycle cycle ->
     print_endline "cycle:";
     let cycle = List.map cycle ~f:name in
     List.map ~f:Pp.text cycle |> Pp.concat ~sep:Pp.space |> print;
@@ -50,11 +51,13 @@ let%expect_test _ =
       child 4 1 child 3 1 child 2 1 child 1 2
       root
     |}]
+;;
 
 let rec adjacent_pairs l =
   match l with
   | [] | [ _ ] -> []
   | x :: y :: rest -> (x, y) :: adjacent_pairs (y :: rest)
+;;
 
 let cycle_test variant =
   let open
@@ -76,8 +79,8 @@ let cycle_test variant =
      path when producing the cycle for some reason (or at least they did in
      2019-03) *)
   (match variant with
-  | `a -> add n2 n3
-  | `b -> ());
+   | `a -> add n2 n3
+   | `b -> ());
   let n4 = node 4 in
   add n3 n4;
   let n5 = node 5 in
@@ -138,10 +141,11 @@ let cycle_test variant =
   | exception Cycle c ->
     let c = List.map c ~f:value in
     List.iter (adjacent_pairs c) ~f:(fun (b, a) ->
-        match List.exists !edges ~f:(fun edge -> edge = (a, b)) with
-        | true -> ()
-        | false -> Printf.ksprintf failwith "bad edge in cycle: (%d, %d)\n" a b);
+      match List.exists !edges ~f:(fun edge -> edge = (a, b)) with
+      | true -> ()
+      | false -> Printf.ksprintf failwith "bad edge in cycle: (%d, %d)\n" a b);
     List.map c ~f:(Pp.textf "%d") |> Pp.concat ~sep:Pp.space |> print
+;;
 
 let%expect_test _ =
   cycle_test `a;
@@ -149,6 +153,7 @@ let%expect_test _ =
     23 22 21 20 14 13 12
     11
   |}]
+;;
 
 let%expect_test _ =
   cycle_test `b;
@@ -156,6 +161,7 @@ let%expect_test _ =
     23 22 21 20 14 13 12
     11
   |}]
+;;
 
 let%expect_test "creating a cycle can succeed on the second attempt" =
   let open
@@ -188,8 +194,8 @@ let%expect_test "creating a cycle can succeed on the second attempt" =
     c4 = (3: k=2) (c4) []
   |}];
   (match add_assuming_missing c4 c2 with
-  | () -> Format.printf "added :o\n"
-  | exception Cycle _ -> Format.printf "cycle\n");
+   | () -> Format.printf "added :o\n"
+   | exception Cycle _ -> Format.printf "cycle\n");
   Format.printf "c1 = %a@.\n" dag_pp_mynode c1;
   Format.printf "c2 = %a@.\n" dag_pp_mynode c2;
   Format.printf "c3 = %a@.\n" dag_pp_mynode c3;
@@ -208,8 +214,8 @@ let%expect_test "creating a cycle can succeed on the second attempt" =
     c4 = (3: k=2) (c4) []
   |}];
   (match add_assuming_missing c4 c2 with
-  | () -> Format.printf "added :o\n"
-  | exception Cycle _ -> Format.printf "cycle\n");
+   | () -> Format.printf "added :o\n"
+   | exception Cycle _ -> Format.printf "cycle\n");
   Format.printf "c1 = %a@.\n" dag_pp_mynode c1;
   (* The output is truncated at depth 20. *)
   [%expect
@@ -234,3 +240,4 @@ let%expect_test "creating a cycle can succeed on the second attempt" =
                                                                (1: k=2) (c2) [
                                                                ...]]]]]]]]]]]]]]]]]]]]
   |}]
+;;
