@@ -6,8 +6,7 @@ open Dune_async_io
 let config =
   { Scheduler.Config.concurrency = 1
   ; stats = None
-  ; insignificant_changes = `Ignore
-  ; signal_watcher = `No
+  ; print_ctrl_c_warning = false
   ; watch_exclusions = []
   }
 ;;
