@@ -11,18 +11,18 @@ module Response = Dune_rpc.Response
 let%expect_test "turn on and shutdown" =
   let test () =
     with_dune_watch (fun _pid ->
-        run_client (fun client ->
-            let+ () = dune_build client "." in
-            printfn "shutting down"))
+      run_client (fun client ->
+        let+ () = dune_build client "." in
+        printfn "shutting down"))
   in
   run test;
   [%expect {|
     Building .
     Build . succeeded
     shutting down |}]
+;;
 
-let files =
-  List.iter ~f:(fun (f, contents) -> Io.String_path.write_file f contents)
+let files = List.iter ~f:(fun (f, contents) -> Io.String_path.write_file f contents)
 
 let on_diagnostic_event diagnostics =
   let cwd = Sys.getcwd () in
@@ -37,9 +37,7 @@ let on_diagnostic_event diagnostics =
       { p with pos_fname = sanitize_path p.pos_fname }
     in
     fun (loc : Lexbuf.Loc.t) ->
-      Loc.of_lexbuf_loc loc
-      |> Loc.map_pos ~f:sanitize_position
-      |> Loc.to_lexbuf_loc
+      Loc.of_lexbuf_loc loc |> Loc.map_pos ~f:sanitize_position |> Loc.to_lexbuf_loc
   in
   (* function to remove remove pp tags and hide junk from paths *)
   let map_event (d : Diagnostic.Event.t) f : Diagnostic.Event.t =
@@ -51,15 +49,15 @@ let on_diagnostic_event diagnostics =
     let directory = Option.map d.directory ~f:sanitize_path in
     let promotion =
       List.map d.promotion ~f:(fun (p : Diagnostic.Promotion.t) ->
-          let in_build = sanitize_path p.in_build in
-          let in_source = sanitize_path p.in_source in
-          { Diagnostic.Promotion.in_build; in_source })
+        let in_build = sanitize_path p.in_build in
+        let in_source = sanitize_path p.in_source in
+        { Diagnostic.Promotion.in_build; in_source })
     in
     let related =
       List.map d.related ~f:(fun (related : Diagnostic.Related.t) ->
-          let loc = sanitize_loc related.loc in
-          let message = sanitize_pp related.message in
-          { Diagnostic.Related.message; loc })
+        let loc = sanitize_loc related.loc in
+        let message = sanitize_pp related.message in
+        { Diagnostic.Related.message; loc })
     in
     { d with
       message = sanitize_pp d.message
@@ -69,68 +67,70 @@ let on_diagnostic_event diagnostics =
     ; related
     }
   in
-  if List.is_empty diagnostics then print_endline "<no diagnostics>"
+  if List.is_empty diagnostics
+  then print_endline "<no diagnostics>"
   else
     List.iter diagnostics ~f:(fun (e : Diagnostic.Event.t) ->
-        (match e with
-        | Remove _ -> ()
-        | Add e ->
-          Diagnostic.promotion e
-          |> List.iter ~f:(fun promotion ->
-                 let path = Diagnostic.Promotion.in_build promotion in
-                 if not (Sys.file_exists path) then
-                   printfn "FAILURE: promotion file %s does not exist"
-                     (sanitize_path path)));
-        let e = map_event e sanitize in
-        printfn "%s" (Dyn.to_string (Diagnostic.Event.to_dyn e)))
+      (match e with
+       | Remove _ -> ()
+       | Add e ->
+         Diagnostic.promotion e
+         |> List.iter ~f:(fun promotion ->
+           let path = Diagnostic.Promotion.in_build promotion in
+           if not (Sys.file_exists path)
+           then printfn "FAILURE: promotion file %s does not exist" (sanitize_path path)));
+      let e = map_event e sanitize in
+      printfn "%s" (Dyn.to_string (Diagnostic.Event.to_dyn e)))
+;;
 
 let setup_diagnostics f =
   let exec _pid =
     run_client (fun client ->
-        (* First we test for regular errors *)
-        files [ ("dune-project", "(lang dune 3.0)") ];
-        f client)
+      (* First we test for regular errors *)
+      files [ "dune-project", "(lang dune 3.0)" ];
+      f client)
   in
   run (fun () -> with_dune_watch exec)
+;;
 
 let poll_exn client decl =
   let+ poll = Client.poll client decl in
   match poll with
   | Ok p -> p
   | Error e -> raise (Dune_rpc.Version_error.E e)
+;;
 
 let print_diagnostics poll =
   let+ res = Client.Stream.next poll in
   match res with
   | None -> printfn "client: no more diagnostics"
   | Some diag -> on_diagnostic_event diag
+;;
 
 let diagnostic_with_build setup target =
   let exec _pid =
     run_client (fun client ->
-        (* First we test for regular errors *)
-        files (("dune-project", "(lang dune 3.0)") :: setup);
-        let* () = dune_build client target in
-        let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
-        let* () = print_diagnostics poll in
-        Client.Stream.cancel poll)
+      (* First we test for regular errors *)
+      files (("dune-project", "(lang dune 3.0)") :: setup);
+      let* () = dune_build client target in
+      let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
+      let* () = print_diagnostics poll in
+      Client.Stream.cancel poll)
   in
   run (fun () -> with_dune_watch exec)
+;;
 
 let%expect_test "error in dune file" =
-  diagnostic_with_build [ ("dune", "(library (name foo))") ] "foo.cma";
-  [%expect
-    {|
+  diagnostic_with_build [ "dune", "(library (name foo))" ] "foo.cma";
+  [%expect {|
     Building foo.cma
     Build foo.cma succeeded
     <no diagnostics> |}]
+;;
 
 let%expect_test "related error" =
   diagnostic_with_build
-    [ ("dune", "(library (name foo))")
-    ; ("foo.mli", "val x : int")
-    ; ("foo.ml", "let x = true")
-    ]
+    [ "dune", "(library (name foo))"; "foo.mli", "val x : int"; "foo.ml", "let x = true" ]
     "foo.cma";
   [%expect
     {|
@@ -214,7 +214,7 @@ let%expect_test "related error" =
     ]
   ] |}];
   diagnostic_with_build
-    [ ("dune", "(library (name foo)) (executable (name foo))"); ("foo.ml", "") ]
+    [ "dune", "(library (name foo)) (executable (name foo))"; "foo.ml", "" ]
     "@check";
   [%expect
     {|
@@ -240,8 +240,7 @@ let%expect_test "related error" =
           ]
         ]
       ; [ "message"
-        ; [ "Verbatim"; "Module \"Foo\" is used in several\n\
-                         stanzas:\n\
+        ; [ "Verbatim"; "Module \"Foo\" is used in several stanzas:\n\
                          " ]
         ]
       ; [ "promotion"; [] ]
@@ -273,6 +272,7 @@ let%expect_test "related error" =
       ; [ "targets"; [] ]
       ]
     ] |}]
+;;
 
 let%expect_test "promotion" =
   diagnostic_with_build
@@ -282,7 +282,7 @@ let%expect_test "promotion" =
 (rule (with-stdout-to x.gen (echo "toto")))
 |}
       )
-    ; ("x", "titi")
+    ; "x", "titi"
     ]
     "(alias foo)";
   [%expect
@@ -310,8 +310,7 @@ let%expect_test "promotion" =
         ]
       ; [ "message"
         ; [ "Verbatim"
-          ; "Error: Files _build/default/x and _build/default/x.gen\n\
-             differ.\n\
+          ; "Error: Files _build/default/x and _build/default/x.gen differ.\n\
              "
           ]
         ]
@@ -326,6 +325,7 @@ let%expect_test "promotion" =
       ; [ "targets"; [] ]
       ]
     ] |}]
+;;
 
 let%expect_test "optional promotion" =
   diagnostic_with_build
@@ -384,11 +384,12 @@ let%expect_test "optional promotion" =
       ; [ "targets"; [] ]
       ]
     ] |}]
+;;
 
 let%expect_test "warning detection" =
   diagnostic_with_build
-    [ ("dune", "(executable (flags -w +26) (name foo))")
-    ; ("foo.ml", "let () = let x = 10 in ()")
+    [ "dune", "(executable (flags -w +26) (name foo))"
+    ; "foo.ml", "let () = let x = 10 in ()"
     ]
     "./foo.exe";
   [%expect
@@ -396,10 +397,11 @@ let%expect_test "warning detection" =
     Building ./foo.exe
     Build ./foo.exe succeeded
     <no diagnostics> |}]
+;;
 
 let%expect_test "error from user rule" =
   diagnostic_with_build
-    [ ("dune", "(rule (target foo) (action (bash \"echo foobar\")))") ]
+    [ "dune", "(rule (target foo) (action (bash \"echo foobar\")))" ]
     "./foo";
   [%expect
     {|
@@ -426,8 +428,8 @@ let%expect_test "error from user rule" =
         ]
       ; [ "message"
         ; [ "Verbatim"
-          ; "Error: Rule failed to generate the following\n\
-             targets:- foo\n\
+          ; "Error: Rule failed to generate the following targets:\n\
+             - foo\n\
              "
           ]
         ]
@@ -437,12 +439,11 @@ let%expect_test "error from user rule" =
       ; [ "targets"; [] ]
       ]
     ] |}]
+;;
 
 let%expect_test "library error location" =
   diagnostic_with_build
-    [ ("dune", "(library (name foo) (libraries fake-library))")
-    ; ("foo.ml", "")
-    ]
+    [ "dune", "(library (name foo) (libraries fake-library))"; "foo.ml", "" ]
     "./foo.cma";
   [%expect
     {|
@@ -468,8 +469,7 @@ let%expect_test "library error location" =
           ]
         ]
       ; [ "message"
-        ; [ "Verbatim"; "Error: Library \"fake-library\" not\n\
-                         found.\n\
+        ; [ "Verbatim"; "Error: Library \"fake-library\" not found.\n\
                          " ]
         ]
       ; [ "promotion"; [] ]
@@ -478,24 +478,22 @@ let%expect_test "library error location" =
       ; [ "targets"; [] ]
       ]
     ] |}]
+;;
 
 let%expect_test "create and fix error" =
   setup_diagnostics (fun client ->
-      files
-        [ ("dune", "(executable (name foo))")
-        ; ("foo.ml", "let () = print_endline 123")
-        ];
-      let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
-      let* () = print_diagnostics poll in
-      [%expect {|
+    files [ "dune", "(executable (name foo))"; "foo.ml", "let () = print_endline 123" ];
+    let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
+    let* () = print_diagnostics poll in
+    [%expect {|
         <no diagnostics> |}];
-      let* () = dune_build client "./foo.exe" in
-      [%expect {|
+    let* () = dune_build client "./foo.exe" in
+    [%expect {|
         Building ./foo.exe
         Build ./foo.exe failed |}];
-      let* () = print_diagnostics poll in
-      [%expect
-        {|
+    let* () = print_diagnostics poll in
+    [%expect
+      {|
         [ "Add"
         ; [ [ "directory"; "$CWD" ]
           ; [ "id"; "0" ]
@@ -529,15 +527,14 @@ let%expect_test "create and fix error" =
           ; [ "targets"; [] ]
           ]
         ] |}];
-      files [ ("foo.ml", "let () = print_endline \"foo\"") ];
-      let* () = dune_build client "./foo.exe" in
-      [%expect
-        {|
+    files [ "foo.ml", "let () = print_endline \"foo\"" ];
+    let* () = dune_build client "./foo.exe" in
+    [%expect {|
         Building ./foo.exe
         Build ./foo.exe succeeded |}];
-      let+ () = print_diagnostics poll in
-      [%expect
-        {|
+    let+ () = print_diagnostics poll in
+    [%expect
+      {|
         [ "Remove"
         ; [ [ "directory"; "$CWD" ]
           ; [ "id"; "0" ]
@@ -572,34 +569,36 @@ let%expect_test "create and fix error" =
           ]
         ] |}]);
   [%expect {| |}]
+;;
 
 let request_exn client req n =
   let* staged = Client.Versioned.prepare_request client req in
   match staged with
   | Ok req -> Client.request client req n
   | Error e -> raise (Dune_rpc.Version_error.E e)
+;;
 
 let%expect_test "formatting dune files" =
   let exec _pid =
     run_client (fun client ->
-        (* First we test for regular errors *)
-        files [ ("dune-project", "(lang dune 3.0)") ];
-        let unformatted = "(\nlibrary (name foo\n))" in
-        printfn "Unformatted:\n%s" unformatted;
-        let run uri what =
-          let+ res =
-            request_exn client Request.format_dune_file
-              (uri, `Contents unformatted)
-          in
-          match res with
-          | Ok s -> printfn "Formatted (%s):\n%s" what s
-          | Error e ->
-            Format.eprintf "Error formatting:@.%s@."
-              (Dyn.to_string (Response.Error.to_dyn e))
+      (* First we test for regular errors *)
+      files [ "dune-project", "(lang dune 3.0)" ];
+      let unformatted = "(\nlibrary (name foo\n))" in
+      printfn "Unformatted:\n%s" unformatted;
+      let run uri what =
+        let+ res =
+          request_exn client Request.format_dune_file (uri, `Contents unformatted)
         in
-        let* () = run Dune_rpc.Path.(relative dune_root "dune") "relative" in
-        [%expect
-          {|
+        match res with
+        | Ok s -> printfn "Formatted (%s):\n%s" what s
+        | Error e ->
+          Format.eprintf
+            "Error formatting:@.%s@."
+            (Dyn.to_string (Response.Error.to_dyn e))
+      in
+      let* () = run Dune_rpc.Path.(relative dune_root "dune") "relative" in
+      [%expect
+        {|
           Unformatted:
           (
           library (name foo
@@ -607,62 +606,63 @@ let%expect_test "formatting dune files" =
           Formatted (relative):
           (library
            (name foo)) |}];
-        let+ () =
-          run
-            (Dune_rpc.Path.absolute (Filename.concat (Sys.getcwd ()) "dune"))
-            "absolute"
-        in
-        [%expect
-          {|
+      let+ () =
+        run (Dune_rpc.Path.absolute (Filename.concat (Sys.getcwd ()) "dune")) "absolute"
+      in
+      [%expect
+        {|
           Formatted (absolute):
           (library
            (name foo)) |}])
   in
   run (fun () -> with_dune_watch exec);
   [%expect {| |}]
+;;
 
 let%expect_test "promoting dune files" =
   let exec _pid =
     run_client (fun client ->
-        (* First we test for regular errors *)
-        let fname = "x" in
-        let promoted = "x.gen" in
-        files
-          [ ("dune-project", "(lang dune 3.0)")
-          ; ("x", "titi")
-          ; ( "dune"
-            , sprintf
-                {|
+      (* First we test for regular errors *)
+      let fname = "x" in
+      let promoted = "x.gen" in
+      files
+        [ "dune-project", "(lang dune 3.0)"
+        ; "x", "titi"
+        ; ( "dune"
+          , sprintf
+              {|
 (rule (alias foo) (action (diff %s %s)))
 (rule (with-stdout-to %s (echo "toto")))
 |}
-                fname promoted promoted )
-          ];
-        let* () = dune_build client "(alias foo)" in
-        [%expect
-          {|
+              fname
+              promoted
+              promoted )
+        ];
+      let* () = dune_build client "(alias foo)" in
+      [%expect {|
           Building (alias foo)
           Build (alias foo) failed |}];
-        print_endline "attempting to promote";
-        let+ res =
-          request_exn client Request.promote
-            Dune_rpc.Path.(relative dune_root fname)
-        in
-        (match res with
-        | Ok () ->
-          let contents = Io.String_path.read_file fname in
-          printfn "promoted file contents:\n%s" contents
-        | Error e ->
-          Format.eprintf "Error formatting:@.%s@."
-            (Dyn.to_string (Dune_rpc.Response.Error.to_dyn e)));
-        [%expect
-          {|
+      print_endline "attempting to promote";
+      let+ res =
+        request_exn client Request.promote Dune_rpc.Path.(relative dune_root fname)
+      in
+      (match res with
+       | Ok () ->
+         let contents = Io.String_path.read_file fname in
+         printfn "promoted file contents:\n%s" contents
+       | Error e ->
+         Format.eprintf
+           "Error formatting:@.%s@."
+           (Dyn.to_string (Dune_rpc.Response.Error.to_dyn e)));
+      [%expect
+        {|
           attempting to promote
           promoted file contents:
           toto |}])
   in
   run (fun () -> with_dune_watch exec);
   [%expect {| |}]
+;;
 
 let%expect_test "multiple errors in one file" =
   let source =
@@ -681,18 +681,18 @@ let g = A.f
 |}
   in
   setup_diagnostics (fun client ->
-      files [ ("dune", "(executable (name foo))"); ("foo.ml", source) ];
-      let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
-      let* () = print_diagnostics poll in
-      [%expect {|
+    files [ "dune", "(executable (name foo))"; "foo.ml", source ];
+    let* poll = poll_exn client Dune_rpc.Public.Sub.diagnostic in
+    let* () = print_diagnostics poll in
+    [%expect {|
         <no diagnostics> |}];
-      let* () = dune_build client "./foo.exe" in
-      [%expect {|
+    let* () = dune_build client "./foo.exe" in
+    [%expect {|
         Building ./foo.exe
         Build ./foo.exe failed |}];
-      let+ () = print_diagnostics poll in
-      [%expect
-        {|
+    let+ () = print_diagnostics poll in
+    [%expect
+      {|
         [ "Add"
         ; [ [ "directory"; "$CWD" ]
           ; [ "id"; "0" ]
@@ -806,3 +806,4 @@ let g = A.f
           ]
         ] |}]);
   [%expect {||}]
+;;
