@@ -9,15 +9,19 @@ let default =
   Clflags.display := Short;
   { Scheduler.Config.concurrency = 1
   ; stats = None
-  ; insignificant_changes = `React
-  ; signal_watcher = `No
+  ; print_ctrl_c_warning = false
   ; watch_exclusions = []
   }
 ;;
 
-let go ?(timeout = 0.3) ?(config = default) f =
+let go ?(timeout_seconds = 0.3) ?(config = default) f =
   try
-    Scheduler.Run.go ~timeout config ~file_watcher:No_watcher ~on_event:(fun _ _ -> ()) f
+    Scheduler.Run.go
+      ~timeout_seconds
+      config
+      ~file_watcher:No_watcher
+      ~on_event:(fun _ _ -> ())
+      f
   with
   | Scheduler.Run.Shutdown.E Requested -> ()
 ;;
@@ -71,10 +75,7 @@ let%expect_test "cancelling a build: effect on other fibers" =
           Scheduler.inject_memo_invalidation (Memo.Cell.invalidate cell ~reason:Unknown)
         in
         let* () = Scheduler.wait_for_build_input_change () in
-        let* res =
-          Fiber.collect_errors (fun () ->
-            Scheduler.with_job_slot (fun _ _ -> Fiber.return ()))
-        in
+        let* res = Fiber.collect_errors (fun () -> Fiber.return ()) in
         print_endline
           (match res with
            | Ok () -> "PASS: we can still run things outside the build"
@@ -83,33 +84,6 @@ let%expect_test "cancelling a build: effect on other fibers" =
   [%expect {| PASS: we can still run things outside the build |}]
 ;;
 
-let%expect_test "empty invalidation wakes up waiter" =
-  let test insignificant_changes =
-    go ~timeout:0.1 ~config:{ default with insignificant_changes }
-    @@ fun () ->
-    let await_invalidation () =
-      print_endline "awaiting invalidation";
-      let+ () = Scheduler.wait_for_build_input_change () in
-      print_endline "awaited invalidation"
-    in
-    Fiber.fork_and_join_unit
-      (fun () -> Scheduler.inject_memo_invalidation Memo.Invalidation.empty)
-      await_invalidation
-  in
-  test `React;
-  [%expect {|
-    awaiting invalidation
-    awaited invalidation |}];
-  test `Ignore;
-  [%expect.unreachable]
-[@@expect.uncaught_exn
-  {|
-  ("shutdown: timeout")
-  Trailing output
-  ---------------
-  awaiting invalidation |}]
-;;
-
 let%expect_test "raise inside Scheduler.Run.go" =
   (try
      (go
